/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


_PRIVATE_

#define old_opmemo 237
/** 
    The cell operator becomes a ternary operator.
*/
void
graphicalTrees::upgrade0To1 (ntrees aTree)
{
  int i, nb, ope;
  ntrees p_new, pfather, l_ports, p_port, p_attr;

  if (aTree == null) return;
  ope = noper (aTree);
  if (ope == old_opmemo)
    {
      p_new = LL_make_memo_tree (nchild (1, aTree), nchild (2, aTree), STree_mkEmptyTree ());
      pfather = nfather (aTree);
      i = nposit (aTree);
      nchangchild (pfather, p_new, i);
      aTree = p_new;
    }

  if (!(LL_is_atomOp (ope)))
    {
      nb = ntreearity (aTree);
      for (i = 1; i <= nb; i++)
	upgrade0To1 (nchild (i, aTree));
    }
  /* On s'occupe des attributs Signal accroches aux ports */
  p_attr = nattrntrees (aTree, GRAPHI_ATTRNUMBER);
  if (p_attr != null)
    if (noper (p_attr) == opschema)
      {
	l_ports = nchild (2, p_attr);
	if (l_ports != null)
	  {
	    p_port = nchild (1, l_ports);
	    while (p_port != null)
	      {
		p_attr = nattrntrees (p_port, SIGNAL_ATTRNUMBER);
		if (p_attr != null)
		  upgrade0To1 (p_attr);
		p_port = ntrees_getNext (p_port);
	      }
	  }
      }
}


/**  
     Changing opmodu operator...(?)
*/
void
graphicalTrees::upgrade1To2 (ntrees aTree)
{
  int i, nb, ope;
  ntrees p_new, pBox, pSon2;
  typcoord tcoord[2];

  if (IS_NILTREE (aTree)) return;
  ope = noper (aTree);
  pSon2 = nchild (2, aTree);
  if (ope == opmodu)
    {
      if (noper (pSon2) != opdescmodu)
	{
	  p_new = LL_make_descmodu_tree (STree_mkEmptyTree (), pSon2);
	  nchangchild (aTree, p_new, 2);
	}
      pBox = nattrntrees (aTree, GRAPHI_ATTRNUMBER);
      if (IS_NILTREE (pBox))	// if there is no box, create it!
	{
	  setDefaultCoord (tcoord);
	  createbte (aTree, tcoord, 2);
	}
    }
  else if (ope == oplmodu)
    {
      nb = ntreearity (aTree);
      for (i = 1; i <= nb; i++)
	upgrade1To2 (nchild (i, aTree));
    }
}


#define old_oplsync 422
/** 2001 September
    Removing oplsync operator, replaced by oplequhor
*/
void
graphicalTrees::upgrade2To3 (ntrees aTree)
{
  // Added by L.Besnard
  int i, nb;
  noperators ope;

  if (IS_NILTREE (aTree)) return;
  ope = noper (aTree);
  if (ope == old_oplsync) nsetop (aTree, oplequhor);
  if (!(LL_is_atomOp (ope)))
    {
      nb = ntreearity (aTree);
      for (i = 1; i <= nb; i++)
	upgrade2To3 (nchild (i, aTree));
    }
}


/** 
    2015 March.
    An identifier is represented by a tree (opident, suffixes()), _ as suffix separator.
 */
void
graphicalTrees::upgrade3To4(ntrees aTree)
{
  int vpos, i, nb;
  noperators ope;
  ntrees pfather, nTree, l_ports, p_port, p_attr;
  ptrepstr vstr;

  if (IS_NILTREE (aTree)) return;
  ope = noper (aTree);
  if (ope == opnom)
    {
      printf("DEBUG  upgrade3To4 : an identifier ---\n"); fflush(stdout);
      nTree = repstr2ident(nstringof (aTree));
      pfather = nfather (aTree);
      vpos = nposit (aTree);
      nchangchild (pfather, nTree, vpos);
    }
  else
    {
      nb = ntreearity (aTree);
      for (i = 1; i <= nb; i++)
	upgrade3To4 (nchild (i, aTree));
    }

  p_attr = nattrntrees (aTree, GRAPHI_ATTRNUMBER);
  if (p_attr != null)
    if (noper (p_attr) == opschema)
      {
	l_ports = nchild (2, p_attr);
	if (l_ports != null)
	  {
	    p_port = nchild (1, l_ports);
	    while (p_port != null)
	      {
		p_attr = nattrntrees (p_port, SIGNAL_ATTRNUMBER);
		if (p_attr != null)
		  {
		    printf("DEBUG  upgrade3To4 : beg a port <<< \n"); fflush(stdout);
		    upgrade3To4 (p_attr);
		    printf("DEBUG  upgrade3To4 : end a port >>>\n"); fflush(stdout);
		  }
		p_port = ntrees_getNext (p_port);
	      }
	  }
      }
}



///
long int
graphicalTrees::TreeGetCurrentVersion (ntrees vtree)
{
  if (!nattrexist (vtree, VERSION_ATTRNUMBER)) return 0;
  return nattrint (vtree, VERSION_ATTRNUMBER);
}


///
void
graphicalTrees::TreeSetCurrentVersion (ntrees vtree)
{
  if (ISNOT_NILTREE (vtree)) 
    nmakeattrint (vtree, VERSION_ATTRNUMBER, CONST_NUM_RELEASE);
}
