/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

/**
  Creation of the root (a process model) of the Signal program.
  This method is called once by Java.
  Return the new process.
*/
ntrees
builder::SignalRootCreate (char *c_name)
{
  ntrees nom, rac;
  int noProcBox;

  nom = LL_make_nom_atom (string2repstr (c_name));
  rac = STree_mkProcess (nom, STree_mkSkeletonInterface (), STree_mkEmptyBody ());
  noProcBox = the_graphicalTrees->createbte_proc (rac);
  pCurObjManager->set_arb_complet (rac);
  pCurObjManager->set_bte_cou_engl (the_graphicTablesManager->graphObj_get_tree (noProcBox));
  the_graphicalTrees->setRoot (rac);
  return (rac);
}

/**
  Associate the Signal tree pSig (in) to the box obj (in). This method is called
  by CONFIRMING, the box obj is the current box (the user has gone down it).
  After the insertion of the Signal tree, if ok, we go up in the tree.
  ClickNameNecessary returns true to offer the possibility to the user to name 
  its ports with the signal names of the expression.
  Returns true if all is ok, otherwise returns false (insertion of the tree 
  incorrect for example)
*/
boolean
builder::setTextToBox (ntrees pSig, int obj, boolean * ClickNameNecessary)
{
  ntrees l_ent, l_sort, arb_gra, arb_schema, p_bte, pCurSig;
  int ope;
  boolean ok;

  ok = pk_false;
  *ClickNameNecessary = pk_false;
  arb_gra = the_graphicTablesManager->graphObj_get_tree (obj);
  if (arb_gra != null)
    {
      arb_schema = arb_gra;
      pCurSig = the_graphicalTrees->get_arb_sig (&arb_schema);
      if (act_autorisee (C_text_win, 0, typ_vide, pSig))
	{
	  pUndo->get_undo_text (pCurSig, pk_true, 0);	/* ATTENTION no_obj = 0? */
	  detruire_chemin_restant (arb_schema, pk_false);
	  ndestroyattr (pCurSig, GRAPHI_ATTRNUMBER);
	  nmakeattrntrees (pSig, GRAPHI_ATTRNUMBER, arb_schema);
	  STree_replaceSon (pCurSig, pSig, 0);
	  pCurObjManager->set_arb_cou_engl (pSig);
	  idents = idente = null;
	  ope = noper (pSig);
	  if (STree_isExprProc (ope))
	    {
	      ok = pk_true;
	      SET_NILTREE (l_ent);
	      SET_NILTREE (l_sort);
	      prempasse (pSig, &l_ent, &l_sort);
	      p_bte = pCurObjManager->get_bte_cou_engl ();
	      pCurObjManager->remonter (pk_false);

	      /* moved forward Fev 2005
	         if (((ope == opappe)||(ope == opinst)) && !the_graphicalTrees->bte_avec_port(p_bte))
	         {
	         gere_appel_proc(pSig, p_bte);
	         *ClickNameNecessary = pk_false;
	         }
	         else
	       */
	      if (!nisemptyList (l_ent) || !nisemptyList (l_sort))
		{
		  pDisplay->sauvegarde_nomport ();
		  cliquer_port_nom (obj, l_ent, l_sort);
		  *ClickNameNecessary = pk_true;
		}
	      else if (((ope == opappe) || (ope == opinst)) && !the_graphicalTrees->bte_avec_port (p_bte))
		{
		  gere_appel_proc (pSig, p_bte);
		  *ClickNameNecessary = pk_false;
		}
	    }
	}
    }
  return (ok);
}


/**
  Cette fonction cree des boites de taille variable.
  Elle est appellee plusieurs fois jusqu'a ce que le bouton droit de
  la souris soit appuye (etat_bout a relache).
  Le calcul de la taille est effectue a chaque fois pour dessiner la boite
  On teste si l'insertion de la boite est possible lorsque l'action est
  terminee.
   ? etat_bout : etat du bouton (enfonce, tire ou relache)
  ? x_ret, y_ret : position de la souris
*/
void
builder::buildBox (int etat_bout, int x_ret, int y_ret)
{
  int width, height;
  typcoord halo[2];

  switch (etat_bout)
    {
    case appuyer:
      if (act_autorisee (C_creerbteexpr, 0, typ_bte, null))
	{
	  x_coin_g = x_ret & mask;
	  y_coin_g = y_ret & mask;
	  x_pred = x_coin_g;
	  y_pred = y_coin_g;
	  tours++;
	}
      else
	the_messageTool->error_mess (actimp);
      break;
    case tirer:
      dessiner_tirer_souris (x_ret, y_ret);
      break;
    case relacher:
      if (tours > 2)
	pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_EFFACE);
      width = ((x_ret & mask) - x_coin_g) / 2;
      height = ((y_ret & mask) - y_coin_g) / 2;
      getSizeMin (&width, &height);	// minimun size for a box
      tcoord[0].x = x_coin_g + width;
      tcoord[0].y = y_coin_g + height;
      tcoord[1].x = abs (width);
      tcoord[1].y = abs (height);
      pDraw->dessiner_rect (tcoord[0], tcoord[1].x, tcoord[1].y, COL_DEF);
      the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord, halo);
      if (pCurObjManager->test_cr_bte (halo))
	createExprBox (LL_make_lfeed_emptyList (), 2, tcoord);
      else
	the_messageTool->error_mess (crbteimp);
      tours = 1;
    }
}


///
void
builder::buildData (int x, int y)
{
  typcoord halo[2], tcoord[2];

  tcoord[0].x = x;
  tcoord[0].y = y;
  tcoord[1].x = 2 * taillepave_y;
  tcoord[1].y = 2 * taillepave_y;
  the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord, halo);
  if (pCurObjManager->test_cr_bte (halo))
    {
      createData (tcoord);
      pDraw->dessiner_data (tcoord[0], tcoord[1].x, tcoord[1].y, COL_REAL);
    }
  else
    the_messageTool->error_mess (crbteimp);
}


 /**
  Cette fonction modifie la taille d'une boite designee par l'utilisateur
  cette procedure est appellee plusieurs fois jusqu'a ce que le bouton
   soit relache.
   Le calcul de la taille est effectue a chaque fois pour controler si
   l'ecartement des ports est suffisant.
   on teste l'insertion de la boite a la fin de l'action (etat_bout a
   relacher).
   Les 4 premiers parametres : idem que pour la procedure ci-dessus
   ? force : etat du bouton details (ne sert pas pour le moment)
 */
void
builder::taille_bte (int etat_bout, int x_ret, int y_ret, boolean force)
{
  int l, h, nbcoord;
  typcoord centre;
  boolean affich_pur;

  switch (etat_bout)
    {
    case appuyer:
      if (act_autorisee (C_resize, 0, typ_bte, null))
	{
	  x_pred = x_ret & mask;
	  y_pred = y_ret & mask;
	  m_t_bte = pCurObjManager->reper_obj (x_pred, y_pred, &typ_obj);
	  if (typ_obj == typ_port)
	    {
	      m_t_bte = the_graphicalTrees->tree_get_graphObj (the_graphicalTrees->boite_mere (m_t_bte));
	      typ_obj = typ_bte;
	    };
	  if (m_t_bte != 0 && typ_obj == typ_bte)
	    {
	      the_graphicalTrees->getCoord (the_graphicTablesManager->graphObj_get_tree (m_t_bte), &nbcoord, bcoord, 0);
	      bouge.x = 0;
	      bouge.y = 0;
	      l = bcoord[1].x;
	      h_bte = bcoord[1].y;
	      centre = bcoord[0];
	      if (x_pred >= centre.x)
		{
		  if (x_pred >= centre.x + l / 2)
		    bouge.x++;
		  x_coin_g = centre.x - l;
		  x_pred = centre.x + l;
		}
	      else
		{
		  if (x_pred <= centre.x - l / 2)
		    bouge.x--;
		  x_coin_g = centre.x + l;
		  x_pred = centre.x - l;
		}
	      if (y_pred >= centre.y)
		{
		  if (y_pred >= centre.y + h_bte / 2)
		    bouge.y++;
		  y_coin_g = centre.y - h_bte;
		  y_pred = centre.y + h_bte;
		}
	      else
		{
		  if (y_pred <= centre.y - h_bte / 2)
		    bouge.y--;
		  y_coin_g = centre.y + h_bte;
		  y_pred = centre.y - h_bte;
		}
	      x_stop = x_pred;
	      y_stop = y_pred;
	      tours++;
	    }
	}
      else
	the_messageTool->error_mess (actimp);
      break;
    case tirer:
      if (tours > 1)
	{
	  pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_EFFACE);
	  if (bouge.x != 0)
	    x_pred = x_ret & mask;
	  else
	    x_pred = x_stop;
	  if (bouge.y != 0)
	    y_pred = y_ret & mask;
	  else
	    y_pred = y_stop;
	  h = (y_pred - y_coin_g) / 2;
	  centre.x = (x_coin_g + x_pred) / 2;
	  centre.y = y_coin_g + h;
	  x_stop = x_pred;
	  if (abs (h) >= h_bte)	/* ecart_port is not called if the box is extended */
	    y_stop = y_pred;
	  else if (ecart_port (the_graphicTablesManager->graphObj_get_tree (m_t_bte), mask, centre.x, centre.y, h, force))
	    y_stop = y_pred;
	  else
	    y_pred = y_stop;
	  pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_DEF);
	  tours++;
	}
      break;
    case relacher:
      if (tours > 2)
	{
	  pDraw->dessiner_rect2 (x_coin_g, y_coin_g, x_pred, y_pred, COL_EFFACE);
	  l = (x_stop - x_coin_g) / 2;
	  h = (y_stop - y_coin_g) / 2;
	  getSizeMin (&l, &h);	// minimun size for a box
	  tcoord[0].x = x_coin_g + l;
	  tcoord[0].y = y_coin_g + h;
	  tcoord[1].x = abs (l);
	  tcoord[1].y = abs (h);
	  pDraw->dessiner_rect (tcoord[0], tcoord[1].x, tcoord[1].y, COL_DEF);
	  positionne_action (rout_tail_bte);
	  pUndo->get_undo_bte (C_resize, m_t_bte, pk_false);
	  affich_pur = pk_true;
	  if (bouge.y == 0)
	    if (abs (l) <= bcoord[1].x)
	      /* si on retrecie la boite dans sa largeur, on ne lance pas de routage,
	         on modifie directement les coordonnees des ports et des segments */
	      chang_taille_direct (m_t_bte, bouge.x, tcoord);
	    else
	      {
		if (!agrandit_taille_direct (m_t_bte, bouge.x, tcoord))
		  if (!test_chang_coord (m_t_bte, typ_bte, 2, tcoord, force, pk_false, &bouge))
		    {
		      pUndo->set_undo_bte ();
		      affich_pur = pk_false;
		    }
	      }
	  else
	    {
	      if (!test_chang_coord (m_t_bte, typ_bte, 2, tcoord, force, pk_false, &bouge))
		{
		  pUndo->set_undo_bte ();
		  affich_pur = pk_false;
		}
	    }
	  tours = 1;
	  if (!affich_pur)
	    pCurObjManager->maj_l_obj_design (pCurObjManager->get_arb_cou_engl ());
	  C2Java_refresh (pk_true);
	}
    }
}



/**
  Fonction appelee lorsque l'utilisateur clique sur le bouton "archi"
  (dans le sous menu "compile") avant d'appeler le compilateur.
  Elle complete le modele ?p_modele representant les composants de 
  l'architecture en vue d'une compilation :
    - cree un modele de processus pour chaque composant, place le contenu 
    du composant dans le corps du modele et remplace le contenu du composant
    par un appel au modele.
    - rajoute un pragma a chaque modele cree pour indiquer au compilo qu'on
    est en repartition. Le pragma est "RunOn no", no etant un numero de
    processeur, de 1 a n.
  Rend NILTREE si la fonction ne peut pas complï¿œter le processus.
*/
ntrees
builder::buildArchi (ntrees p_modele)
{
  ntrees p_expr, l_decl_loc, p_bte, p_dupl_sig, p_nom, p_proc, p_expr_appel, pere, p, p_dupl_bte, p_dupl_expr;
  int no, i, j, nb_bte, nb_fils, no_bte_p_modele;
  ptrepstr nom;
  ntrees pereproc;
  int narit, ii;

  no = 1;
  l_decl_loc = STreeModel_getLocalDeclarations (p_modele);
  pere = STreeModel_getBody (p_modele);
  if (noper (pere) != oplfeed)
    return (NILTREE);
  pUndo->get_undo_archi (p_modele);

  pereproc = nfather (p_modele);
  narit = ntreearity (pereproc);
  for (ii = 1; ii <= narit; ii++)
    p = nchild (ii, pereproc);

  no_bte_p_modele = the_graphicalTrees->no_boite_proc (p_modele);
  nb_fils = ntreearity (pere);
  for (i = 1; i <= nb_fils; i++)
    {
      p_expr = nchild (i, pere);
      p_bte = nattrntrees (p_expr, GRAPHI_ATTRNUMBER);
      if (p_bte != null)
	{
	  /* pour chaque boite de ce niveau, on construit un s/processus
	     qui va contenir le contenu de la boite. */
	  nom = construire_nom ("PROC", no);
	  p_nom = LL_make_nom_atom (nom);
	  p_dupl_sig = dupl_bte (p_bte);
	  p_proc = STree_mkProcess (p_nom, STree_mkSkeletonInterface (), STree_mkEmptyBody ());
	  p_dupl_expr = STreeModel_getBody (p_proc);
	  ntreeList_insertFirst (p_dupl_sig, p_dupl_expr);
	  /* on construit le processus graphique  */
	  p_dupl_bte = nattrntrees (p_dupl_sig, GRAPHI_ATTRNUMBER);
	  build_proc_gra (p_proc, p_dupl_bte, i);
	  /* on rajoute les pragmas necessaires a la repartition */
	  build_prag_repart (p_proc, no);
	  insertGraphicalModel (p_proc, p_modele, pk_false, pk_true);
	  C2Java_SetCurrentTree (no_bte_p_modele);	// java world
	  detruire_chemin_restant (p_bte, pk_false);	/*on detruit les chemins partant 
							   des ports d'entree de la boite, etant donne qu'on remplace du 
							   graphique par du texte */
	  if (noper (p_expr) == oplfeed)	/* pour blinder */
	    {
	      nb_bte = ntreearity (p_expr);
	      for (j = 1; j <= nb_bte; j++)	/* on detruit les boites de p_expr */
		ndestroyListElem (p_expr, 1);
	    }
	  p_expr_appel = STree_mkModelCall (nom);
	  /* on remplace par l'appel de proc. et on recupere l'attribut */
	  STree_replaceSon (p_expr, p_expr_appel, 0);
	  nmakeattrntrees (p_expr_appel, GRAPHI_ATTRNUMBER, p_bte);
	  no++;
	}
      for (ii = 1; ii <= narit; ii++)
	p = nchild (ii, pereproc);
    }
  C2Java_SetCurrentTree (no_bte_p_modele);	// java world
  return (p_modele);
}


/**
  Restauration du modï¿œle ?p_archi tel qu'il ï¿œtait avant la compilation
  d'architecture.
*/
void
builder::restaure_pre_archi (ntrees p_archi)
{
  /*
     int    no_bte;
     ntrees arb_cou, rac, proc_cou;
   */
  /*  rac = pCurObjManager->get_racine();
     arb_cou = pCurObjManager->get_arb_cou_engl();
     while (!STree_isModel(arb_cou))
     {
     pCurObjManager->remonter(pk_false);
     arb_cou = pCurObjManager->get_arb_cou_engl();
     }
     if (arb_cou != rac)
     {
     pCurObjManager->remonter( pk_false);  on remonte jusqu'aux declarations locales 
     no_bte = pUndo->set_undo_archi(pCurObjManager->getModelInWhich_cou_engl());
     pCurObjManager->descendre(no_bte, pk_false);
     } */

}



/**
  Fonction appelï¿œe aprï¿œs la compilation d'une architecture.
  On parcourt chaque bte processeur, et on cherche ï¿œ connecter tous les ports
  de mï¿œme nom non connectï¿œs (ces ports correspondent aux signaux rajoutï¿œs 
  par le compilo.
  ?p_proc : processus venant d'ï¿œtre compilï¿œ.
*/
void
builder::add_new_links (ntrees p_proc)
{
  ntrees p_expr, p_bte, l_ports, port, p_corps;
  int i, j, nb, nb_fils;
  boolean ok;

  p_corps = STreeModel_getBody (p_proc);
  nb_fils = ntreearity (p_corps);
  i = 1;
  while (i <= nb_fils)
    {
      p_expr = nchild (i, p_corps);
      if (ISNOT_NILTREE ((p_bte = nattrntrees (p_expr, GRAPHI_ATTRNUMBER))))
	{
	  port = the_graphicalTrees->getFirstPort (p_bte);
	  while (ISNOT_NILTREE (port))
	    {
	      if (!the_graphicalTrees->is_connected_port (port, pk_false) && (noper (port) == gpkOp_ports))
		{
		  l_ports = the_graphicalTrees->get_ports_same_name (p_corps, port);
		  if (ISNOT_NILTREE (l_ports))
		    {
		      nb = ntreearity (l_ports);
		      for (j = 1; j <= nb; j++)
			ok = rout_port_port (the_graphicTablesManager->graphObj_get_tree ((int) nintof (nchild (j, l_ports))), port, pk_false, pk_false);
		    }
		}
	      port = ntrees_getNext (port);
	    }
	}
      i++;
    }
}




/**
  Construction graphique de l'arbre ?p_compil qui est un des processus
  rï¿œsultant de la compilation du processus ?p_archi
  ?p_compil est le processus construit par le compilo correspondant au 
  processus de pragma RunOn ?no qu'on a compilï¿œ.
  le processus ?p_compil a pour expression une composition ï¿œ 3 fils:
        1 ier fils : toutes les synchro
	2 iï¿œme fils: 1 composition ï¿œ 2 fils (|boolï¿œens |  non boolï¿œens|)
           1 ier fils : 1 composition (|boolï¿œens simples | boolï¿œens retardï¿œs|)
           2 ier fils : 1 composition (|autres simples | autres retardï¿œs|)
	3 iï¿œme fils : les dï¿œpendances
  On recherche l'appel du processus de pragma ?no pour le remplacer par le
  corps de ?p_compil et pour lui rajouter les signaux boolï¿œens calculï¿œs 
  par le compilo (signaux correspondant ï¿œ des horloges).
*/
void
builder::build_compiled_proc (ntrees p_archi, ntrees p_compil, int no)
{
  boolean b;
  ntrees p_modele, p_appel, p_corps, p_bte, p_decl_locales;
  ptrepstr str_no;
  char buf_aux[20];

  sprintf (buf_aux, "\"%d\"", no);
  str_no = string2repstr (buf_aux);
  /* on recherche le modï¿œle qui a pour pragma:  RunON "no" */
  p_modele = get_modele_same_prag (p_archi, str_no);
  /* on recherche son appel, il y en a un et un seul */
  b = STreeModel_getCallToIn (STreeModelModule_getName (p_modele), p_archi, &p_appel);
  p_bte = nattrntrees (p_appel, GRAPHI_ATTRNUMBER);
  p_corps = STreeModel_getBody (p_compil);
  STree_replaceSon (p_appel, p_corps, 0);	/*on remplace l'appel par le nouveau corps */
  nmakeattrntrees (p_corps, GRAPHI_ATTRNUMBER, p_bte);
  construire_btes_internes_compilees (p_corps);
  /* on rajoute les signaux calculï¿œs par le compilo en tant que nouveaux ports */
  add_new_ports (p_compil, p_corps);
  p_decl_locales = STreeModel_getLocalDeclarations (p_archi);
  /* on rajoute les declarations locales venant du compilateur */
  the_graphicalTrees->mergeLocalDecl (p_decl_locales, STreeModel_getLocalDeclarations (p_compil));
  /*on detruit le modele de pragma RunON no" qu'on avait construit pour la compilation */
  //  detruire_bte(the_graphicalTrees->tree_get_graphObj(nattrntrees(p_modele, GRAPHI_ATTRNUMBER)));
}


/**
  Cette fonction est utilisï¿œe pour la compilation embarquï¿œe:
  Elle construit un processus contenant la boite dï¿œsignï¿œe par l'utilisateur
  ainsi que toutes les boites directement connectï¿œes ï¿œ ?bte. En principe,
  ?bte reprï¿œsente la description d'une architecture et les boites connectï¿œes
  ï¿œ ?bte reprï¿œsentent des canaux de communication. Ces canaux sont
  eux-mï¿œme composï¿œs de 3 boites (une boï¿œte pour la reception, une pour 
  l'emission et une pour la ligne).
  Rend le nouveau processus crï¿œï¿œ s'il n'y a pas eu de problï¿œme, rend 
  NILTREE sinon.
*/
ntrees
builder::build_embedded_proc (int bte)
{
  ntrees p_dupl_proc, p_ret, p, p_proc, p_ou_inserer, p_sig, rac;

  SET_NILTREE (p_ret);
  p = the_graphicTablesManager->graphObj_get_tree (bte);
  p_sig = the_graphicalTrees->get_arb_sig (&p);
  p_proc = STree_getModelInWhich (p_sig);
  // nDEBUG_subtree(0, 8, "entering builder::build_embedded_proc, p_sig is", p_sig);
  /* on va travailler sur une copie du modï¿œle contenant la boite cliquï¿œe;
     si ce processus est la racine, il n'a pas de bte associï¿œe, alors on
     l'a crï¿œe */
  rac = pCurObjManager->get_racine ();
  if (p_proc == rac)
    the_messageTool->error_mess (not_allowed_embed);
  else
    {
      p_ou_inserer = nfather (p_proc);
    }
  p_dupl_proc = the_graphicalTrees->dupl_arbre (p_proc);	/* on fait la copie du processus */
  STree_addDeclarationToList (p_ou_inserer, p_dupl_proc);	/* on insï¿œre le processus */
  if (clean_embedded_box (bte, p_dupl_proc))
    p_ret = p_dupl_proc;
  else
    the_messageTool->error_mess (not_allowed_embed);
  // nDEBUG_subtree(0, 8, " exiting builder::build_embedded_proc, p_ret is", p_ret);
  return (p_ret);
}


/**
   Delete the box bte (in) and the Signal tree associated. If the box is
   associated with the highest level expression, the complete model is deleted.
*/
void
builder::detruire_bte (int bte, boolean detach)
{
  ntrees p_gra, p_sig, p;
  boolean isMainBox;

  // printf("DEBUG entering builder::detruire_bte, bte is %i\n", bte); fflush(stdout);
  isMainBox = the_graphicalTrees->isBoxProc (bte);
  if (isMainBox)
    {				// the model is cut
      p_sig = the_graphicalTrees->getModel (bte);
      // nDEBUG_subtree(3, 2, "     in   builder::detruire_bte 1, p_sig is ", p_sig);
      if (detach)
	{
	  detach_arb_sig (p_sig);
	}			/* on dï¿œtruit l'arbre Signal associï¿œ ï¿œ la boite */
      else
	{
	  the_graphicTablesManager->enlever_obj (bte);
	  ntreesDestroy (p_sig);
	}
    }
  else
    {
      p_gra = the_graphicTablesManager->graphObj_get_tree (bte);
      p_sig = the_graphicalTrees->get_arb_sig (&p_gra);
      pCurObjManager->enlever_l_bte_d (bte);	/* on l'enlï¿œve de la liste des objets dï¿œsignables */
      p = the_graphicalTrees->getFirstPort (p_gra);
      while (p != null)
	{			/* on dï¿œtruit les liens et les ports associï¿œs a la boite */
	  detruire_liens (p, typ_port);
	  the_graphicTablesManager->enlever_obj (the_graphicalTrees->tree_get_graphObj (p));
	  p = ntrees_getNext (p);
	};
      the_graphicTablesManager->enlever_obj (bte);
      if (detach)
	detach_arb_sig (p_sig);	/* on dï¿œtruit l'arbre Signal associï¿œ ï¿œ la boite */
      else
	detruire_arb_sig (p_sig);	/* on dï¿œtruit l'arbre Signal associï¿œ ï¿œ la boite */
    }
}



/**
  Creation de la structure de donnees associee a une boite de type
  expression de processus et insertion de la boite dans l'arbre
  courant.
  arb_sig : arbre Signal auquel on accroche la nouvelle boite crï¿œï¿œe
  ?nbcoord et ?tcoord sont le nombre et le tableau de coordonnees 
  necessaires pour dessiner la boite.
  (implementation limitee aux rectangles)
*/
void
builder::createExprBox (ntrees arb_sig, int nbcoord, typcoord * tcoord)
{
  ntrees p, currentTree;
  int bte_design;
  typcoord nouvtcoord[maxcoord];

  transfcoord (1, tcoord, nouvtcoord);
  nouvtcoord[1].x = tcoord[1].x;
  nouvtcoord[1].y = tcoord[1].y;
  bte_design = the_graphicalTrees->createbte (arb_sig, nouvtcoord, nbcoord);
  pCurObjManager->inser_l_bte_d (bte_design);
  currentTree = pCurObjManager->get_arb_cou_engl ();
  if (ISNOT_NILTREE (currentTree))
    if (noper (currentTree) == oplfeed)
      {				/* add the new tree as a son of currentTree */
	if ((p = nchild (1, currentTree)) != null)
	  if (nattrntrees (the_graphicalTrees->desc_expr_sig (p), GRAPHI_ATTRNUMBER) == null)
	    /* cas ou la boite englobante ne contenait que du texte : on detruit le texte */
	    while (p != currentTree)
	      p = ndestroyListElem (currentTree, 1);
	ntreeList_appendLast (currentTree, arb_sig);
      }
    else
      {
	/* replace currentTree by the new tree */
	p = LL_make_lfeed_list (arb_sig);
	STree_moveAttribute (currentTree, p, GRAPHI_ATTRNUMBER);
	STree_replaceSon (currentTree, p, 0);
	pCurObjManager->set_arb_cou_engl (p);
      }
}


///
void
builder::createData (typcoord * tcoord)
{
  int no;
  typcoord nouvtcoord[maxcoord];

  transfcoord (1, tcoord, nouvtcoord);
  nouvtcoord[1].x = tcoord[1].x;
  nouvtcoord[1].y = tcoord[1].y;
  no = the_graphicalTrees->createData (nouvtcoord, 2);
  pCurObjManager->inser_l_bte_d (no);
}




/**
    Creation de la structure de donnees associee a une boite de type 
    composition de processus qui encapsule une ou plusieurs autres boites.
    nbcoord et tcoord sont le nombre et le tableau de coordonnees necessaires 
    pour  dessiner la boite.
   ?l_bte_enc: liste des numeros des boites encapsulees
   Creation de port pour tous chemins dont un segment coupe la boite creee et
   routage de ces chemins.
   Rend vrai si le routage n'a pas echoue et si tous les ports ont ete 
   crees, sinon faux.
   Si un port n'a pas pu etre cree !cr_ports a faux
*/
boolean
builder::createGlueBox (int nbcoord, typcoord * tcoord, ntrees l_bte_enc, boolean * cr_ports)
{
  ntrees arb_no_bte, arb_gra, arb_sig, arb_sig_enc;
  ntrees l_ports_diff_encaps;
  typcoord nouvtcoord[maxcoord];
  int bte;
  boolean ok;

  *cr_ports = pk_true;
  transfcoord (1, tcoord, nouvtcoord);
  nouvtcoord[1].x = tcoord[1].x;
  nouvtcoord[1].y = tcoord[1].y;
  arb_sig = LL_make_lfeed_emptyList ();	/* on cree la boite qui va encapsuler les autres */
  bte = the_graphicalTrees->createbte (arb_sig, nouvtcoord, nbcoord);
  ntreeList_appendLast (pCurObjManager->get_arb_cou_engl (), arb_sig);
  l_ports_diff_encaps = LL_make_lports_emptyList ();
  if (createports_encaps (bte, l_bte_enc, l_ports_diff_encaps))
    {
      arb_no_bte = nchild (1, l_bte_enc);
      while (arb_no_bte != null)
	{			/* insertion des boites dans la nouvelle boite creee */
	  arb_gra = the_graphicTablesManager->graphObj_get_tree ((int) nintof (arb_no_bte));
	  recalcul_bte (bte, arb_gra, pk_true);
	  arb_sig_enc = the_graphicalTrees->get_arb_sig (&arb_gra);
	  ntreesDestroy (arb_sig_enc);
	  ntreeList_insertFirst (arb_sig_enc, arb_sig);
	  arb_no_bte = ntrees_getNext (arb_no_bte);
	}
      pCurObjManager->setCurrentObjectsList ();	/*on met a jour la liste des objets designables */
      detr_liens_ports_encaps (bte, l_bte_enc, pk_false);
      detr_liens_ports_encaps (bte, l_bte_enc, pk_true);
      if (rout_ports_bte_encaps (bte, l_bte_enc, l_ports_diff_encaps, pk_false))
	ok = rout_ports_bte_encaps (bte, l_bte_enc, l_ports_diff_encaps, pk_true);
    }
  else
    {
      *cr_ports = pk_false;
      ok = pk_false;
    }
  the_graphicalTrees->detr_attr_ports_bte (bte, ALGO_ATTRNUMBER);	/* on detruit les attributs accroches
									   aux ports de bte pour le routage */
  return (ok);
}



/**
  Duplication d'un arbre representant une boite et annulation des liens 
  externes (si les ports de la boite avaient des connexions externes).
  Rend l'arbre Signal dupliquï¿œ.
  Modifiï¿œ le 18/10/99
*/
ntrees
builder::dupl_bte (ntrees arb_gra)
{
  ntrees arb_sig_dupl, arb_gra_dupl;

  // nDEBUG_subtree(0, 8, "entering builder::dupl_bte, arb_gra is", arb_gra);
  arb_sig_dupl = the_graphicalTrees->dupl_arbre (the_graphicalTrees->get_arb_sig (&arb_gra));
  arb_gra_dupl = nattrntrees (arb_sig_dupl, GRAPHI_ATTRNUMBER);
  the_graphicalTrees->annuler_chem_ext (arb_gra_dupl);
  // nDEBUG_subtree(0, 8, " exiting builder::dupl_bte, arb_sig_dupl is", arb_sig_dupl);
  return (arb_sig_dupl);
}



/**
  Recopie d'une boite et insertion dans l'arbre courant englobant
  (correspond a l'action "PASTE" de l'utilisateur).
    -  recopie de l'arbre Signal associe et de l'ensemble des attributs 
       graphiques (boite, ports, boites internes et liens internes).
    -  le nouvel arbre Signal est insere dans arb_cou_engl.
  ?arb_gra : arbre representant la boite a dupliquer
 ?tcoord : coordonnees de la bte dupliquee.
 ?modtail a vrai si la taille a ete modifiee.
 rend l'arbre Signal recopie.
*/
void
builder::createPasteBox (ntrees arb_gra, boolean modtail, typcoord * tcoord)
{
  ntrees arb_sig_dupl, bte_dupl, currentTree;
  int no_bte, nbcoord;
  typcoord ocoord[2], halo[2];

  the_graphicalTrees->getCoord (arb_gra, &nbcoord, ocoord, 0);
  arb_sig_dupl = dupl_bte (arb_gra);
  the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord, halo);
  bte_dupl = the_graphicalTrees->get_arb_boite (arb_sig_dupl);
  no_bte = the_graphicalTrees->tree_get_graphObj (bte_dupl);
  modif_coord (no_bte, typ_bte, halo, nbcoord, tcoord, 0);
  if (modtail)
    {
      ocoord[0].x = tcoord[0].x;
      ocoord[0].y = tcoord[0].y;
      the_graphicalTrees->modif_port_bte (bte_dupl, ocoord, tcoord, pk_true);
    }
  currentTree = pCurObjManager->get_arb_cou_engl ();
  the_graphicalTrees->inser_arb_sig (arb_sig_dupl, &currentTree);
  if (currentTree != pCurObjManager->get_arb_cou_engl ())
    pCurObjManager->set_arb_cou_engl (currentTree);
  pCurObjManager->inser_l_bte_d (no_bte);	/* on insere la nouvelle boite dans la liste des btes designables */
  //  return(arb_sig_dupl); 
}


/**
  * cette fonction est appelee lorsque l'utilisateur remplace une boite ?bte
  * par ses boites internes. Elle detruit l'arbre associe a la boite ?bte,
  * remonte les boites internes d'un niveau (?l_bte_dec) et effectue les
  * routages necessaires.
  * cette fonction rend faux (si les boites internes n'ont pas la taille
  * necessaire (!ec_port a vrai) et (!tst a faux) ou si l'ecartement 
  * des ports est insuffisant (!ec_port a faux) et (!tst a faux)) 
  * ou (si le routage a echoue (!tst a vrai))
  */
boolean
builder::detr_bte_decaps (int bte, ntrees l_bte_dec, int *noErrMess)
{
  ntrees l_ports, arb_gra, arb_gra_bte, arb_no_bte, arb_sig_dec, arb_sig_bte, arb_port, p_ch, p_lien;
  int port, ope;
  boolean intern, ok;

  // printf("DEBUG entering  builder::detr_bte_decaps\n"); fflush(stdout);
  ok = pk_true;
  if (test_decaps_bte (bte, noErrMess))
    {
      l_ports = creer_l_info_bte_decaps (bte, l_bte_dec);
      arb_gra_bte = the_graphicTablesManager->graphObj_get_tree (bte);
      arb_sig_bte = nfather (the_graphicalTrees->get_arb_sig (&arb_gra_bte));
      arb_no_bte = nchild (1, l_bte_dec);
      while (arb_no_bte != null)
	{
	  arb_gra = the_graphicTablesManager->graphObj_get_tree ((int) nintof (arb_no_bte));
	  arb_sig_dec = the_graphicalTrees->get_arb_sig (&arb_gra);
	  ntreesDestroy (arb_sig_dec);
	  pCurObjManager->inser_l_bte_d ((int) nintof (arb_no_bte));
	  ntreeList_appendLast (arb_sig_bte, arb_sig_dec);
	  arb_no_bte = ntrees_getNext (arb_no_bte);
	}
      pCurObjManager->enlever_l_bte_d (bte);
      the_graphicTablesManager->enlever_obj (bte);
      arb_sig_bte = nattrcarrier (arb_gra_bte);
      if (noper (arb_gra_bte) == opschema)
	{
	  arb_port = the_graphicalTrees->getFirstPort (arb_gra_bte);
	  while (arb_port != null)
	    {
	      port = the_graphicalTrees->tree_get_graphObj (arb_port);
	      ope = noper (arb_port);
	      p_ch = nchild (3, arb_port);
	      p_lien = nchild (4, arb_port);
	      if (p_ch != null)
		{
		  intern = (ope == gpkOp_porte);
		  detruire_chemin (p_ch, intern);
		}
	      if (p_lien != null)
		{
		  p_ch = the_graphicTablesManager->graphObj_get_tree ((int) nintof (p_lien));
		  intern = (ope == gpkOp_ports);
		  detruire_chemin (p_ch, intern);
		}
	      the_graphicTablesManager->enlever_obj (port);
	      arb_port = ntrees_getNext (arb_port);
	    }
	}
      detruire_arb_sig (arb_sig_bte);
      ok = rout_ports_btes_decaps (l_ports, l_bte_dec);
      if (!ok)
	(*noErrMess) = routimpnodecaps;
    }
  else
    ok = pk_false;
  // printf("DEBUG  exiting  builder::detr_bte_decaps\n"); fflush(stdout);
  return (ok);
}


/**
  Cette procedure (appelee par taille_bte, module builder lorsqu'on
  diminue la largeur de la boite et par agrandit_taille_direct ci-dessous
  lorsqu'on augmente la largeur de la boite) modifie les coordonnees 
  des segments qui arrivent sur la boite sans lancer le routage (on
  raccourcit ou rallonge le dernier segment)
  ?bte : numero de la boite qu'on modifie.
  ?sens : =1 lorsqu'on touche au cote droit, =-1 pour le cote gauche.
  ?tcoord_bte : nouvelles coordonnees de la boite
*/
void
builder::chang_taille_direct (int bte, int sens, typcoord * tcoord_bte)
{
  boolean saute;
  int nbcoord, x, y;
  ntrees p_port, lien, chemin, p, p_seg, p_coord;
  typcoord halo[2], pcoord_rela[2], tcoord[2];

  the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord_bte, halo);
  modif_coord (bte, typ_bte, halo, 2, tcoord_bte, 0);	/* cette procedure ne modifie que le
							   centre de la boite */
  p = the_graphicTablesManager->graphObj_get_tree (bte);
  nmodifvalint (nchild (1, nchild (2, nchild (2, p))), tcoord_bte[1].x);
  if ((p = ntrees_getNext (p)) != null)
    {
      p_port = nchild (1, p);
      while (p_port != null)
	{
	  saute = pk_false;
	  p_coord = nchild (2, p_port);
	  the_graphicalTrees->getCoord (p_coord, &nbcoord, pcoord_rela, 1);
	  if (pcoord_rela[0].x < 0)
	    {
	      if (sens == 1)
		saute = pk_true;
	      nmodifvalint (nchild (1, p_coord), -tcoord_bte[1].x);
	    }
	  else
	    nmodifvalint (nchild (1, p_coord), tcoord_bte[1].x);
	  if (!saute)
	    {
	      switch (noper (p_port))
		{
		case gpkOp_porte:
		  if ((lien = nchild (4, p_port)) != null)
		    {
		      chemin = the_graphicTablesManager->graphObj_get_tree ((int) nintof (lien));
		      if (nchild (2, chemin) != null)
			{
			  p = nchild (4, chemin);
			  p_seg = nchild (ntreearity (p), p);
			  the_graphicalTrees->getCoord (nchild (2, p_seg), &nbcoord, tcoord, 0);
			  get_coord_seg_port (p_port, typ_port, &x, &y);
			  tcoord[1].x = x;
			  the_graphicalTrees->calcul_nouv_halo (typ_seg, tcoord, halo);
			  modif_coord (the_graphicalTrees->tree_get_graphObj (p_seg), typ_seg, halo, 2, tcoord, 0);
			}
		    }
		  break;
		case gpkOp_ports:
		  if ((chemin = nchild (3, p_port)) != null)
		    {
		      p_seg = nchild (1, nchild (4, chemin));
		      the_graphicalTrees->getCoord (nchild (2, p_seg), &nbcoord, tcoord, 0);
		      get_coord_seg_port (p_port, typ_port, &x, &y);
		      tcoord[0].x = x;
		      the_graphicalTrees->calcul_nouv_halo (typ_seg, tcoord, halo);
		      modif_coord (the_graphicalTrees->tree_get_graphObj (p_seg), typ_seg, halo, 2, tcoord, 0);
		    }
		  break;
		}
	    }
	  p_port = ntrees_getNext (p_port);
	}
    }
}



/**
  Cette fonction est appelee par taille_bte (module builder) lorsqu'
  on augmente la largeur de la boite. Elle teste si la boite peut etre
  agrandit en reduisant uniquement la taille des segments qui arrivent
  sur la boite. Si oui, elle appelle la procedure chang_taille_direct,
  si non, il y a echec.
  Memes parametres que pour la procedure ci-dessus.
  Si il y a echec, la procedure test_chang_coord (qui lance le routage)
  sera appelee par la suite.
*/
boolean
builder::agrandit_taille_direct (int bte, int sens, typcoord * tcoord_bte)
{
  boolean saute;
  int nbcoord;
  ntrees p_port, lien, chemin, p, p_seg;
  typcoord halo[2], pcoord_rela[2];

  p = the_graphicTablesManager->graphObj_get_tree (bte);
  if ((p = ntrees_getNext (p)) != null)
    {
      p_port = nchild (1, p);
      while (p_port != null)
	{
	  saute = pk_false;
	  the_graphicalTrees->getCoord (p_port, &nbcoord, pcoord_rela, 1);
	  if (pcoord_rela[0].x < 0)
	    if (sens == 1)
	      saute = pk_true;
	  if (!saute)
	    {
	      switch (noper (p_port))
		{
		case gpkOp_porte:
		  if ((lien = nchild (4, p_port)) != null)
		    {
		      chemin = the_graphicTablesManager->graphObj_get_tree ((int) nintof (lien));
		      if (nchild (2, chemin) != null)
			{
			  p = nchild (4, chemin);
			  p_seg = nchild (ntreearity (p), p);
			  the_graphicTablesManager->annuler_halo_seg (the_graphicalTrees->tree_get_graphObj (p_seg));
			}
		    }
		  break;
		case gpkOp_ports:
		  if ((chemin = nchild (3, p_port)) != null)
		    {
		      p_seg = nchild (1, nchild (4, chemin));
		      the_graphicTablesManager->annuler_halo_seg (the_graphicalTrees->tree_get_graphObj (p_seg));
		    }
		  break;
		}
	    }
	  p_port = ntrees_getNext (p_port);
	}
    }
  pCurObjManager->enlever_l_bte_d (bte);
  the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord_bte, halo);
  if (pCurObjManager->test_cr_bte (halo))
    {
      pCurObjManager->inser_l_bte_d (bte);
      chang_taille_direct (bte, sens, tcoord_bte);
      return (pk_true);
    }
  else
    return (pk_false);
}





/**
  Fonction appelï¿œe par la fonction "build_embedded_proc"
  ?bte est le numï¿œro de la boite dont on veut faire une compilation
  "embedded".
  ?p_proc est une copie du processus qui contient ?bte et dans laquelle
  on va travailler.
  Cette fonction "nettoie" le processus : on enlï¿œve toutes les btes et
  tous les ports non directement connectï¿œs ï¿œ ?bte.
  Rend vrai si tout s'est bien passï¿œ.
*/
boolean
builder::clean_embedded_box (int bte, ntrees p_proc)
{
  ntrees p_emb_bte, p_bte, p_port, p_aux;
  acces_gra *pt;
  int no, nouv_bte;
  boolean ok;

  nouv_bte = the_graphicTablesManager->locinconvnum (bte, 0);	/* numï¿œro de la bte dans la copie */
  p_emb_bte = the_graphicTablesManager->graphObj_get_tree (nouv_bte);
  /* descente simulï¿œe dans la boite de plus haut niveau de l'expression du processus */
  pCurObjManager->descendre (the_graphicalTrees->no_boite_proc (p_proc), pk_true);
  pt = pCurObjManager->recup_l_bte_design ();
  if (pt == null)		/* on n'a que du texte-->pas la peine de faire une compil "embedded" */
    ok = pk_false;
  else
    {				/* on dï¿œtruit toutes les btes de mï¿œme niveau non connectï¿œes ï¿œ nouv_bte */
      while (pt != null)
	{
	  p_bte = pt->elem_gra;
	  no = the_graphicalTrees->tree_get_graphObj (p_bte);
	  if (no != nouv_bte)
	    if (!the_graphicalTrees->is_btes_connectees (no, nouv_bte))
	      detruire_bte (no, pk_false);	// false: destroy not only detach
	  pt = pt->suiv_d;
	}
      /* pour chaque bte restante, on dï¿œtruit les ports non connectï¿œs ï¿œ nouv_bte et
         on dï¿œtruit les btes internes qui n'ont que des connexions internes */
      pt = pCurObjManager->recup_l_bte_design ();
      ok = pk_true;
      while (pt != null)
	{
	  p_bte = pt->elem_gra;
	  no = the_graphicalTrees->tree_get_graphObj (p_bte);
	  if (no != nouv_bte)
	    {
	      detr_ports_inutiles (p_bte, nouv_bte);
	      ok = detr_btes_internes (p_bte, p_proc);
	    }
	  pt = pt->suiv_d;
	}
    }
  if (ok)
    {
      /* au niveau de l'interface du processus, on dï¿œtruit tous les ports non connectï¿œs */
      p_port = the_graphicalTrees->getFirstPort (the_graphicTablesManager->graphObj_get_tree (the_graphicalTrees->no_boite_proc (p_proc)));
      while (ISNOT_NILTREE (p_port))
	{
	  p_aux = p_port;
	  p_port = ntrees_getNext (p_port);
	  if (!the_graphicalTrees->is_connected_port (p_aux, pk_true))
	    detruire_port (the_graphicalTrees->tree_get_graphObj (p_aux), typ_port);
	}
    }
  pCurObjManager->descendre (bte, pk_true);	/* on se replace lï¿œ oï¿œ ï¿œtait l'utilisateur */
  pCurObjManager->remonter (pk_true);
  return (ok);
}

/**
   Cette fonction, appelee lorsque l'utilisateur clique sur le bouton
   "glue" dessine une boite qui englobe une ou plusieurs boites. 
   Ces boites encapsulees se retouvent a l'interieur de la boite.
   Cette fonction rajoute un niveau de hierarchie au niveau de l'arbre
   Signal, rajoute des ports et modifie les liens.
   Pour les parametres, voir la fonction buildBox.
*/
void
builder::encapsuler_bte (int etat_bout, int x_ret, int y_ret)
{
  int x, y;
  typcoord halo[2];
  ntrees l_bte_enc;
  boolean ec_ports, no_bte;

  switch (etat_bout)
    {
    case appuyer:
      if (act_autorisee (C_encaps, 0, typ_vide, NILTREE))
	{
	  x_coin_g = x_ret & mask;
	  y_coin_g = y_ret & mask;
	  x_pred = x_coin_g;
	  y_pred = y_coin_g;
	  tours++;
	}
      else
	the_messageTool->error_mess (actimp);
      break;
    case tirer:
      dessiner_tirer_souris (x_ret, y_ret);
      break;
    case relacher:
      if (tours > 1)
	{
	  x = ((x_ret & mask) - x_coin_g) / 2;
	  y = ((y_ret & mask) - y_coin_g) / 2;
	  tcoord[0].x = x_coin_g + x;
	  tcoord[0].y = y_coin_g + y;
	  tcoord[1].x = abs (x);
	  tcoord[1].y = abs (y);
	  the_graphicalTrees->calcul_nouv_halo (typ_bte, tcoord, halo);
	  if (pCurObjManager->test_encaps_bte (halo, &l_bte_enc, &no_bte))
	    {
	      pUndo->get_undo_bte (C_encaps, 0, pk_true);
	      if (!createGlueBox (2, tcoord, l_bte_enc, &ec_ports))
		{
		  pUndo->set_undo_bte ();
		  pCurObjManager->setCurrentObjectsList ();
		  if (!ec_ports)
		    the_messageTool->error_mess (crptimpnoencaps);
		  else
		    the_messageTool->error_mess (routimpnoencaps);
		}
	    }
	  else
	    {
	      if (!no_bte)
		the_messageTool->error_mess (encapsimphalo);
	      else
		the_messageTool->error_mess (encapsimpbte);
	    }
	  tours = 1;
	  C2Java_refresh (pk_true);
	}
    }
}


/**
  Cette fonction est le dual de la precedente : elle remplace la boite
  cliquee par ses boites internes (supprime un niveau de hierarchie
  au niveau de l'arbre Signal et modifie les liens).
*/
void
builder::decapsuler_bte (int x_reticule, int y_reticule)
{
  int bte, noErrMess;
  objet_type typ_obj;
  ntrees l_bte_dec;
  boolean undo;

  bte = pCurObjManager->reper_obj (x_reticule, y_reticule, &typ_obj);
  if (bte != 0 && act_autorisee (C_decaps, bte, typ_obj, null))
    {
      if (the_graphicalTrees->une_boite_decaps (bte, &l_bte_dec))
	{
	  undo = pk_false;
	  pUndo->get_undo_bte (C_decaps, 0, pk_true);
	  if (!detr_bte_decaps (bte, l_bte_dec, &noErrMess))
	    {
	      pUndo->set_undo_bte ();
	      undo = pk_true;
	      the_messageTool->error_mess (noErrMess);
	    }
	  if (undo)
	    pCurObjManager->setCurrentObjectsList ();
	  C2Java_refresh (pk_true);
	}
      else
	the_messageTool->error_mess (decapsimpbte);
    }
  else
    the_messageTool->error_mess (actimp);
}
