/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
//_PUBLIC_


/// Constructor
drawing::drawing ()
{
}


/**
  * dessine un port d'entree a gauche ou a droite de la boite
  * suivant la valeur de ?ou
  * ?x_ret, ?y_ret : coordonnees du port (x_ret : abscisse du bord de la
  * boite ou est le port, y_ret : ordonnee du milieu du port).
  * ?pat : booleen a vrai lorsqu'on veut afficher la patte du port, a
  *        faux sinon.
  */
void
drawing::drawPort (int x_ret, int y_ret, boolean isInput, boolean demi, int ou, int col, boolean pat)
{
  int x_deb_pat, x_fin_pat, t_pat, sizeX, sizeY;
  int tx[3], ty[3];

  if (demi)
    {
      sizeY = triangle_y / 2;
      sizeX = taillepave_x / 2;
      t_pat = (taillepatte / 2);
    }
  else
    {
      sizeY = triangle_y;
      sizeX = taillepave_x;
      t_pat = taillepatte;
    }
  ty[0] = y_ret - sizeY;
  ty[1] = y_ret;
  ty[2] = y_ret + sizeY;
  if (isInput)
    {
      if (ou == droite)
	{
	  x_deb_pat = x_ret + 1;
	  x_fin_pat = x_deb_pat + t_pat + sizeX;
	  tx[0] = x_ret - 1;
	  tx[1] = x_ret - sizeX - 1;
	}
      else
	{
	  x_deb_pat = x_ret - t_pat - sizeX - 1;
	  x_fin_pat = x_ret;
	  tx[0] = x_ret + 1;
	  tx[1] = x_ret + sizeX;
	};
      tx[2] = tx[0];
      C2Java_drawPolygon (tx, ty, 3, col, 1);
      if (pat)
	C2Java_drawLine (x_deb_pat, y_ret, x_fin_pat, y_ret, col, 1);
    }
  else
    {
      if (ou == droite)
	{
	  tx[1] = x_ret + sizeX;
	  tx[0] = x_ret + 1;
	}
      else
	{
	  tx[1] = x_ret - sizeX;
	  tx[0] = x_ret - 1;
	}
      tx[2] = tx[0];
      C2Java_FillPolygon (tx, ty, 3, col, 1);
    }
}


/**
  * dessine le halo d'un port d'entree a gauche ou a droite de la boite
  * suivant la valeur de ?ou
  * ?x_ret, ?y_ret : coordonnees du port (x_ret : abscisse du bord de la
  * boite ou est le port, y_ret : ordonnee du milieu du port).
  * ?pat : booleen a vrai lorsqu'on veut afficher la patte du port, a
  *        faux sinon.
  */
void
drawing::drawHaloPort (int x_ret, int y_ret, boolean isInput, int ou, int col)
{
  int sizeX, sizeY;
  int tx[3], ty[3];

  sizeY = triangle_y + 1;
  sizeX = taillepave_x + 1;
  ty[0] = y_ret - sizeY;
  ty[1] = y_ret;
  ty[2] = y_ret + sizeY;
  if (isInput)
    {
      if (ou == droite)
	{
	  tx[0] = x_ret - 1;
	  tx[1] = x_ret - sizeX - 1;
	}
      else
	{
	  tx[0] = x_ret + 1;
	  tx[1] = x_ret + sizeX;
	};
      tx[2] = tx[0];
      C2Java_drawPolygon (tx, ty, 3, col, 1);
    }
  else
    {
      if (ou == droite)
	{
	  tx[1] = x_ret + sizeX;
	  tx[0] = x_ret + 1;
	}
      else
	{
	  tx[1] = x_ret - sizeX;
	  tx[0] = x_ret - 1;
	}
      tx[2] = tx[0];
      C2Java_FillPolygon (tx, ty, 3, col, 1);
    }
}


/**
  dessine un port d'entree externe sur la boite englobante.
  */
void
drawing::drawExternPort (int x_ret, int y_ret, boolean isInput, int ou, int col)
{
  int tx[3], ty[3], x_deb_pat, x_fin_pat;

  tx[0] = tx[2] = x_ret;
  ty[0] = y_ret - triangle_y;
  ty[1] = y_ret;
  ty[2] = y_ret + triangle_y;
  if (isInput)
    {
      if (ou == gauche)
	tx[1] = x_ret + taillepave_x;
      else
	tx[1] = x_ret - taillepave_x;
      C2Java_FillPolygon (tx, ty, 3, col, 1);
    }
  else
    {
      if (ou == gauche)
	{
	  x_deb_pat = x_ret + 1;
	  x_fin_pat = x_deb_pat + taillepave_x / 2 + taillepatte;
	  tx[1] = x_ret - taillepave_x;
	}
      else
	{
	  x_deb_pat = x_ret - 1;
	  x_fin_pat = x_deb_pat - taillepave_x / 2 - taillepatte;
	  tx[1] = x_ret + taillepave_x;
	};
      C2Java_drawPolygon (tx, ty, 3, col, 1);
      C2Java_drawLine (x_deb_pat, y_ret, x_fin_pat, y_ret, col, 1);
    }
}


/**
   Met en evidence d'un objet selectionne en le remplissant avec couleur (in)
*/
void
drawing::dessiner_select (typcoord milieu, int larg, int haute, int col)
{
  int x, y;

  x = milieu.x - larg;
  y = milieu.y - haute;
  C2Java_FillRectangle (x, y, 2 * larg, 2 * haute, col);
}


/**
  * dessine une boite rectangle en double epaisseur
  * ?milieu : coordonnee du milieu de la boite
  * ?larg, ?haut : hauteur et largeur de la boite.
  */
void
drawing::dessiner_rect (typcoord milieu, int larg, int haute, int col)
{
  int x, y;
  x = milieu.x - larg;
  y = milieu.y - haute;
  C2Java_drawRectangle (x, y, 2 * larg, 2 * haute, col, 2);
}


/**
  * dessine un rectangle : utilisé en cours de construction d'un rectangle
  * ?x_coin_g,y_coin_g : coordonnee du point bas-gauche
  * ?x_coin_d,y_coin_d : coordonnee du point haut-droit.
  */
void
drawing::dessiner_rect2 (int x_coin_g, int y_coin_g, int x_coin_d, int y_coin_d, int col)
{
  int l, h, x, y;

  l = abs (x_coin_d - x_coin_g);
  h = abs (y_coin_d - y_coin_g);
  if (x_coin_g < x_coin_d)
    x = x_coin_g;
  else
    x = x_coin_d;
  if (y_coin_g < y_coin_d)
    y = y_coin_g;
  else
    y = y_coin_d;
  C2Java_drawRectangle (x, y, l, h, col, 1);
}


/**
  display a data
  middle, width, height (in): coordinates of the data
  col : color of the data
*/
void
drawing::dessiner_data (typcoord middle, int width, int height, int col)
{
  int x, y;
  x = middle.x - width;
  y = middle.y - height;
  C2Java_drawRectangle (x, y, 2 * width, 2 * height, col, 1);
  C2Java_drawRectangle (x, y, 2 * (width - 1), 2 * (height - 1), col, 1);
}


/**
  * dessine des liens sous forme de segments de droite
  * ?nbcoord nombre de coordonnees dans le tableau ?tcoord.
  * Si ?epais est a vrai, il s'agit d'un bus (le trait est plus epais)
  * ?col = 
        0 : on efface, 
	COL_DEF : on affiche avec la couleur par defaut (noir),
        COL_INT : on dessine avec la couleur reservee aux entiers,
        COL_BOOL : on dessine avec la couleur reservee aux booleens
*/
void
drawing::dessiner_lien (int nbcoord, typcoord * tcoord, boolean epais, int col)
{
  int tx[maxcoord], ty[maxcoord];
  int i, ep;

  if (epais)
    ep = 2;
  else
    ep = 1;			/* epaisseur du trait */
  for (i = 0; i < nbcoord; i++)
    {
      tx[i] = tcoord[i].x;
      ty[i] = tcoord[i].y;
    }
  C2Java_drawPolyline (tx, ty, nbcoord, col, ep);
}


/**
   dessine un segment de droite
   ?(x_deb,y_deb) coordonnee de la premiere extremite du segment
   ?(x_fin,y_fin) coordonnee de la seconde extremite.
  Fonction appelée uniquement en cours de construction de lien.
*/
void
drawing::dessiner_seg (int x_deb, int y_deb, int x_fin, int y_fin, int col)
{
  C2Java_drawLine (x_deb, y_deb, x_fin, y_fin, col, 1);
}


/**
  Affiche le ?halo d'un objet rectangle
*/
void
drawing::dessiner_halo (typcoord * halo)
{
  int l, h;

  l = halo[1].x - halo[0].x;
  h = halo[1].y - halo[0].y;
  C2Java_FillRectangle (halo[0].x, halo[0].y, l, h, COL_HALO);
}


/**
  Dessine une croix dont le centre se trouve en (?x,?y)
*/
void
drawing::dessiner_croix (int x, int y)
{
  int x1, y1, x2, y2, l;

  l = 15;
  x1 = x - l;
  y1 = y - l;
  x2 = x + l;
  y2 = y + l;
  C2Java_drawLine (x1, y1, x2, y2, COL_SEL, 3);
  C2Java_drawLine (x1, y2, x2, y1, COL_SEL, 3);
}


/**
  * Affiche du texte (?chaine) dans la fenetre graphique. Le debut 
  * du texte sera en (xcoord, ycoord).
  * typeFont : 
  */
void
drawing::dessiner_text (const char *chaine, int xcoord, int ycoord, int typeFont)
{
  C2Java_drawString (chaine, xcoord, ycoord, COL_DEF, typeFont);
}

/**
  Dessine une icône en (x, y), l'icône se trouve dans le fichier image.gif 
*/
void
drawing::dessiner_image (char *image, typcoord mil, int lg, int ht)
{
  //  fprintf(stderr, " mil.x : %d, mil.y : %d, lg : %d, ht: %d \n ", mil.x, mil.y, lg, ht);
  C2Java_drawImage (image, mil.x - lg, mil.y - ht);
}
