/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

///
extern float
dist (typcoord point1, typcoord point2)
{
  db_reel z;

  z.x = point1.x - point2.x;
  z.y = point1.y - point2.y;
  return (((z.x * z.x) + (z.y * z.y)) / 100);
}


///
extern int
maxi (int x, int y)
{
  if (x >= y)
    return (x);
  else
    return (y);
}

///
extern int
mini (int x, int y)
{
  if (x <= y)
    return (x);
  else
    return (y);
}


///
extern float
fmini (float x, float y)
{
  if (x <= y)
    return (x);
  else
    return (y);
}

/**
  Returns true if the point {x_ret,y_ret} (in) belongs to the rectangle halo (in), 
  false otherwise.
*/
extern boolean
isInRectangle (int x_ret, int y_ret, typcoord * halo)
{
  return (x_ret < halo[1].x && x_ret > halo[0].x && y_ret < halo[1].y && y_ret > halo[0].y);
}


/**
  Returns true if the intersection of the 2 rectangles rect1, rect2 (in) are empty,
  false otherwise.
*/
extern boolean
isIntersectionEmpty (typcoord * rect1, typcoord * rect2)
{
  boolean empty;

  if ((rect1[0].x > rect2[1].x) || (rect1[1].x < rect2[0].x))
    empty = pk_true;
  else if ((rect1[0].y > rect2[1].y) || (rect1[1].y < rect2[0].y))
    empty = pk_true;
  else
    empty = pk_false;
  return (empty);
}

/**
  Returns true if the rectangle (x1,y1), (x2,y2) is inside the rectangle which
  representes the frame (the mother box), false otherwise.
 */
boolean
isInFrame (int x1, int y1, int x2, int y2)
{
  return (!(x1 <= decal_cadre_x) || (x2 >= abs_mil_cadre + lg_bte_stand) || (y1 <= decal_cadre_y) || (y2 >= ord_mil_cadre + ht_bte_stand));
}


/**
  Sets tcoord (out) to default value
*/
void
setDefaultCoord (typcoord * tcoord)
{
  tcoord[0].x = abs_mil_cadre;
  tcoord[0].y = mil_bte_exp_y;
  tcoord[1].x = lg_bte_exp;
  tcoord[1].y = ht_bte_exp;
}


/**
  Inverse le tableau ?tcoord qui a ?nbcoord+1 elements.
  Le tableau va de 0 à nbcoord
*/
extern void
inverser (typcoord * tcoord, int nbcoord)
{
  typcoord pcoord[1];
  int i;

  for (i = 0; i < (nbcoord + 1) / 2; i++)
    {
      pcoord[0].x = tcoord[i].x;
      pcoord[0].y = tcoord[i].y;
      tcoord[i].x = tcoord[nbcoord - i].x;
      tcoord[i].y = tcoord[nbcoord - i].y;
      tcoord[nbcoord - i].x = pcoord[0].x;
      tcoord[nbcoord - i].y = pcoord[0].y;
    }
}
