/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

/**
  Mapping des boites contenues dans le buffer ?select_buff dans la boite
  cliquee par l'utilisateur en ?(x_ret, y_ret).
  Le mapping consiste a inserer les boites selectionnees dans la boite
  designee, a laisser les connexions internes entre ces boites, a creer
  les ports necessaires sur la boite designee et a effectuer le routage
  entre les boites internes et la boite designee (si ces boites etaient 
  connectees avec des boites n'etant pas selectionnees ou avec l'exterieur).
*/
void
builder::mapper_bte (int select_buff[], int x_ret, int y_ret)
{
  int obj, i, j;
  objet_type typ_obj;
  int nouv_num_buff[TBS];	/* contient les nouveaux numeros des boites puisqu'il
				   y a recopie des s/arbres associes les btes */
  ntrees arb_bte, arb_sig;

  obj = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj);
  if ((obj != 0) && (typ_obj == typ_bte) && (select_buff[0] != 0))
    {
      arb_bte = the_graphicTablesManager->graphObj_get_tree (select_buff[0]);
      if (arb_bte != null)
	{
	  arb_sig = the_graphicalTrees->get_arb_sig (&arb_bte);
	  arb_map = the_graphicalTrees->dupl_arbre (nfather (arb_sig));	// modif 27 juin 2000
	}
    }
  if ((obj != 0) && (typ_obj == typ_bte) && (select_buff[0] != 0))
    {
      i = 0;
      while (select_buff[i] != 0)
	{
	  nouv_num_buff[i] = the_graphicTablesManager->locinconvnum (select_buff[i], 0);
	  i++;
	}
      for (j = i; j < TBS; j++)
	nouv_num_buff[j] = 0;
      if (createmapping_bte (obj, select_buff, nouv_num_buff))
	{
	  i = 0;
	  while (map_buffer[i] != 0)
	    i++;
	  map_buffer[i] = obj;
	  C2Java_refresh (pk_false);
	}
    }
}


/**
  Terminaison du mapping : fonction appelee lorsque l'utilisateur clique 
  sur le bouton "end mapping" du menu.
  ?map_buffer contient les numeros des btes dans lesquelles on a fait du
  mapping et cette fonction va construire les connexions entre ces boites.
*/
void
builder::end_mapper_bte (void)
{
  int i;

  terminer_mapping (map_buffer);
  for (i = 0; i < TBM; i++)
    map_buffer[i] = 0;
}


/**
  Mapping d'un lien logique sur un lien physique :
  Pour cela, l'utilisateur clique sur un lien et sur 2 ports appartenant
  a une boite representant un processus de communication.
  Alors, le lien est detruit et 2 liens sont etablis pour etablir la
  communication via le processus de communication.
*/
void
builder::mapper_lien (int etat, int x_ret, int y_ret)
{
  int obj;
  objet_type typ_obj;
  ntrees p_seg, port_diff, l_ports_recept, p_chem, port_recept, p_decl1, p_decl2, p1;

  switch (etat)
    {
    case 0:
      obj = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj);
      if (typ_obj != typ_seg)
	the_messageTool->error_mess (maplien1);
      else
	l_map.no_seg = obj;
      break;
    case 1:
      obj = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj);
      if (typ_obj != typ_port)
	the_messageTool->error_mess (maplien1);
      else
	l_map.no_port1 = obj;
      break;
    case 2:
      obj = pCurObjManager->reper_obj (x_ret, y_ret, &typ_obj);
      if (typ_obj != typ_port)
	the_messageTool->error_mess (maplien1);
      else
	if ((the_graphicalTrees->boite_mere (l_map.no_port1) != the_graphicalTrees->boite_mere (obj)) ||
	    (noper (the_graphicTablesManager->graphObj_get_tree (l_map.no_port1)) == noper (the_graphicTablesManager->graphObj_get_tree (obj))))
	the_messageTool->error_mess (maplien2);
      else
	{
	  l_map.no_port2 = obj;
	  p_seg = the_graphicTablesManager->acces_segment (l_map.no_seg);
	  port_diff = the_graphicalTrees->get_port_diff (p_seg);
	  l_ports_recept = LL_make_lports_emptyList ();
	  the_graphicalTrees->get_ports_recept (p_seg, l_ports_recept);
	  if (ntreearity (l_ports_recept) != 1)
	    the_messageTool->error_mess (maplien2);
	  else
	    {
	      p_chem = the_graphicalTrees->get_chemin (p_seg);
	      port_recept = the_graphicTablesManager->graphObj_get_tree ((int) nintof (nchild (1, l_ports_recept)));
	      pDisplay->display_obj (p_chem, COL_EFFACE);
	      detruire_chemin (p_chem, pk_false);
	      createmapping_lien (port_diff, port_recept, l_map.no_port1, l_map.no_port2);
	      p_decl1 = nattrntrees (port_diff, SIGNAL_ATTRNUMBER);
	      p_decl2 = nattrntrees (port_recept, SIGNAL_ATTRNUMBER);
	      p1 = the_graphicTablesManager->graphObj_get_tree (l_map.no_port1);
	      if (noper (p1) == gpkOp_porte)
		{
		  the_graphicalTrees->assoc_decl_port (p1, p_decl1);
		  the_graphicalTrees->assoc_decl_port (the_graphicTablesManager->graphObj_get_tree (l_map.no_port2), p_decl2);
		}
	      else
		{
		  the_graphicalTrees->assoc_decl_port (p1, p_decl2);
		  the_graphicalTrees->assoc_decl_port (the_graphicTablesManager->graphObj_get_tree (l_map.no_port2), p_decl1);
		}
	      pDisplay->display (pk_false);
	    }
	}
      break;
    }
}
