/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

#define COPY_ATTRIBUTEDSTREE(ATREE) ncopytree(ATREE, pk_true)
/**
  Fonction appelee pour le mapping par la fonction ""createports_mapping"
  Creation d'un port sur la boite de numero ?bte qui correspond a la boite
  dans laquelle le mapping est fait.
  ?no_port_map : le numero (ancien) du port d'une des boites mappees 
  auquel on va connecter le  nouveau port.
  ?no_port_lie : le numero (ancien) d'un des ports qui etaient connectes
  a ?no_port_map et qui  ne fait pas partie des boites mappees.
  ?rg : rang du nouveau port cree (utilise pour la creation du port)
  Le port est cree du meme cote que le port mappe (c'est-a-dire que si
  ?no_port_map est sur le bord gauche de sa boite, alors le nouveau port
  sera a gauche). Il est place en principe a la meme hauteur que le port
  mappe; POUR LE MOMENT, IL N'Y A PAS DE TEST POUR VERIFIER S'IL Y A DE 
  LA PLACE.
  Au nouveau port cree, on lui accroche 2 attributs:
      - un attribut de type "ALGO_ATTRNUMBER" constitue d'un arbre a 2 fils, le 
      premier fils etant le numero du port diffuseur (soit no_port_map,
      soit no_port_lie) et le second fils la liste de ses ports lies. 
      - un attribut de type  signal contenant la declaration (si elle existe)
      du port diffuseur. 
*/
boolean
builder::createport_mapping (int bte, int no_port_map, int no_port_lie, int rg)
{
  typcoord nouvcoord, port_coord[2], port_coord_rela[2];
  int nbcoord, no_port, no_port_diff, x, y, ou;
  boolean en_entree, intern;
  ntrees port_map, p_decl, l_ports_lies, info_map, ch;

  // printf("DEBUG entering builder::createport_mapping\n"); fflush(stdout);
  /* calcul des coordonnees du nouveau port en alignant sur le port mappe */
  port_map = the_graphicTablesManager->graphObj_get_tree (no_port_map);
  the_graphicalTrees->getCoord (port_map, &nbcoord, port_coord_rela, 1);
  the_graphicalTrees->getCoord (port_map, &nbcoord, port_coord, 0);
  if (port_coord_rela[0].x < 0)
    ou = gauche;
  else
    ou = droite;
  y = port_coord[0].y;
  pCurObjManager->externCoord2Normal (bte, &x, &y, ou);
  nouvcoord.x = x;
  nouvcoord.y = y;
  /* creation du port */
  en_entree = (noper (port_map) == gpkOp_porte);
  no_port = the_graphicalTrees->creatept_connex (bte, nouvcoord, en_entree, rg);
  /* on cree l'attribut qui va permettre de faire les routages entre le port
     mappe et le nouveau port  et le nouveau port et les ports lies */
  if (en_entree)
    no_port_diff = no_port_lie;
  else
    no_port_diff = no_port_map;
  intern = (noper (the_graphicTablesManager->graphObj_get_tree (no_port_diff)) == gpkOp_porte);
  l_ports_lies = the_graphicalTrees->get_ports_lies (no_port_diff, intern, &ch, &intern);
  info_map = LL_make_encaps (LL_make_port_atom (no_port_diff), l_ports_lies);
  nmakeattrntrees (the_graphicTablesManager->graphObj_get_tree (no_port), ALGO_ATTRNUMBER, info_map);
  p_decl = nattrntrees (the_graphicTablesManager->graphObj_get_tree (no_port_diff), SIGNAL_ATTRNUMBER);
  if (p_decl != null)
    nmakeattrntrees (the_graphicTablesManager->graphObj_get_tree (no_port), SIGNAL_ATTRNUMBER, COPY_ATTRIBUTEDSTREE (p_decl));
  // printf("DEBUG  exiting builder::createport_mapping\n"); fflush(stdout);
  return (pk_true);
}


/**
  Cette fonction est calquee sur la fonction createports_encaps
  Elle parcourt le buffer contenant les boites mappees dans ?bte et
  pour chaque port de ces boites connecte a une boite non mappee, 
  il y a un port de construit sur ?bte englobante.
  Remarque : pour un port d'une boite mappee qui n'a pas de connexion
             avec l'exterieur, il n'y aura pas de port cree sur la boite
	     englobante.
  ATTENTION : les numeros contenus dans select_buff correspondent aux
              numeros des boites du depart (avant recopie pour le mapping).
*/
boolean
builder::createports_mapping (int bte, int select_buff[])
{
  ntrees arb_port, l_ports_lies, arb_bte, ch, p_map, p_port, p_info;
  int rg, i, j, k, n, no_port, no_port_lie, no_bte_liee;
  boolean ok, trouve, b;

  ok = pk_true;
  i = 0;
  rg = 0;
  while (select_buff[i] != 0)
    {
      arb_bte = the_graphicTablesManager->graphObj_get_tree (select_buff[i]);
      if (arb_bte != null)	/* apres la selection, on a pu detruire la boite */
	{			/* donc l'arbre peut etre nul */
	  arb_port = the_graphicalTrees->getFirstPort (arb_bte);
	  while (arb_port != null)
	    {
	      no_port = the_graphicalTrees->tree_get_graphObj (arb_port);
	      l_ports_lies = the_graphicalTrees->get_ports_lies (no_port, pk_false, &ch, &b);
	      n = ntreearity (l_ports_lies);
	      j = 1;
	      trouve = pk_true;
	      while ((j <= n) && trouve)
		{
		  trouve = pk_false;
		  no_port_lie = (int) nintof (nchild (j, l_ports_lies));
		  no_bte_liee = the_graphicalTrees->tree_get_graphObj (the_graphicalTrees->boite_mere (no_port_lie));
		  k = 0;
		  while (select_buff[k] != 0)
		    {
		      if (no_bte_liee == select_buff[k])
			trouve = pk_true;
		      k++;
		    }
		  j++;
		}
	      if (!trouve && (n != 0))	/* il existe au moins une connexion avec 
					   une bte n'appartenant pas au buffer */
		{
		  if (noper (arb_port) == gpkOp_porte)
		    {
		      /* on verifie que, dans le cas ou no_port est un port 
		         d'entree, le port diffuseur n'a pas deja ete cree */
		      p_map = the_graphicTablesManager->graphObj_get_tree (bte);
		      p_port = the_graphicalTrees->getFirstPort (p_map);
		      trouve = pk_false;
		      while ((p_port != null) && !trouve)
			{
			  if ((p_info = nattrntrees (p_port, ALGO_ATTRNUMBER)) != null)
			    trouve = ((int) nintof (nchild (1, p_info)) == no_port_lie);
			  p_port = ntrees_getNext (p_port);
			}
		    }
		  if (!trouve)
		    {
		      ok = createport_mapping (bte, no_port, no_port_lie, rg);
		      rg++;
		    }
		}
	      arb_port = ntrees_getNext (arb_port);
	    }
	}
      i++;
    }
  return (ok);
}


/**
  Routage externe des boites dans lesquelles on a fait du mapping.
*/
boolean
builder::routage_externe_mapping (ntrees p_port1, ntrees p_port2)
{
  boolean b;
  int ope1, ope2;
  ntrees p_diff, p_recep;

  ope1 = noper (p_port1);
  ope2 = noper (p_port2);
  if (ope1 == ope2)		/* il y a forcemment un port externe */
    {
      if (ope1 == gpkOp_porte)
	b = rout_port_port (p_port1, p_port2, pk_false, pk_true);
      else
	b = rout_port_port (p_port1, p_port2, pk_true, pk_false);
    }
  else
    {
      if (noper (p_port1) == gpkOp_porte)
	{
	  p_recep = p_port1;
	  p_diff = p_port2;
	}
      else
	{
	  p_diff = p_port1;
	  p_recep = p_port2;
	}
      b = rout_port_port (p_recep, p_diff, pk_false, pk_false);
    }
  if (!b)
    the_messageTool->error_mess (routimp);
  return (b);
}


/**
  Creation des connexions avec la boite englobante ?bte (boite qui contient
  la boite dans laquelle est fait le mapping). Il y a de telles connexions
  lorsque un port qui a ete mappe etait connecte avec un port de sa boite 
  englobante.
  Il y a donc creation d'un port au niveau de la boite ?bte sauf s'il a 
  deja ete cree (s'il s'agit d'un port d'entree, c'est un port qui diffuse 
  vers  plusieurs  ports,; s'il s'agit d'un port de sortie, c'est que la
  boite mappee qui possède le port de sortie a été mappée plusieurs fois).
  Et le routage est lance entre ce port et le port ?no_port_map.
  ?p_decl : declaration Signal accroche au port de la boite englobante
  d'origine.
*/
boolean
builder::createlien_ext_map (int bte, int no_port_map, int rg, ntrees p_decl, ntrees l_ambig)
{
  typcoord nouvcoord, port_coord[2], port_coord_rela[2];
  int nbcoord, no_port_diff, x, y, ou, no_port_cree, no;
  boolean en_entree, ok, deja_cree, trouve;
  ntrees p_port_cree, p_port_map, p_info_map, p_attr_map, p;

  // printf("DEBUG entering builder::createlien_ext_map\n"); fflush(stdout);
  deja_cree = pk_false;
  p_port_map = the_graphicTablesManager->graphObj_get_tree (no_port_map);
  en_entree = (noper (p_port_map) == gpkOp_porte);
  p_info_map = nattrntrees (p_port_map, ALGO_ATTRNUMBER);
  if (ISNOT_NILTREE (p_info_map))
    {
      no_port_diff = (int) nintof (nchild (1, p_info_map));
      p = the_graphicalTrees->getFirstPort (the_graphicTablesManager->graphObj_get_tree (bte));
      while ((ISNOT_NILTREE (p)) && !deja_cree)
	{
	  p_attr_map = nattrntrees (p, ALGO_ATTRNUMBER);
	  if (ISNOT_NILTREE (p_attr_map))
	    {
	      no = (int) nintof (nchild (1, p_attr_map));
	      deja_cree = ((int) nintof (nchild (1, p_attr_map)) == no_port_diff);
	      if (deja_cree)
		p_port_cree = p;
	    }
	  p = ntrees_getNext (p);
	}
    }
  if (!deja_cree)		/* si le port n'est pas deja cree, on le cree */
    {				/* calcul des coordonnees du nouveau port en alignant sur le port mappe */
      the_graphicalTrees->getCoord (p_port_map, &nbcoord, port_coord_rela, 1);
      the_graphicalTrees->getCoord (p_port_map, &nbcoord, port_coord, 0);
      if (port_coord_rela[0].x < 0)
	ou = gauche;
      else
	ou = droite;
      y = port_coord[0].y;
      pCurObjManager->externCoord2Normal (bte, &x, &y, ou);
      nouvcoord.x = x;
      nouvcoord.y = y;
      no_port_cree = the_graphicalTrees->creatept_connex (bte, nouvcoord, en_entree, rg);
      p_port_cree = the_graphicTablesManager->graphObj_get_tree (no_port_cree);
      /* on cree l'attribut qui va permettre de faire les routages */
      if (ISNOT_NILTREE (p_info_map))
	nmakeattrntrees (p_port_cree, ALGO_ATTRNUMBER, COPY_ATTRIBUTEDSTREE (p_info_map));
      if (p_decl != null)
	nmakeattrntrees (p_port_cree, SIGNAL_ATTRNUMBER, COPY_ATTRIBUTEDSTREE (p_decl));
      else /* si le port d'interface n'a pas de declaration, on prend celle du port mappe */
      if (ISNOT_NILTREE ((p_decl = nattrntrees (p_port_map, SIGNAL_ATTRNUMBER))))
	nmakeattrntrees (p_port_cree, SIGNAL_ATTRNUMBER, COPY_ATTRIBUTEDSTREE (p_decl));
    }
  if (en_entree)		/* on lance le routage */
    {
      ok = routage_externe_mapping (p_port_map, p_port_cree);
    }
  else
    {
      no_port_cree = the_graphicalTrees->tree_get_graphObj (p_port_cree);
      p = nchild (1, l_ambig);
      trouve = pk_false;
      while ((ISNOT_NILTREE (p)) && !trouve)
	{
	  trouve = ((int) nintof (nchild (1, p)) == no_port_cree);
	  if (!trouve)
	    p = ntrees_getNext (p);
	}
      /*      fprintf(stderr, "FONCTION createlien_ext_map -- trouve: %d \n", trouve); */
      if (!trouve)
	{
	  p = LL_make_lports_list (LL_make_port_atom (no_port_cree));
	  ntreeList_appendLast (p, LL_make_port_atom (no_port_map));
	  ntreeList_appendLast (l_ambig, p);
	}
      else
	ntreeList_appendLast (p, LL_make_port_atom (no_port_map));
      /*      fprintf(stderr, "FONCTION createlien_ext_map -- trouve: %d %d %d\n", trouve, no_port_cree, no_port_map); */
    }
  // printf("DEBUG  exiting builder::createlien_ext_map\n"); fflush(stdout);
  return (ok);
}


/**
  Routage a l'interieur d'une boite dans laquelle on a fait un mapping.
*/
boolean
builder::routage_interne_mapping (int bte, ntrees l_bte_enc)
{
  ntrees arb_gra, p_port, arb_port_diff, arb_no_port_diff, arb_port_lie, arb_no_port_lie, l_no_ports_lies, p_mapping, p_rout_b, p_rout_a, p;
  boolean ok, p_ext_b;
  int no_bte_diff, no_port_diff, no_bte_lie, no_port_lie;

  arb_gra = the_graphicTablesManager->graphObj_get_tree (bte);
  p_port = the_graphicalTrees->getFirstPort (arb_gra);
  pCurObjManager->descendre (bte, pk_true);
  while (p_port != null)
    {
      p_mapping = nattrntrees (p_port, ALGO_ATTRNUMBER);
      if (p_mapping != null)
	{
	  arb_no_port_diff = nchild (1, p_mapping);
	  no_port_diff = (int) nintof (arb_no_port_diff);
	  arb_port_diff = the_graphicTablesManager->graphObj_get_tree (no_port_diff);
	  l_no_ports_lies = nchild (2, p_mapping);
	  no_bte_diff = the_graphicalTrees->no_boite_mere (arb_port_diff);
	  arb_no_port_lie = nchild (1, l_no_ports_lies);
	  while ((arb_no_port_lie != null))
	    {
	      no_port_lie = (int) nintof (arb_no_port_lie);
	      arb_port_lie = the_graphicTablesManager->graphObj_get_tree (no_port_lie);
	      no_bte_lie = the_graphicalTrees->no_boite_mere (arb_port_lie);
	      if (the_graphicalTrees->is_in_list (no_bte_lie, l_bte_enc))
		{
		  if (noper (p_port) == noper (arb_port_lie))
		    {		/*cas ou le port diffuseur n'est pas dans les btes mappees */
		      p_rout_b = p_port;
		      p_ext_b = pk_true;
		      p_rout_a = the_graphicTablesManager->graphObj_get_tree (the_graphicTablesManager->locinconvnum (no_port_lie, 0));
		      ok = rout_port_port (p_rout_a, p_rout_b, pk_false, p_ext_b);
		    }
		}
	      arb_no_port_lie = ntrees_getNext (arb_no_port_lie);
	    }
	  if (noper (p_port) == gpkOp_ports)
	    {
	      p = the_graphicTablesManager->graphObj_get_tree (the_graphicTablesManager->locinconvnum (no_port_diff, 0));
	      ok = rout_port_port (p_port, p, pk_true, pk_false);
	    }
	}
      p_port = ntrees_getNext (p_port);
    }
  pCurObjManager->remonter (pk_true);
  return (pk_true);		/* A FAIRE : envoyer un message lorsqu'on n'a pas pu tout router */
}



/**
  Cette fonction est utilisee pour faire le routage externe entre boites 
  mappees :
  Elle parcourt le buffer ?map_buff (en sautant la boite ?i qui est la
  boite courante pour trouver tous les ports d'entrée auquel est accroche 
  un attribut de type "ALGO_ATTRNUMBER" et dans lequel le 1ier fils est egal a  
  ?no_port_diff (qui est forcemment un port de sortie).

  Pour que 2 btes de mapping soient connectees, il faut :
     - qu'elles aient chacune un port qui a les memes informations 
       dans l'attribut "ALGO_ATTRNUMBER":  meme port diffuseur et 1 port recepteur 
       identique) 
     - que les 2 ports soient de types differents (c'est-a-dire qu'on ait 
       forcemment un port d'entree et un port de sortie). Si les ports 
       sont de même type, c'est qu'il s'agit d'une même boîte mappée 
       dans 2 boîtes différentes; dans ce cas, on affiche un warning
       et c'est l'utilisateur qui devra créer lui-même sa connexion,
       vu qu'il y a ambiguité.
  Rend une liste des numéros de ports concernes, s'il n'y a pas 
  d'ambiguité possible, "null" sinon ou si on a trouve aucun port.
*/
void
builder::createliens_inter_map (ntrees p_port_diff, int no_port_diff, int map_buff[], int i, int *no)
{
  ntrees p_port, p_mapping, p_bte, l_rout, l_ambig, p;
  int no_port, j, bte;
  boolean ambiguous, ok;

  j = 0;
  ambiguous = pk_false;
  l_rout = LL_make_lports_emptyList ();
  while (map_buff[j] != 0)
    {
      if (j != i)		/* on saute la boite courante */
	{
	  bte = map_buff[j];
	  p_bte = the_graphicTablesManager->graphObj_get_tree (bte);
	  if (p_bte != null)	/* la boite peut avoir ete detruite avant la fin du mapping */
	    {			/* donc on teste si l'arbre n'est pas egal a "null" */
	      p_port = the_graphicalTrees->getFirstPort (p_bte);
	      while (p_port != null)
		{
		  p_mapping = nattrntrees (p_port, ALGO_ATTRNUMBER);
		  if (p_mapping != null)
		    {
		      no_port = (int) nintof (nchild (1, p_mapping));
		      /*                      fprintf(stderr,"FONCTION createliens_inter_map --  %d,%d,%d,%d\n", bte, no_port_diff, no_port,
		         (int)nintof(nchild(1, nchild(2,p_mapping)))); */
		      if ((no_port == no_port_diff) && (noper (p_port) == gpkOp_porte))
			{
			  ntreeList_insertFirst (LL_make_port_atom (the_graphicalTrees->tree_get_graphObj (p_port)), l_rout);
			  ndestroyattr (p_port, ALGO_ATTRNUMBER);	/*on detruit l'attribut du port recepteur */
			}
		      else if (no_port == no_port_diff)
			{	/* Ambiguité: 2 ports de sortie peuvent alimenter le même port */
			  /*                      fprintf(stderr,"FONCTION createliens_inter_map --  ambiguite possible %d \n", no_port); */
			  ambiguous = pk_true;
			  nettoyage_attrib (p_port);
			  ntreeList_insertFirst (LL_make_port_atom (the_graphicalTrees->tree_get_graphObj (p_port)), l_rout);
			}
		    }
		  p_port = ntrees_getNext (p_port);
		}
	    }
	}
      j++;
    }
  if (!ambiguous)
    {				/* s'il n'y a pas d'ambiguité, on lance le routage */
      p = nchild (1, l_rout);
      while (ISNOT_NILTREE (p))
	{
	  p_port = the_graphicTablesManager->graphObj_get_tree ((int) nintof (p));
	  ok = routage_externe_mapping (p_port_diff, p_port);
	  p = ntrees_getNext (p);
	}
    }
  else
    {
      ntreeList_insertFirst (LL_make_port_atom (the_graphicalTrees->tree_get_graphObj (p_port_diff)), l_rout);
      l_ambig = LL_make_lports_list (l_rout);
      pDisplay->display_ports_ambig (l_ambig, *no);
      (*no)++;
    }
}

/**
  Nettoyage de l'attribut "ALGO_ATTRNUMBER" (lié au mapping) accroché au port ?p_port
  Rappel : 1 ier fils de l'attribut : le port diffuseur
           2 ième fils              : la liste des ports recepteurs.
  Le nettoyage consiste à enlever de la liste des ports diffuseurs tous les
  ports qui ne sont pas des ports externes. Cette fonction est appelée par
  "createliens_inter_map" après que les liens inter-boites ont été créés.
*/
void
builder::nettoyage_attrib (ntrees p_port)
{
  ntrees p_attr, p_list, port_recep, p_bte, frere;
  int no_port_diff, no_port_recep;

  p_attr = nattrntrees (p_port, ALGO_ATTRNUMBER);
  no_port_diff = (int) nintof (nchild (1, p_attr));
  p_list = nchild (2, p_attr);
  port_recep = nchild (1, p_list);
  while (ISNOT_NILTREE (port_recep))
    {
      no_port_recep = (int) nintof (port_recep);
      p_bte = the_graphicalTrees->boite_mere (no_port_diff);
      if (!the_graphicalTrees->isExternPort (p_bte, the_graphicTablesManager->graphObj_get_tree (no_port_recep)))
	{
	  frere = ntrees_getNext (port_recep);
	  ntreesDestroy (port_recep);
	  port_recep = frere;
	}
      else
	port_recep = ntrees_getNext (port_recep);
    }
  if (ntreearity (p_list) == 0)
    ndestroyattr (p_port, ALGO_ATTRNUMBER);
}

/**
  Cette fonction fait le mapping des boites contenues dans ?nouv_select_buf 
  dans la boite ?bte.
  ?select_buff contient les numeros des boites selectionnees par 
  l'utilisateur et ?nouv_select_buf contient les numeros des copies de
  ces boites.
*/
boolean
builder::createmapping_bte (int bte, int select_buff[], int nouv_num_buff[])
{
  ntrees arb_gra, p, p_sig, p_sig_engl;
  ntrees l_bte_map;
  boolean ok;
  int i;

  if (createports_mapping (bte, select_buff))
    {
      l_bte_map = LL_make_design_emptyList ();
      p = the_graphicTablesManager->graphObj_get_tree (bte);
      p_sig_engl = the_graphicalTrees->get_arb_sig (&p);	/* arbre Signal correspondant a la boite */
      i = 0;			/* dans laquelle on fait le mapping */
      if (noper (p_sig_engl) != oplfeed)
	{
	  p = LL_make_lfeed_emptyList ();
	  STree_moveAttribute (p_sig_engl, p, GRAPHI_ATTRNUMBER);
	  STree_replaceSon (p_sig_engl, p, 0);
	  p_sig_engl = p;
	}
      while (nouv_num_buff[i] != 0)
	{			/* on insere toutes les boites selectionnees dans la boite ?bte */
	  arb_gra = the_graphicTablesManager->graphObj_get_tree (nouv_num_buff[i]);
	  if (arb_gra != null)
	    {
	      p_sig = the_graphicalTrees->get_arb_sig (&arb_gra);
	      ntreesDestroy (p_sig);
	      ntreeList_insertFirst (p_sig, p_sig_engl);
	      /* on construit un arbre contenant la liste des btes mappees */
	      ntreeList_insertFirst (LL_make_design_atom (select_buff[i]), l_bte_map);
	    }
	  i++;
	}
      pCurObjManager->setCurrentObjectsList ();	/*on met a jour la liste des objets designables */
      detr_liens_mapping (bte, l_bte_map);
      /* on fait le routage a l'interieur de la boite englobante */
      ok = routage_interne_mapping (bte, l_bte_map);
    }
  else
    ok = pk_false;
  return (ok);
}


/**
  Cette fonction est appelee lorsque l'utilisateur clique sur le bouton
  "refresh" et qu'on est dans l'etat de mapping. Elle termine le mapping,
  c'est-a-dire qu'elle va construire les liens entre les boites contenant
  le mapping et les liens entre les boites contenant le mapping et leur 
  boite englobante.
  ?map_buf contient les numeros des btes dans lesquelles on a fait le
  mapping.
*/
void
builder::terminer_mapping (int map_buff[])
{
  int i, rg, no_port_lie, no_port_diff, no_port, no_bte_engl, no;
  boolean b, ok;
  ntrees p_bte, p_port, p_mapping, p_port_lie, p, p_decl, l_ambig, frere_droit;

  i = 0;
  no = 1;
  ok = pk_true;
  b = pk_true;
  l_ambig = LL_make_lports_emptyList ();
  while (map_buff[i] != 0)
    {
      p_bte = the_graphicTablesManager->graphObj_get_tree (map_buff[i]);
      if (p_bte != null)	/* on peut detruire une bte avant de terminer le mapping, */
	{			/* ce qui fait qu'il faut tester si le numero correspond bien à une bte */
	  p_port = the_graphicalTrees->getFirstPort (p_bte);
	  while (p_port != null)
	    {
	      rg = 0;
	      no_port = the_graphicalTrees->tree_get_graphObj (p_port);
	      p_mapping = nattrntrees (p_port, ALGO_ATTRNUMBER);
	      if (p_mapping != null)
		{
		  no_port_diff = (int) nintof (nchild (1, p_mapping));
		  no_bte_engl = the_graphicalTrees->tree_get_graphObj (pCurObjManager->get_bte_cou_engl ());
		  if (noper (p_port) == gpkOp_ports)
		    {		/* on s'occupe des liens entre btes contenant le mapping */
		      createliens_inter_map (p_port, no_port_diff, map_buff, i, &no);
		      /*on s'occupe des liens en sortie avec la boite englobante */
		      p_port_lie = nchild (1, nchild (2, p_mapping));
		      while (p_port_lie != null)
			{
			  no_port_lie = (int) nintof (p_port_lie);
			  p = the_graphicTablesManager->graphObj_get_tree (no_port_lie);
			  if (the_graphicalTrees->isExternPort (the_graphicalTrees->boite_mere (no_port_diff), p))
			    {
			      p_decl = nattrntrees (p, SIGNAL_ATTRNUMBER);
			      createlien_ext_map (no_bte_engl, no_port, rg, p_decl, l_ambig);
			      rg++;
			    }
			  p_port_lie = ntrees_getNext (p_port_lie);
			}

		    }
		  else
		    {		/* on s'occupe des liens en entree avec la boite englobante */
		      p_port_lie = nchild (1, nchild (2, p_mapping));
		      p = the_graphicTablesManager->graphObj_get_tree (no_port_diff);
		      if (noper (p) == gpkOp_porte)
			{
			  p_decl = nattrntrees (p, SIGNAL_ATTRNUMBER);
			  b = createlien_ext_map (no_bte_engl, no_port, rg, p_decl, l_ambig);
			  ok = ok && b;
			  rg++;
			}
		    }
		}
	      p_port = ntrees_getNext (p_port);
	    }
	}
      i++;
    }
  p = nchild (1, l_ambig);
  while (ISNOT_NILTREE (p))
    {
      frere_droit = ntrees_getNext (p);
      if (ntreearity (p) == 2)
	{
	  b = routage_externe_mapping (the_graphicTablesManager->graphObj_get_tree ((int) nintof (nchild (1, p))),
				       the_graphicTablesManager->graphObj_get_tree ((int) nintof (nchild (2, p))));
	  ntreesDestroy (p);
	}
      p = frere_droit;
      ok = ok && b;
    }
  if (ntreearity (l_ambig) != 0)
    pDisplay->display_ports_ambig (l_ambig, no);
  C2Java_refresh (pk_false);
  if (!ok)
    the_messageTool->error_mess (routimp);
}


/**
  Cette fonction est appelee lors du mapping d'un lien logique sur un lien 
  physique. Elle effectue le routage entre :
     - le port diffuseur ?port_diff du lien logique et un des ports du 
     canal de communication ?no_port1 ou ?no_port2
     - le port recepteur ?port_recept et l'autre port du canal.
*/
void
builder::createmapping_lien (ntrees port_diff, ntrees port_recept, int no_port1, int no_port2)
{
  boolean ok1, ok2;
  ntrees port1, port2;

  port1 = the_graphicTablesManager->graphObj_get_tree (no_port1);
  port2 = the_graphicTablesManager->graphObj_get_tree (no_port2);
  if (noper (port1) == gpkOp_porte)
    {
      ok1 = rout_port_port (port1, port_diff, pk_false, pk_false);
      ok2 = rout_port_port (port_recept, port2, pk_false, pk_false);
    }
  else
    {
      ok1 = rout_port_port (port_recept, port1, pk_false, pk_false);
      ok2 = rout_port_port (port2, port_diff, pk_false, pk_false);
    }
  if (!ok1 || !ok2)
    the_messageTool->error_mess (routimp);
}


/**
  Fonction appelée par la fonction "clean_embedded_box"
  ?p_bte est une boite de même niveau que la boite dont on veut faire
  une compilation "embedded". En principe, ?p_bte est une boîte modélisant
  un module de communication.
  Cette fonction détruit tous les ports de ?p_bte qui ne sont pas 
  connectés à un port de ?embedded_bte.
*/
void
builder::detr_ports_inutiles (ntrees p_bte, int embedded_bte)
{
  ntrees p_port, p_port_lie, l_ports_lies, chem;
  boolean is_connect, b;
  int no_port;

  p_port = the_graphicalTrees->getFirstPort (p_bte);
  while (ISNOT_NILTREE (p_port))
    {
      is_connect = pk_false;
      no_port = the_graphicalTrees->tree_get_graphObj (p_port);
      p_port = ntrees_getNext (p_port);
      l_ports_lies = the_graphicalTrees->get_ports_lies (no_port, pk_false, &chem, &b);
      if (ISNOT_NILTREE (l_ports_lies))
	{
	  p_port_lie = nchild (1, l_ports_lies);
	  while (ISNOT_NILTREE (p_port_lie) && !is_connect)
	    {
	      is_connect = (the_graphicalTrees->tree_get_graphObj (the_graphicalTrees->boite_mere ((int) nintof (p_port_lie))) == embedded_bte);
	      p_port_lie = ntrees_getNext (p_port_lie);
	    }
	}
      if (!is_connect)
	detruire_port (no_port, typ_port);
    }
}


/**
  Fonction appelée par la fonction "clean_embedded_box"
  ?p_bte est une boite de même niveau que la boite dont on veut faire
  une compilation "embedded". En principe, ?p_bte est une boîte modélisant
  un module de communication.
  Cette fonction détruit toutes les boites internes de ?p_bte qui n'ont
  que des connexions internes (pas de connexions avec ?p_bte); ensuite,
  elle détruit les ports des boites internes restantes qui ne sont plus
  connectés.
  rend false si ?p_bte n'a pas la représentation adéquate pour modéliser
  un canal de communication (dans ce cas, il n'y aura pas de compilation
  embarquée après).
*/
boolean
builder::detr_btes_internes (ntrees p_bte, ntrees p_proc)
{
  ntrees p_bte_int, p_sig_int, p_comp, p_port, p_decl_sig, p_bte_proc, p_port_proc, p, p_proc_decl;
  int no, no_int;
  boolean en_entree, b, fini;

  fini = pk_false;
  p_comp = the_graphicalTrees->get_arb_sig (&p_bte);
  if (noper (p_comp) == oplfeed)
    {
      no = the_graphicalTrees->tree_get_graphObj (p_bte);
      p_sig_int = nchild (1, p_comp);
      while (ISNOT_NILTREE (p_sig_int) && !fini)
	{
	  p_bte_int = nattrntrees (p_sig_int, GRAPHI_ATTRNUMBER);
	  if (ISNOT_NILTREE (p_bte_int))
	    {
	      no_int = the_graphicalTrees->tree_get_graphObj (p_bte_int);
	      /* on positionne le frère droit avant la destruction (possible)   */
	      p_sig_int = ntrees_getNext (p_sig_int);	/* de l'arbre courant p_sig_int */
	      if (!the_graphicalTrees->is_btes_connectees (no_int, no))
		detruire_bte (no_int, pk_false);	// false: destroy not only detach
	    }
	  else
	    fini = pk_true;
	}
      if (!fini)
	{
	  p_sig_int = nchild (1, p_comp);
	  while (ISNOT_NILTREE (p_sig_int))
	    {			/* pour chaque bte interne restante, on détruit les ports non connectés,
				   et si cette bte correspond à un appel de processus, on va détruire les
				   ports correspondant dans la déclaration du processus */
	      p_bte_int = nattrntrees (p_sig_int, GRAPHI_ATTRNUMBER);
	      SET_NILTREE (p_bte_proc);
	      if (noper (p_sig_int) == opinst)
		{
		  p_proc_decl = the_graphicalTrees->inser_decl_proc (p_proc, p_proc, nchild (1, p_sig_int), &b);
		  if (ISNOT_NILTREE (p_proc_decl))
		    p_bte_proc = the_graphicTablesManager->graphObj_get_tree (the_graphicalTrees->no_boite_proc (p_proc_decl));
		}
	      p_port = the_graphicalTrees->getFirstPort (p_bte_int);
	      while (ISNOT_NILTREE (p_port))
		{
		  p = p_port;
		  p_decl_sig = nattrntrees (p, SIGNAL_ATTRNUMBER);
		  en_entree = (noper (p) == gpkOp_porte);
		  p_port = ntrees_getNext (p_port);
		  if (!the_graphicalTrees->is_connected_port (p, pk_false))
		    {
		      detruire_port (the_graphicalTrees->tree_get_graphObj (p), typ_port);
		      if (ISNOT_NILTREE ((p_decl_sig)) && (ISNOT_NILTREE (p_bte_proc)))
			{	/* on détruit le port de même nom dans la déclaration du processus */
			  p_port_proc = the_graphicalTrees->getPortWithDecl (p_decl_sig, p_bte_proc, en_entree);
			  if (ISNOT_NILTREE (p_port_proc))
			    detruire_port (the_graphicalTrees->tree_get_graphObj (p_port_proc), typ_port);
			}
		    }
		}
	      p_sig_int = ntrees_getNext (p_sig_int);
	    }
	}
    }
  return (!fini);
}

/**
  Fonction utilisee pour la repartion de programmes (l'utilisateur
  a clique sur le bouton "SET ARCHI"). Elle est appelee par la fonction 
  "traiter_repartition".
  Cette fonction rajoute un port a la declaration du processus ?p_proc, 
  soit en entree, soit en sortie suivant ?en_entree et lui associe un
  nom et un type ?p_decl. 
  Ce port correspond a un signal d'interface cree par le compilo;
  le processus ?p_proc correspond a la description d'un processeur.
*/
void
builder::add_signal_proc_inte (ntrees p_proc, ntrees p_decl, boolean en_entree)
{
  ntrees p_boite;
  int no_boite, no_port, nbcoord, y, rg, b_inf, b_sup, sens;
  typcoord coord[2], pcoord;

  // printf("DEBUG entering builder::add_signal_proc_inte\n"); fflush(stdout);
  no_boite = the_graphicalTrees->no_boite_proc (p_proc);
  p_boite = the_graphicTablesManager->graphObj_get_tree (no_boite);
  the_graphicalTrees->getCoord (p_boite, &nbcoord, coord, 0);
  y = coord[0].y - coord[1].y + 20;
  if (en_entree)
    {
      sens = gauche;
      pcoord.x = coord[0].x - coord[1].x;
    }
  else
    {
      sens = droite;
      pcoord.x = coord[0].x + coord[1].x;
    }
  b_inf = coord[0].y - coord[1].y;
  b_sup = coord[0].y + coord[1].y;
  if (the_graphicalTrees->place_trou_port (no_boite, sens, b_inf, b_sup, &y, &rg))
    {
      pcoord.y = y;
      no_port = the_graphicalTrees->creatept_connex (no_boite, pcoord, en_entree, rg);
      the_graphicalTrees->assoc_decl_port (the_graphicTablesManager->graphObj_get_tree (no_port), p_decl);
    }
  else
    the_messageTool->error_mess (crportimp);
  // printf("DEBUG  exiting builder::add_signal_proc_inte\n"); fflush(stdout);
}
