/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

#define var_etat 1
#define var_cond 2
#define var_mesu 3
#define var_sort 4
#define var_config 5

#define COPY_ATTRIBUTEDSTREE(ATREE) ncopytree(ATREE, pk_true)
///
static ptMessage the_messageTool;	/// gestionnaire de messages
static ptGraph_text pGraph_text;
///
static ptGraphicTrees pGraph_arb;

/// static ptreg reg;

static int val_automate;
///
static int compteur;		/// compteur pour generer des chaines uniques INTERM1, INTERM2,...

static ntrees /*p_codage, */ p_vect;
///
static ptrepstr chaine0, chaine1, chaine2, chaine_X, chaine_X_etend, chaine_Y,
  chaine_U, chaine_cond, chaine_INTERM, chaine_Tick, chaine_TTick, chaine_vt;

///
static int nb_mesu, nb_etat, nb_cond, nb_sort; /** variables globales au module :
correspondent au nombre d'entrees incontrolables, nombre d'etats, nombre de conditions
et nombre de sorties presents dans le fichier engendre par Sigali "fich.sim" ou
fich est le nom du programme Signal. */

static int taille_vect_etat, taille_vect_cond, taille_vect_mesu; /** variables
correspondant a la taille des vecteurs construits (par exemple, si le nombre
d'etats est 0, la taille du vecteur d'etats est 1 */

static char nom_fich_sigali[TBUF];
///
static char *nom_fich;


/**==============================================================
  Cree et rend le sous-arbre :
                 const when nom_sig 
  ou la constante const est representee par sa chaine ?ch_const
 ===============================================================*/
static ntrees
creer_when (ptrepstr ch_const, ptrepstr nom_sig)
{
  ntrees p;

  p = LL_make_extr_tree (LL_make_ent_atom (ch_const), LL_make_nom_atom (nom_sig));
  return (p);
}


/**=============================================================
  Cree et rend le sous-arbre :
                 2 when not nom_sig  (codage de la valeur faux)
 ==============================================================*/
static ntrees
creer_notwhen (ptrepstr nom_sig)
{
  ntrees p;

  p = LL_make_extr_tree (LL_make_ent_atom (chaine2), LL_make_non_tree (LL_make_nom_atom (nom_sig)));
  return (p);
}


/**========================================================================
  Cree et rend l'arbre :
      (1 when nom_sig) default (2 when not nom_sig) default (0 when Tick)
  =======================================================================*/
static ntrees
creer_default (ptrepstr nom_sig)
{
  ntrees p;

  p = LL_make_mele_tree (LL_make_mele_tree (creer_when (chaine1, nom_sig), creer_notwhen (nom_sig)), creer_when (chaine0, chaine_Tick));
  return (p);
}


/**=======================================================================
  Cette fonction cree et rend la chaine (de type ptrepstr) correspondant
  a l'entier ?i
  =======================================================================*/
static ptrepstr
cr_chaine_ent (int i)
{
  //  ptrepstr ch;
  //  boolean b;

  /*  strreset(reg);  b = strconcinttoreg(reg, i);
     strstore(reg, &ch);  return(ch);
   */
  return integer2repstr (i);
}


/**=================================================================
  Cree et rend l'arbre affectation :
           fils_g := fils_d
  Attention, ?fils_g est une chaine et ?fils_d est un arbre.
  =================================================================*/
static ntrees
creer_aff (ptrepstr fils_g, ntrees fils_d)
{
  ntrees p;
  p = LL_make_defsig_tree (LL_make_nom_atom (fils_g), fils_d);
  pGraph_text->addMark (p);
  return (p);
}


/**==============================================================================
  Cette fonction cree l'affectation intermediaire :
        INTERM := 1 when nom_sig default 2 when not nom_sig default 0 when Tick
  l'insere dans ?p_expr et rend la chaine unique INTERM
  ============================================================================*/
static ptrepstr
creer_aff_interm (ptrepstr nom_sig, ntrees p_expr)
{
  ptrepstr ch_num, ch_uniq;
  ntrees p_default;

  /* creation d'un nom unique */
  ch_num = cr_chaine_ent (compteur);
  compteur++;
  /*  strreset(reg);  strconcstr(reg, chaine_INTERM); 
     strconcstr(reg, ch_num); strstore(reg, &ch_uniq);  */
  strconc2str (chaine_INTERM, ch_num, pk_false, &ch_uniq);

  p_default = creer_default (nom_sig);
  ntreeList_appendLast (p_expr, creer_aff (ch_uniq, p_default));
  return (ch_uniq);
}




/**=======================================================================
  Cette fonction cree et insere dans l'arbre Signal ?p_expr le sous-arbre 
  Signal qui correspond aux codages des etats ou des conditions 
  suivant la valeur de ?ch_cod :
         INTERM1 := 1 when nom_sig default 2 when not nom_sig default 0 when Tick
	 INTERM2 :=  1 when ...
	 INTERM3 ...
         cod_X := [[0]: INTERM1
	           [1]: INTERM2
		   ...
		  ]
   ou    cod_cond := ....
   ?nb_codage : numero du codage
   Si ?nb_codage vaut 0, on engendre (par exemple pour les etats):
         cod_X := [[0]: 0 when Tick ou Tick est une horloge engendree par
         le resolveur.
   ?nb_codage ne vaut 0 que s'il n'y a rien dans le fichier concernant
   ce codage; dans ce cas-la, on cree un codage avec "0 when Tick"
  =======================================================================*/
static void
creer_codage_etat (ptrepstr ch_cod, ptrepstr nom_sig, int nb_codage, ntrees p_expr)
{
  ntrees p_elem;
  ptrepstr chaine_indice, chaine_uniq;

  if (nb_codage == 0)
    {
      p_vect = LL_make_levect_emptyList ();
      ntreeList_appendLast (p_expr, creer_aff (ch_cod, p_vect));
    }
  chaine_uniq = creer_aff_interm (nom_sig, p_expr);
  chaine_indice = cr_chaine_ent (nb_codage);
  p_elem = LL_make_rest_tree (LL_make_levect_list (LL_make_ent_atom (chaine_indice)), LL_make_nom_atom (chaine_uniq));
  ntreeList_appendLast (p_vect, p_elem);
}

/**=======================================================================
  Cette fonction cree et insere dans l'arbre Signal ?p_expr le sous-arbre 
  Signal qui correspond au codage de l'etat ou d'une condition (suivant 
  la valeur de ?ch_cod) lorsqu'il n'y a rien dans le fichier concernant 
  ce codage.
  Dans ce cas, on engendre (par exemple pour les etats):
     cod_X := 0 when Tick ou Tick est une horloge engendree par le resolveur.
  =======================================================================*/
static void
creer_codage_etat_vide (ptrepstr ch_cod, ptrepstr nom_sig, ntrees p_expr)
{
  ntrees p_elem, p_when;

  p_vect = LL_make_levect_emptyList ();
  ntreeList_appendLast (p_expr, creer_aff (ch_cod, p_vect));
  p_when = creer_when (chaine0, chaine_Tick);
  p_elem = LL_make_rest_tree (LL_make_levect_list (LL_make_ent_atom (chaine0)), p_when);
  ntreeList_appendLast (p_vect, p_elem);

}


/**=======================================================================
  Cette fonction cree et insere dans l'arbre Signal ?p_expr le sous-arbre 
  Signal qui correspond aux codages des entrees incontrolables :
         nom_sig := true when (cod_Y[1] = 1) default 
	            false when (cod_Y[1] = -1) default false
   ?nb_ent : numero de l'entree incontrolable
  =======================================================================*/
static void
creer_codage_incontrolable (ptrepstr nom_sig, int nb_ent, ntrees p_expr)
{
  ntrees p_default, p_elem, p_ega, p_when1, p_when2;
  ptrepstr chaine_indice;

  chaine_indice = cr_chaine_ent (nb_ent);
  p_elem = LL_make_elem_tree (LL_make_nom_atom (chaine_Y), LL_make_nom_atom (chaine_indice));
  p_ega = LL_make_egal_tree (p_elem, LL_make_nom_atom (chaine1));
  p_when1 = LL_make_extr_tree (LL_make_true_atom (), p_ega);
  p_ega = LL_make_egal_tree (COPY_ATTRIBUTEDSTREE (p_elem), LL_make_nega_tree (LL_make_ent_atom (chaine1)));
  p_when2 = LL_make_extr_tree (LL_make_false_atom (), p_ega);
  p_default = LL_make_mele_tree (p_when1, p_when2);
  p_default = LL_make_mele_tree (p_default, LL_make_false_atom ());
  ntreeList_appendLast (p_expr, creer_aff (nom_sig, p_default));
}


/**=======================================================================
  Cette fonction cree et insere dans l'arbre Signal ?p_expr le sous-arbre 
  Signal qui correspond aux codages des sorties du resolveur :
         nom_sig := true when (cod_U[1] = 1) default 
	            false when (cod_U[1] = -1) default false
   ?nb_sort : numero de la sortie
  =======================================================================*/
static void
creer_codage_sort (ptrepstr nom_sig, int nb_sort, ntrees p_expr)
{
  ntrees p_default, p_elem, p_ega, p_when1, p_when2;
  ptrepstr chaine_indice;

  chaine_indice = cr_chaine_ent (nb_sort);
  p_elem = LL_make_elem_tree (LL_make_nom_atom (chaine_U), LL_make_nom_atom (chaine_indice));
  p_ega = LL_make_egal_tree (p_elem, LL_make_nom_atom (chaine1));
  p_when1 = LL_make_extr_tree (LL_make_true_atom (), p_ega);
  p_ega = LL_make_egal_tree (COPY_ATTRIBUTEDSTREE (p_elem), LL_make_nega_tree (LL_make_ent_atom (chaine1)));
  p_when2 = LL_make_extr_tree (LL_make_false_atom (), p_ega);
  p_default = LL_make_mele_tree (p_when1, p_when2);
  p_default = LL_make_mele_tree (p_default, LL_make_false_atom ());
  ntreeList_appendLast (p_expr, creer_aff (nom_sig, p_default));
}


/**==========================================================================
  On engendre l'appel au resolveur:   RESOLVER{ncond, nx, nu, ny} 	  |
  ?p_exp : arbre ou on insere les sous-arbres crees.
  Si nb_cond ou nb_etat est a zero, on engendre le codage quand meme avec 0:
      cod_X := [[0]: 0 when Tick] 
  ==========================================================================*/
static void
creer_appel_resolv (ntrees p_expr)
{
  ptrepstr ch_nom_appel;
  ntrees p_nom_appel, p_appel, p_nb, l_param;

  /* construction de : RESOLVEUR{}  */
  ch_nom_appel = string2repstr ("RESOLVER");
  p_nom_appel = LL_make_nom_atom (ch_nom_appel);
  l_param = LL_make_lepara_emptyList ();
  if (nb_etat == 0)
    {
      creer_codage_etat_vide (chaine_X, nilstr, p_expr);
      taille_vect_etat = 1;
    }
  else
    {
      nb_etat--;
      taille_vect_etat = nb_etat;
    }
  if (nb_cond == 0)
    {
      creer_codage_etat_vide (chaine_cond, nilstr, p_expr);
      taille_vect_cond = 1;
    }
  else
    {
      nb_cond--;
      taille_vect_cond = nb_cond;
    }
  if (nb_mesu == 0)
    {
      taille_vect_mesu = 1;
    }
  else
    {
      nb_mesu--;
      taille_vect_mesu = nb_mesu;
    }
  p_nb = LL_make_nom_atom (cr_chaine_ent (taille_vect_cond));
  ntreeList_appendLast (l_param, p_nb);
  p_nb = LL_make_nom_atom (cr_chaine_ent (taille_vect_etat));
  ntreeList_appendLast (l_param, p_nb);
  nb_sort--;
  p_nb = LL_make_nom_atom (cr_chaine_ent (nb_sort));
  ntreeList_appendLast (l_param, p_nb);
  p_nb = LL_make_nom_atom (cr_chaine_ent (taille_vect_mesu));
  ntreeList_appendLast (l_param, p_nb);
  p_appel = LL_make_inst_tree (p_nom_appel, l_param);
  pGraph_text->addMark (p_appel);
  ntreeList_appendLast (p_expr, p_appel);
}


/**=====================================================================
  Cette fonction insère les entrées et sorties du processus qu'on veut
  controler en tant que signaux locaux du processus qui contient l'appel
  au resolveur et l'appel au processus controlé.
  pProc : processus qui va contenir l'appel au resolveur
  pRoot : racine de l'arbre
  Attention, le processus dont on recherche les entrées/sorties  a 
  pour nom : vt
  ====================================================================*/
static void
creer_decl_IO_model (ntrees pRoot, ntrees pProc)
{
  ntrees pName, pModel, lDecl, pOutputs, pInputs, newDecl;
  int nb, i;

  SET_NILTREE (pModel);
  pName = LL_make_nom_atom (chaine_vt);
  if (!pGraph_arb->isModelInList (pName, STreeModel_getLocalDeclarations (pRoot), &pModel))
    pGraph_arb->isModelInList (pName, STreeModel_getLocalDeclarations (pProc), &pModel);
  if (ISNOT_NILTREE (pModel))
    {
      lDecl = STreeModel_getLocalDeclarations (pProc);
      pInputs = STree_getInputs (pModel);
      nb = ntreearity (pInputs);
      for (i = 1; i <= nb; i++)
	{
	  newDecl = COPY_ATTRIBUTEDSTREE (nchild (i, pInputs));
	  pGraph_text->addMark (newDecl);
	  ntreeList_insertFirst (newDecl, lDecl);
	}
      pOutputs = STree_getOutputs (pModel);
      nb = ntreearity (pOutputs);
      for (i = 1; i <= nb; i++)
	{
	  newDecl = COPY_ATTRIBUTEDSTREE (nchild (i, pOutputs));
	  pGraph_text->addMark (newDecl);
	  ntreeList_insertFirst (newDecl, lDecl);
	}
    }
}


/**========================================================================
  Creation d'une declaration de signal utilise pour le codage des etats
  ou mesures ou conditions ou sorties :
      [nu] integer cod_U     ou  
      [nx] integer cod_X     ou
      [nx] integer X_etend     ou
      [ncond] integer cod_cond   ou
      integer val_Y    ou 
      boolean pres_Y
   ?dim : nombre de variables --> correspond a la taille du vecteur
   s'il vaut 0, alors c'est un scalaire  
   ? nom : chaine du nom du vecteur ("cod_U" ou "cod_X"...)
  Ces declarations sont inserees dans les declarations locales du processus
  ?p_decl_loc.
  ========================================================================*/
static void
creer_decl_codage (ntrees p_decl_loc, int dim, ptrepstr nom)
{
  ntrees p_dim, p_typ, p_decl, p_list;
  ntrees typ = LL_make_integer_atom ();
  if (dim != 0)
    {
      p_dim = LL_make_nom_atom (cr_chaine_ent (dim));
      p_typ = LL_make_tvect_tree (LL_make_lesign_list (p_dim), typ);
    }
  else
    p_typ = typ;
  p_list = LL_make_lsign_list (LL_make_nom_atom (nom));
  p_decl = LL_make_decsig_tree (p_typ, p_list);
  pGraph_text->addMark (p_decl);
  ntreeList_insertFirst (p_decl, p_decl_loc);
}


/**==============================================================================
  Création des déclarations des signaux intermédiaires sous la forme :
  integer INTERM0, INTERM1...
  ?nb : nombre de signaux intermédiaires
  ?typ : type des signaux (actuellement appelé avec opinteger) 
  ? nom : chaine des nom des signaux (actuellement appelé avec "INTERM") 
  Ces declarations sont inserees dans les declarations locales du processus
  ?p_decl_loc.
==============================================================================*/
static void
creer_decl_interm (ntrees p_decl_loc, int nb, ptrepstr str_nom)
{
  ntrees p_typ, p_decl, p_list;
  ptrepstr str_interm;
  int i;

  if (nb != 0)
    {
      p_typ = LL_make_integer_atom ();
      p_list = LL_make_lsign_emptyList ();
      for (i = 0; i < nb; i++)
	{
	  /* strreset(reg); strconcstr(reg, str_nom);
	     strconcinttoreg(reg, i); strstore(reg, &str_interm); 
	   */
	  strconc2str (str_nom, integer2repstr (i), pk_false, &str_interm);
	  ntreeList_appendLast (p_list, LL_make_nom_atom (str_interm));
	}
      p_decl = LL_make_decsig_tree (p_typ, p_list);
      pGraph_text->addMark (p_decl);
      ntreeList_insertFirst (p_decl, p_decl_loc);
    }
}


/**==========================================================================
          Analyse du fichier .sim engendre par Sigali
  Representation des donnees dans le fichier .sim :
      $E suivi d'une liste d'identificateurs  --> etats
      $Y            "                "        --> les entrees incontrolables
      $C            "                "        --> les conditions
      $O            "                "        --> les sorties du resolveur
      $R -->  changement de configuaration pour le resolveur

  La premiere configuration sert a calculer les Y (entrees incontrolables)
  La seconde sert a calculer les U (entrees controlables)
  Les 2 configs sont separees par un $$
  Pour les sorties du resolveur, on ne prend que celles de la seconde config.
  ?p_expr : expression du processus dans lequel on rajoute l'appel au resolveur
  Rend faux si problème de lecture du fichier .sim
  ==========================================================================*/
static boolean
traiter_fich_sigali (char *fich_sigali, ntrees p_expr)
{
  FILE *fich;
  char nom_var[50];
  boolean premiere_config, fini;
  ptrepstr nom_sig;
  ntrees p_sync_mesu, p_sync_sort;

  if ((fich = fopen (fich_sigali, "r")) == NULL)
    return (pk_false);
  else
    {
      fini = pk_false;
      premiere_config = pk_true;
      /* construction des déclarations locales correspondants aux E/S du modèle à masquer */
      while (!fini && (fscanf (fich, "%s", nom_var) != EOF))
	{
	  if (nom_var[0] == '$')
	    switch (nom_var[1])
	      {
	      case 'Y':
		val_automate = var_mesu;
		nb_mesu = 1;
		break;
	      case 'E':
		val_automate = var_etat;
		nb_etat = 1;
		break;
	      case 'C':
		val_automate = var_cond;
		nb_cond = 1;
		break;
	      case 'R':
		val_automate = var_config;
		break;
	      case 'O':
		val_automate = var_sort;
		nb_sort = 1;
		break;
	      case '$':
		if (premiere_config)
		  premiere_config = pk_false;
		else
		  fini = pk_true;
	      }
	  else
	    {
	      nom_sig = string2repstr (nom_var);
	      switch (val_automate)
		{
		case var_mesu:
		  creer_codage_incontrolable (nom_sig, nb_mesu - 1, p_expr);
		  /* on engendre les synchro sur toutes les mesures */
		  if (nb_mesu == 1)
		    {
		      p_sync_mesu = LL_make_lequhor_list (LL_make_nom_atom (nom_sig));
		      pGraph_text->addMark (p_sync_mesu);
		    }
		  else
		    ntreeList_appendLast (p_sync_mesu, LL_make_nom_atom (nom_sig));
		  nb_mesu++;
		  break;
		case var_etat:
		  creer_codage_etat (chaine_X, nom_sig, nb_etat - 1, p_expr);
		  nb_etat++;
		  break;
		case var_cond:
		  creer_codage_etat (chaine_cond, nom_sig, nb_cond - 1, p_expr);
		  nb_cond++;
		  break;
		case var_sort:
		  if (!premiere_config)
		    {
		      creer_codage_sort (nom_sig, nb_sort - 1, p_expr);
		      /* on engendre les synchro sur toutes les commandes de sortie */
		      if (nb_sort == 1)
			{
			  p_sync_sort = LL_make_lequhor_list (LL_make_nom_atom (nom_sig));
			  pGraph_text->addMark (p_sync_sort);
			}
		      else
			ntreeList_appendLast (p_sync_sort, LL_make_nom_atom (nom_sig));
		      nb_sort++;
		    }
		  break;
		}
	    }
	}
      if (nb_sort > 1)
	{
	  ntreeList_appendLast (p_sync_sort, LL_make_nom_atom (chaine_Tick));
	  ntreeList_appendLast (p_expr, p_sync_sort);
	}
      if (nb_mesu > 1)
	{
	  ntreeList_appendLast (p_sync_mesu, LL_make_nom_atom (chaine_Tick));
	  ntreeList_appendLast (p_expr, p_sync_mesu);
	}
      fclose (fich);
      return (pk_true);
    }
}

///
static void
initResolv (ptMessage pMess, ptGraph_text pGrText, ptrepstr strNameProc)
{
  the_messageTool = pMess;
  pGraph_text = pGrText;
  //  reg = strnewreg();/*allocation d'un registre pour la manipulation des chaines*/
  chaine0 = string2repstr ("0");
  chaine1 = string2repstr ("1");
  chaine2 = string2repstr ("2");
  chaine_X = string2repstr ("cod_x");
  chaine_X_etend = string2repstr ("X_etend");
  chaine_Y = string2repstr ("cod_y");
  chaine_U = string2repstr ("cod_u");
  chaine_cond = string2repstr ("cod_cond");
  chaine_INTERM = string2repstr ("INTERM");
  chaine_Tick = string2repstr ("Tick");
  chaine_TTick = string2repstr ("TTick");
  chaine_vt = string2repstr ("vt");
  nom_fich = repstr2string (strNameProc);	// le nom du fichier est le nom du processus
  nom_fich_sigali[0] = '\0';
  strcat (nom_fich_sigali, nom_fich);
  strcat (nom_fich_sigali, ".sim");
  nb_mesu = 0;
  nb_sort = 0;
  nb_cond = 0;
  nb_etat = 0;
  compteur = 0;
}



// /*======================================================================
//   rend une nouvelle chaine qui est la concatenation de la chaine ?ch
//   avec la chaine representant l'entier ?no  (exemple compt_Y5)
//   =====================================================================*/
// static ptrepstr creer_chaine_indicee(ptrepstr ch, int no)
// {
//   ptrepstr ch_no, ch_indicee;

//     ch_no = cr_chaine_ent(no);
//     /* strreset(reg); strconcstr(reg, ch);
//        strconcstr(reg, ch_no); strstore(reg, &ch_indicee); */
//     strconc2str(ch, ch_no , pk_false, &ch_indicee);
//     return(ch_indicee);
// }

// static ptrepstr chaine_val_Y, chaine_val_YP, chaine_pres_Y, chaine_get_Y, chaine_compt_Y, 
// /*=======================================================================
//   Cette fonction cree et insere dans l'arbre Signal ?p_expr le sous-arbre 
//   qui correspond a la lecture de l'entree incontrolable de nom ?nom_sig :

//      (|(val_Y, pres_Y) := get_Y{}(no_ent, X_etend)
//          | val_YP := ((true when (val_Y = 1) default 
//                  false when (val_Y = -1)) when pres_Y) $ 1 init false 
//       | nom_sig := val_YP when Tick
//       | val_Y ^= pres_Y ^= TTick
//       |) /val_Y, pres_Y

//    ?no_ent : numero de l'entree.
//   =======================================================================*/
// static void creer_codage_ent(ptrepstr nom_sig, int no_ent, ntrees p_expr)
// {
//   ntrees p_appel, l_sig1, l_sig2, p_aff1, p_aff2, p_aff3, p_droite, p_when_pk_true, 
//     p_when_false, l_comp, l_synchro, p_default, p_compt;

//   /* construction du sous-arbre : (val_Y, pres_Y)  */
//   l_sig1 = LL_make_leflot_list( LL_make_nom_atom( chaine_val_Y));
//   ntreeList_appendLast(l_sig1, LL_make_nom_atom( chaine_pres_Y));

//   /* construction du sous-arbre : get_Y{}(no_ent, X_etend)  */
//   l_sig2 =  LL_make_lesign_list( LL_make_nom_atom( cr_chaine_ent(no_ent)));
//   ntreeList_appendLast(l_sig2,  LL_make_nom_atom( chaine_X_etend));
//   p_appel = LL_make_appe_tree(  LL_make_nom_atom( chaine_get_Y), l_sig2);

//   /* construction du sous-arbre : (val_Y, pres_Y) = get_Y{}(no_ent, X_etend) */
//   p_aff1 = LL_make_defsig_tree( l_sig1, p_appel);

//   /* construction du sous-arbre  val_YP := ((true when (val_Y = 1) default 
//                  false when (val_Y = -1)) when pres_Y) $ 1 init 1  */
//   p_when_true =  LL_make_extr_tree( LL_make_true_atom(), LL_make_egal_tree( LL_make_nom_atom( chaine_val_Y), LL_make_nom_atom( chaine1)));
//   p_when_false =  LL_make_extr_tree( LL_make_false_atom(), LL_make_egal_tree( LL_make_nom_atom( chaine_val_Y), 
//                                                                              LL_make_nega_tree( LL_make_ent_atom( chaine1))));
//   p_default =  LL_make_mele_tree( p_when_pk_true, p_when_false);
//   p_droite = LL_make_extr_tree( p_default, LL_make_nom_atom( chaine_pres_Y));
//   p_droite =  LL_make_ret( p_droite, LL_make_ent_atom( chaine1),
//                         LL_make_false_atom());
//   p_aff2 =  LL_make_defsig_tree( LL_make_nom_atom( chaine_val_YP), p_droite);
//   pGraph_text->addMark(p_aff2);
//   /* construction du sous-arbre nom_sig := val_YP when Tick */
//   p_droite =  LL_make_extr_tree( LL_make_nom_atom( chaine_val_YP),
//                        LL_make_nom_atom( chaine_Tick));
//   p_aff3 =  LL_make_defsig_tree( LL_make_nom_atom( nom_sig), p_droite);
//   pGraph_text->addMark(p_aff3);

//   /* construction de compt_Yi = (1 when pres_Y) default 0 when TTick  */
//   p_default =  LL_make_mele_tree( creer_when(chaine1, chaine_pres_Y),
//                         creer_when(chaine0, chaine_TTick));
//   p_compt = LL_make_defsig_tree( LL_make_nom_atom( 
//                      creer_chaine_indicee(chaine_compt_Y, no_ent)), p_default);
//   pGraph_text->addMark(p_compt);

//   /* construction du sous-arbre : val_Y ^= pres_Y ^= TTick  */
//   l_synchro =  LL_make_lequhor_list( LL_make_nom_atom( chaine_val_Y));
//   ntreeList_appendLast(l_synchro, LL_make_nom_atom( chaine_pres_Y));
//   ntreeList_appendLast(l_synchro, LL_make_nom_atom( chaine_TTick));
//   pGraph_text->addMark(l_synchro);

//   /* construction de la composition pour accrocher les 4 sous-arbres et 
//      construction du masquage de val_Y, val_YP et pres_Y. */
//   l_comp =  LL_make_lfeed_list( p_aff1);
//   ntreeList_appendLast(l_comp,  p_aff2);
//   ntreeList_appendLast(l_comp,  p_aff3);
//   ntreeList_appendLast(l_comp,  p_compt);
//   ntreeList_appendLast(l_comp,  l_synchro);
//   pGraph_text->addMark(l_comp);
//   l_sig1 = LL_make_lident_list( LL_make_nom_atom( chaine_pres_Y));
//   ntreeList_appendLast(l_sig1, LL_make_nom_atom( chaine_val_Y));
//   ntreeList_appendLast(l_sig1, LL_make_nom_atom( chaine_val_YP));
//   pGraph_text->addMark(l_sig1);
//   ntreeList_appendLast(p_expr, LL_make_obli_tree( l_comp, l_sig1));
// }  
