/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

_PUBLIC_
/**
   Compilation interactive : récupération de la fonctionalité du compilateur choisie par l'utilisateur.
   Cette méthode est appelée lorsque l'utilisateur a cliqué sur un des boutons de la fenêtre des options 
   du compilateur. Le code fonction est retourné par la méthode C2Java_getCompilerFunct() (monde Java) qui rend NOTHING
   tant que l'utilisateur n'a pas cliqué.
*/
  void
supervis::GET_FUNC (function_type * func, boolean * b)
{
  int no;

  no = C2Java_getCompilerFunct ();
  *b = (no != NOTHING);
  if (no == END)
    (*func) = func_end;
  else
    (*func) = array_of_functions[no];
}

/** 
    Compilation interactive : appelée par le supervieur après l'application d'une fonctionnalité.
    @param b (in) non utilisé.
*/
void
supervis::END_OF_FUNCTIONALITY ()
{
  pK_messenger->message (endFuncMess);
}


/** 
    Compilation interactive : méthode qui modifie l'apparence du label associé à une fonctionalité (func), en fonction de etat.
*/
void
supervis::func_modify (int func, func_type_color etat)
{
  int col;

  switch (etat)
    {
    case allowed:
      col = GREEN;
      break;
    case forbidden:
      col = RED;
      break;
    case withouteffect:
      col = YELLOW;
      break;
    }
  status_of_functions[func] = etat;
  C2Java_setColorCompilerFunct (func, col);	// vers JAVA
}

/**
   Compilation interactive: mise à jour de la fenêtre des fonctionnalités en fonction de state.
*/
void
supervis::ALLOWED_FUNC_DISPLAY (DCGLevel state)
{
  /* Always allowed */
  ALLOWED (func_gen_SIGNAL);
  ALLOWED (func_gen_Z3Z);
  switch (state)
    {
    case undefDCG:
      FORBIDDEN (func_clockCalculus);
      FORBIDDEN (func_retiming);
      FORBIDDEN (func_booleans2events);
      FORBIDDEN (func_unify);
      FORBIDDEN (func_abstraction);
      FORBIDDEN (func_poly2endo);
      FORBIDDEN (func_booleanizing);
      FORBIDDEN (func_trains);
      FORBIDDEN (func_serializing);
      FORBIDDEN (func_flattening);

      FORBIDDEN (func_gen_CANSI);
      FORBIDDEN (func_gen_CPP);
      FORBIDDEN (func_gen_JAVA);
      // FORBIDDEN(func_gen_Syndex);
      FORBIDDEN (func_profiling);

      break;
    case basicDCG:
      ALLOWED (func_clockCalculus);
      ALLOWED (func_retiming);
      ALLOWED (func_booleans2events);
      ALLOWED (func_unify);
      FORBIDDEN (func_abstraction);
      FORBIDDEN (func_poly2endo);
      FORBIDDEN (func_booleanizing);
      FORBIDDEN (func_trains);
      FORBIDDEN (func_serializing);
      FORBIDDEN (func_flattening);
      FORBIDDEN (func_gen_CANSI);
      FORBIDDEN (func_gen_CPP);
      FORBIDDEN (func_gen_JAVA);
      // FORBIDDEN(func_gen_Syndex);
      FORBIDDEN (func_profiling);
      break;

    case polyDCG:
      WITHOUTEFFTECT (func_clockCalculus);
      WITHOUTEFFTECT (func_retiming);
      WITHOUTEFFTECT (func_booleans2events);
      FORBIDDEN (func_booleanizing);
      ALLOWED (func_unify);
      ALLOWED (func_poly2endo);
      FORBIDDEN (func_trains);
      ALLOWED (func_abstraction);
      FORBIDDEN (func_serializing);
      FORBIDDEN (func_flattening);

      FORBIDDEN (func_gen_CANSI);
      FORBIDDEN (func_gen_CPP);
      FORBIDDEN (func_gen_JAVA);
      //      FORBIDDEN(func_gen_Syndex);
      FORBIDDEN (func_profiling);

      break;

    case endoDCG:
      WITHOUTEFFTECT (func_clockCalculus);
      WITHOUTEFFTECT (func_retiming);
      WITHOUTEFFTECT (func_booleans2events);
      ALLOWED (func_booleanizing);
      ALLOWED (func_unify);
      ALLOWED (func_poly2endo);
      FORBIDDEN (func_trains);
      ALLOWED (func_abstraction);
      FORBIDDEN (func_serializing);
      FORBIDDEN (func_flattening);
      FORBIDDEN (func_gen_CANSI);
      FORBIDDEN (func_gen_CPP);
      FORBIDDEN (func_gen_JAVA);
      //      FORBIDDEN(func_gen_Syndex);
      FORBIDDEN (func_profiling);
      break;

    case boolDCG:
      WITHOUTEFFTECT (func_clockCalculus);
      WITHOUTEFFTECT (func_retiming);
      WITHOUTEFFTECT (func_booleans2events);
      WITHOUTEFFTECT (func_booleanizing);
      WITHOUTEFFTECT (func_unify);
      ALLOWED (func_trains);
      FORBIDDEN (func_abstraction);
      ALLOWED (func_serializing);
      ALLOWED (func_flattening);
      FORBIDDEN (func_gen_CANSI);
      FORBIDDEN (func_gen_CPP);
      FORBIDDEN (func_gen_JAVA);
      //      FORBIDDEN(func_gen_Syndex);
      ALLOWED (func_profiling);
      break;

    case seqDCG:
      WITHOUTEFFTECT (func_clockCalculus);
      WITHOUTEFFTECT (func_retiming);
      WITHOUTEFFTECT (func_booleans2events);
      WITHOUTEFFTECT (func_unify);
      FORBIDDEN (func_trains);
      FORBIDDEN (func_abstraction);
      WITHOUTEFFTECT (func_booleanizing);
      WITHOUTEFFTECT (func_serializing);
      FORBIDDEN (func_flattening);
      ALLOWED (func_gen_CANSI);
      ALLOWED (func_gen_CPP);
      ALLOWED (func_gen_JAVA);
      // ALLOWED(func_gen_Syndex);
      ALLOWED (func_profiling);
      break;

    case flatDCG:
      WITHOUTEFFTECT (func_clockCalculus);
      WITHOUTEFFTECT (func_retiming);
      WITHOUTEFFTECT (func_booleans2events);
      WITHOUTEFFTECT (func_booleanizing);
      WITHOUTEFFTECT (func_unify);
      ALLOWED (func_trains);
      FORBIDDEN (func_abstraction);
      FORBIDDEN (func_serializing);
      WITHOUTEFFTECT (func_flattening);
      FORBIDDEN (func_gen_CANSI);
      FORBIDDEN (func_gen_CPP);
      FORBIDDEN (func_gen_JAVA);
      //      FORBIDDEN(func_gen_Syndex);
      ALLOWED (func_profiling);
      break;
    }
}

/**
   Compilation interactive: macro (ensemble de fonctionnalités) appelée pour le calcul des lignées d'un graphe (vgraph).
   @return vrai si l'opération est effectuée, faux sinon.
*/
boolean
supervis::trainsCalculus (ptDCG_bool vgraph)
{
  vgraph->setIntrinsicClocks ();
  vgraph->nonProductiveSignalElimination ();	// if (!booleanizingAndPruning(vgraph)) return pk_false;
  vgraph->setClockDependences ();
  if (!CycleDetection (vgraph))
    return pk_false;
  vgraph->nonProductiveSignalElimination ();
  vgraph->phylumClustering ();
  vgraph->nonProductiveSignalElimination ();
  clustersApplied = pk_true;
  return pk_true;
}


/**
   Compilation interactive: macro (ensemble de fonctionnalités) appelée pour la séquentialisation d'un graphe (vgraph).
   @return vrai si le graphe est sans sycle, faux sinon (cycle ou problème interne).
*/
boolean
supervis::totallyOrder (ptDCG_bool vgraph, ptDCG_sch * gSeq)
{
  if (!clustersApplied)
    {
      vgraph->setIntrinsicClocks ();
      Pruning (vgraph);
      vgraph->setClockDependences ();
      if (!CycleDetection (vgraph))
	return pk_false;
    }
  return Serializing (vgraph, OM->req_export_threads_subopt (), OM->req_force_subopt (), OM->req_export_check_subopt (),
		      OM->req_run_delayMultiplexing_opt (), gSeq);
}

/**
   Compilation interactive: macro (ensemble de fonctionnalités) appelée pour le "retiming", appliquée à un graphe (vgraph).
   @return vrai si l'opération est effectuée, faux sinon.
   @see pKServicesAPI::moveIn2OutDelays
*/
boolean
supervis::moveIn2OutDelays (ptDCG_basic vgraph)
{
  return pKServicesAPI::moveIn2OutDelays (vgraph);
}

/**
   Compilation interactive: macro (ensemble de fonctionnalités) appelée pour la détection de cycle, appliquée à un graphe (vgraph).
   @return vrai si l'opération est effectuée, faux sinon.
   @see hasUnsolvedClockConstraints(), isStaticCycleFree().
*/
boolean
supervis::CycleDetection (ptDCG_bool vgraph)
{
  boolean b;
  b = vgraph->hasUnsolvedClockConstraints ();
  if (b)
    if (!OM->req_force_subopt ())
      return pk_false;
  nonProductiveSignalElimination (vgraph);
  //  b = vgraph->isStaticCycleFree(pk_true, pk_false);  return b;

  return pKServicesAPI::isStaticCycleFree (vgraph, pk_true, pk_true);
  //  return pk_true;
}

/**
   Compilation interactive: 
   @see pKServicesAPI::transformBooleanClocks2Events()
*/
boolean
supervis::BooleansEvents (ptDCG_basic vgraph)
{
  return pKServicesAPI::transformBooleanClocks2Events (vgraph);
}

/**
   The basic graph (vgraph) is transformed into a poly graph one.
*/
boolean
supervis::DCGBasic2DCGPoly (ptDCG_basic gb, ptDCG_poly * gp)
{
  return pKServicesAPI::DCGBasic2DCGPoly (gb, OM->get_solvingDepth_opt (), gp);
}

/**
   Equivalent to supervis::DCGBasic2DCGPoly().
*/
boolean
supervis::ClockCalcul (ptDCG_basic gb, ptDCG_poly * gp)
{
  return DCGBasic2DCGPoly (gb, gp);
}

/**
   Compilation interactive:
   @see pKServicesAPI::ClockCalculus()
*/
boolean
supervis::ClockCalcul (ptDCG_poly vgraph)
{
  return pKServicesAPI::clockCalculus (vgraph);
}

/**
   Compilation interactive:
   @see pKServicesAPI::applyTabularMorphism()
*/
boolean
supervis::mkProfiling (ptDCG_poly vgraph)
{
  return pKServicesAPI::applyTabularMorphism (vgraph);
}

/**
   Compilation interactive:
   @see pKServicesAPI::removeSignalClones
*/
boolean
supervis::Unifications (ptDCG_basic vgraph)
{
  return pKServicesAPI::removeSignalClones (vgraph);
}


/**
   Compilation interactive:
   @see pKServicesAPI::unparseAbstraction()
*/
boolean
supervis::GenSignalAbs (ptDCG_bool vgraph)
{
  pKServicesAPI::unparseAbstraction (vgraph);
  return pk_true;
}


/**
   Compilation interactive: Sans effet (?).
   @return toujours "vrai".
*/
boolean
supervis::transitiveClosure (ptDCG_basic vgraph)
{
  return (pk_true);
}

/**
   Compilation interactive: macro (ensemble de fonctionnalités) appelée pour  la transformation d'un graphe (vgraph) au format "horloges booléennes".
   @see pKServicesAPI::setIntrinsicClocks(), booleanizingAndPruning().
*/
boolean
supervis::__transformToBooleanControl (ptDCG_basic vgraph)
{
  //   pour voir Loic, j'ai supprimé.
  //pKServicesAPI::setIntrinsicClocks(vgraph); 
  //  return booleanizingAndPruning(vgraph);
  return pk_false;
}

/**
   Compilation interactive: macro (ensemble de fonctionnalités) appelée pour la transformation d'un graphe (vgraph) au format STS. 
   (un seul niveau de hiérarchie d'horloge)
   @return vrai si l'opération est effectuée, faux sinon.
   @see pKServicesAPI::DCGPoly2DCGFlat()
*/
boolean
supervis::DCGBool2DCGFlat (ptDCG_bool vgraph, ptDCG_flat * gf)
{
  return pKServicesAPI::DCGPoly2DCGFlat (vgraph, gf, OM->req_traceALL ());
}



boolean
supervis::DCGPoly2DCGBool (ptDCG_poly gp, ptDCG_bool * gb)
{
  return pKServicesAPI::DCGPoly2DCGBool (gp, gb, OM->req_traceALL ());
}


boolean
supervis::DCGEndo2DCGBool (ptDCG_endo gp, ptDCG_bool * gb)
{
  return pKServicesAPI::DCGEndo2DCGBool (gp, gb);
}

boolean
supervis::DCGPoly2DCGEndo (ptDCG_poly gp, ptDCG_endo * gb)
{
  return pKServicesAPI::DCGPoly2DCGEndo (gp, gb);
}

/**
   Compilation interactive: restitution textuelle d'un graphe (vgraph).
   @see pKServicesAPI::IGraph2Signal()
*/
boolean
supervis::GenSignal (ptDCG_poly vgraph)
{
  return pKServicesAPI::unparse (vgraph);

}

/**
   Compilation interactive: production de code C pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2C()
*/
boolean
supervis::GenC (ptDCG_sch vgraph)
{
  return pKServicesAPI::genCodeC (vgraph, 1);
}


/**
   Compilation interactive: production de code C, en mode compilation séparée, pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2CSeparate()
*/
boolean
supervis::GenCseparate (ptDCG_sch vgraph)
{
  return pKServicesAPI::genCodeC (vgraph, 3);
}


/**
   Compilation interactive: production de code C++ pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2CPP()
*/
boolean
supervis::GenCPP (ptDCG_sch vgraph)
{
  return pKServicesAPI::genCodeCPP (vgraph, 1);
}

/**
   Compilation interactive: production de code C++, en mode compilation séparée, pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2CPPSeparate()
*/

boolean
supervis::GenCPPseparate (ptDCG_sch vgraph)
{
  return pKServicesAPI::genCodeCPP (vgraph, 3);
}

/**
   Compilation interactive: production de code Java pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2Java()
*/
boolean
supervis::GenJAVA (ptDCG_sch vgraph)
{
  return pKServicesAPI::genCodeJava (vgraph, 1);
}


/**
   Compilation interactive: production de code pour l'outil SIGALI pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2z3z()
*/
boolean
supervis::GenZ3z (ptDCG_poly vgraph)
{
  return pKServicesAPI::SigaliPrintExtraction (vgraph);
}


/**
   Compilation interactive: production de code pour l'outil SYNDEX pour un graphe (vgraph).
   @see pKServicesAPI::IGraph2SynDEx()
*/
boolean
supervis::GenSyndexCode (ptDCG_flat vgraph)
{
  return pKServicesAPI::export_SynDEx (vgraph, pk_true);
}

/**
   Compilation interactive : construction de l'abstraction d'un graphe (vgraph).
   @see pKServicesAPI::separatedCompiling()
*/
boolean
supervis::BuildAbstraction (ptDCG_poly vgraph, ptDCG_bool * gBool)
{
  vgraph->buildInterfaceAbstraction ();
  if (!pKServicesAPI::DCGPoly2DCGBool (vgraph, gBool, pk_false))
    return pk_false;
  (*gBool)->buildInterfaceAbstraction ();
  // if (! pKServicesAPI::InterfaceAbstraction(vgraph, gBool, OM->req_traceALL())) return pk_false;
  ALLOWED (func_gen_abstract);
  return pk_true;
}

/**
   Compilation interactive: partitionnement d'un graphe (vgraph) selon le pragma RunOn (partitionnement utilisateur)
   et restitution à l'utilisateur (fenêtre graphique).
   @see pKServicesAPI::partitionningRunOn()
*/
void
supervis::PartitioningRunOn (ptDCG_bool vgraph)
{
  int no;
  ptlINode lSubgraph;
  ptINode aSubGraph;
  ntrees p_proc;

  Pruning (vgraph);
  InterfaceCalculus (vgraph);
  if (vgraph->partitionningRunOn (&lSubgraph))
    {
      no = 1;
      while (ptlINode_get_elem (&lSubgraph, &aSubGraph))
	{
	  p_proc = vgraph->produceSubgraph (aSubGraph);
//        print(p_proc); // printTree(p_proc, -1);
	  pBuilder->build_compiled_proc (pCurObjManager->getModelInWhich_cou_engl (), p_proc, no);
	  no++;
	}
      pBuilder->add_new_links (pCurObjManager->getModelInWhich_cou_engl ());
    }
}

/**
   Compilation interactive: mise à jour de la forme interne d'un graphe (vgraph).
   @see pKServicesAPI::setIntrinsicClocks
*/
boolean
supervis::ClockEnvBuilding (ptDCG_poly vgraph)
{
  return pKServicesAPI::setIntrinsicClocks (vgraph);
}

/**
   Compilation interactive: nettoyage d'un graphe (vgraph).
   @see pKServicesAPI::nonProductiveSignalElimination
*/
boolean
supervis::Pruning (ptDCG_basic vgraph)
{
  return pKServicesAPI::nonProductiveSignalElimination (vgraph);
}

/**
   Compilation interactive: calcul d'interface appliqué à un graphe (vgraph).
   @see pKServicesAPI::buildClockProcessInterface
*/
boolean
supervis::InterfaceCalculus (ptDCG_poly vgraph)
{
  return pKServicesAPI::buildClockProcessInterface (vgraph);
}


/**
   Compilation interactive: affichage de la fenêtre présentant les différentes 
   functionalités du compilateur en mode interactif. 
*/
boolean
supervis::WIN_DISPLAY (void)
{
  C2Java_displayCompilerFunct ();
  ALLOWED_FUNC_DISPLAY (polyDCG);
  FORBIDDEN (func_gen_abstract);
  clustersApplied = pk_false;
  return (pk_true);
}
