/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */

package gui_dependent;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.lang.System;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

public class PolychronyWindowDependent extends JPanel {
    protected boolean isEclipse = true;

    /* Eclipse specific declarations */
    private static final long serialVersionUID = 1L;
    private Frame frame;
    private final String ID = "fr.irisa.espresso.polychrony.views.PolychronyView";
    
    /** Method called on top window creation */
    protected void setFrame(Frame awtFrame)
    {
	this.frame = awtFrame;
	frame.add(this);
	repaint();
    }
    
    /** Method called to exit the polychrony session 
	Swing compatibility  */
    protected void Destroy() {} 
    
    // PLG endPolychronyWindowConstructorDependent renamed assembleInTopWindow
    /** Method called on top window creation
        fill this polychrony window */
    protected void assembleInTopWindow(JPanel globalMenuPanel, JSplitPane signalPanelPair ) {
	
	setLayout(new BorderLayout());
	add(globalMenuPanel, BorderLayout.NORTH);
	add(signalPanelPair, BorderLayout.CENTER);

	repaint();
	
	frame.add(this);
    }
    
    /** Method called on user window creation */
    protected void newWindowInEclipse(final String numWindow)
    {
	Display.getDefault().asyncExec(new Runnable() {
		public void run() {
		    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		    try {
			page.showView(ID, numWindow, IWorkbenchPage.VIEW_ACTIVATE);
			IViewPart pView = page.findViewReference(ID,numWindow).getView(false);
			System.out.println(pView.getTitle() + " " + pView.getClass());
		    } catch (PartInitException e1) {
			e1.printStackTrace();
		    }
		}
	    });
	
    }
    
    /** Method called when closing the polychrony session  
	Swing compatibility */
    protected void CloseOperationDependent() {}

    /** Method called when creating a polychronoy window to listen for corner closing
	Swing compatibility */
    protected void  addCloseListenerDependent() {}
    
    /** Method called to set or reset the polychrony window title 
	Swing compatibility */
    protected void setTitleDependent(String vtitle) {}
    
    /** Method called when the user clicks on the button "quit".closing a polychrony window
        @return "true" if the user confirms the exit, "false" otherwise.
	In Eclipse, the user must close the view.
    */
    public boolean confirmPolychronyExit() {
	JOptionPane.showMessageDialog(this, "To quit, use  the close view, please");
	return false;
    }
    
    
}

