/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.lang.Object;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import common.UserActions;
import common.PolychronyWindow;

class JTree2ActionListener implements ActionListener {
    JTree2 myTree;
    int JTree2ActionListener_userAction;
    int JTree2ActionListener_complement;

    JTree2ActionListener(JTree2 jt, int noAct) {
        myTree = jt;
        JTree2ActionListener_userAction = noAct;
    }

    JTree2ActionListener(JTree2 jt, int noAct, int complement) {
        myTree = jt;
        JTree2ActionListener_userAction = noAct;
        JTree2ActionListener_complement = complement;
    }

    public void actionPerformed(ActionEvent event) {
        myTree.actionPerformedTreeListener(JTree2ActionListener_userAction, JTree2ActionListener_complement);
    }
}

class JTree2ModelListener implements TreeModelListener {
    JTree2 this_JTree2;

    JTree2ModelListener(JTree2 p) {
        this_JTree2 = p;
    }

    public void treeNodesChanged(TreeModelEvent e) {
    }

    public void treeNodesInserted(TreeModelEvent e) {
    }

    public void treeNodesRemoved(TreeModelEvent e) {
    }

    public void treeStructureChanged(TreeModelEvent e) {
    }

}

class JTree2CellRenderer extends DefaultTreeCellRenderer {
    private static final long serialVersionUID = 1L;
    ImageIcon processIcon, functionIcon, nodeIcon, actionIcon;
    ImageIcon processIconError, functionIconError, nodeIconError, actionIconError;
    LiaisonSignal LS;
    SignalTreeNode vroot;

    public JTree2CellRenderer(LiaisonSignal ls, SignalTreeNode treeRoot, PolychronyWindow gw) {
        LS = ls;
        vroot = treeRoot;
        String x = gw.getgifHtmlDir();
        processIcon = new ImageIcon(x + "process.gif");
        functionIcon = new ImageIcon(x + "function.gif");
        nodeIcon = new ImageIcon(x + "node.gif");
        actionIcon = new ImageIcon(x + "action.gif");

	processIconError = new ImageIcon(x + "processError.png");
        functionIconError = new ImageIcon(x + "functionError.png");
        nodeIconError = new ImageIcon(x + "nodeError.png");
        actionIconError = new ImageIcon(x + "actionError.png");
    }

    // class JTree2CellRenderer
    public void changeIcon(SignalTreeNode node, ImageIcon icon, ImageIcon iconError) 
    {
	ImageIcon s;	
	if (node.hasError()) s = iconError; else s = icon; 
	setIcon(s);
    }

    // class JTree2CellRenderer
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) 
    {
        super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
        SignalTreeNode node = (SignalTreeNode) value;
        if (node != vroot)
            if (node.isProc())
		changeIcon(node, processIcon, processIconError);
            else if (node.isFunc())
		changeIcon(node, functionIcon, functionIconError);
            else if (node.isNode())
                changeIcon(node, nodeIcon, nodeIconError);
            else if (node.isAction())
		changeIcon(node, actionIcon, actionIconError);
        return this;
    }

}

public class JTree2 extends JPanel implements MouseListener {
//JTree
    // getLastSelectedPathComponent() de la classe JTree
    // setSelectionPath(TreePath path) DÃ©finir la sÃ©lection avec les noeuds dont les chemins sont fournis en paramÃštre
    // void setSelectionRow(int row)


    // SignalTreeNode extends DefaultMutableTreeNode that extends TreeNode
//TreeNode
    // TreeNodegetParent
    // TreeNode getChildAt(int index) 	Renvoie le noeud fils correspondant Ã  l'index fourni en paramÃštre
    // int getChildCount() 	renvoie le nombre de noeuds fils directs du noeud
    // int getIndex(TreeNode child) 	renvoie l'index du noeud passÃ© en paramÃštre
    // TreeNode getParent() 	renvoie le noeud pÃšre
//MutableTreeNode
    // void insert(MutableTreeNode child, int index);
    // void remove(int index);
    // void remove(MutableTreeNode node);
    // void removeFromParent();
    // void setParent(MutableTreeNode parent);
    // void setUserObject(Object userObject);
    
    private static int the_setNewNodeNumber = 1;
    private static JTree the_JTreeClipboard = null; //  Allows to save the tree to copy
    private static SignalTreeNode the_JTreeClipboardNode = null; // root of the treeTemp tree.
    private static DefaultTreeModel the_JTreeClipboardNodeModel = null;

    /** Connection with the environment */
    private PolychronyWindow this_Pwindow; // "this" is thisTreeWindow of this_Pwindow
    private LiaisonSignal this_C_interface; // Link with the C world in herited from this_Pwindow

    private JTree this_JTree; // tree contained in this JTree2
    private SignalTreeNode this_JTreeRoot; // the never null root of this_JTree
    private DefaultTreeModel this_JTreeRootModel; // is never null
    private SignalTreeNode this_activeSTNode, this_selectedSTNode, clonedSTNode;


    private JScrollPane this_scrollPane;
    private JPopupMenu this_emptyPopup, this_emptySelectionPopup, this_commonModelPopup, this_rootModelPopup, this_windowModelPopup, this_commonModulePopup;

    private static final long serialVersionUID = 1L;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();
    private MutableTreeNode parentNode = null; // Father of the selected node

    private String searchString = ""; // Contains the String to search

    private boolean gwCourante = false; // Allows to know which GW does a paste or cut action, etc.
    private SignalTreeNode nodeFound; // Node selected in the multiwindowing

    private int this_MOUSE_position;

    private void show_DEBUG(String message) {
	JOptionPane.showMessageDialog(this, message, "DEBUG", JOptionPane.INFORMATION_MESSAGE);
    }

    private void displayError(String errorMessage) {
	JOptionPane.showMessageDialog(this, errorMessage, "Polychrony alert", JOptionPane.ERROR_MESSAGE);
    }

    private void displayError(int errorNum) {
	displayError(Constants.ERRORS_TAB[errorNum]);
    }

    // class JTree2
    private Color get_color3() {
        return pKGUIColors.Color3;
    }

    // Contextual menu of JTree associated with a module
    // class JTree2
    private void add_JMenuItem(JPopupMenu popup, String itemLabel, int noAct) {
        JMenuItem aMenuItem = new JMenuItem(itemLabel);
        aMenuItem.addActionListener(new JTree2ActionListener(this, noAct));
        popup.add(aMenuItem);
    }

    // Contextual menu of JTree associated with a module
    // class JTree2
    private void add_JMenuItem(JPopupMenu popup, String itemLabel, int noAct, int complement) {
        JMenuItem aMenuItem = new JMenuItem(itemLabel);
        aMenuItem.addActionListener(new JTree2ActionListener(this, noAct, complement));
        popup.add(aMenuItem);
    }

    // Contextual menu of JTree associated with a module
    // class JTree2
    private void add_JMenuSubItem(JMenu parentItem, String itemLabel, int noAct, int complement) {
        JMenuItem aMenuItem = new JMenuItem(itemLabel);
        aMenuItem.addActionListener(new JTree2ActionListener(this, noAct, complement));
        parentItem.add(aMenuItem);
    }

     // Contextual menu of JTree associated with a module
    // class JTree2
    private void add_JMenuSubItem(JMenu parentItem, String itemLabel, int noAct) {
        JMenuItem aMenuItem = new JMenuItem(itemLabel);
        aMenuItem.addActionListener(new JTree2ActionListener(this, noAct));
        parentItem.add(aMenuItem);
    }

    // class JTree2
    private void add_whereJMenuItem(JMenu masterMenu, int userAct, boolean isRoot) {
	JMenuItem item;

        item = new JMenuItem("cut");
        item.addActionListener(new JTree2ActionListener(this, userAct, UserActions.J_cut));
        masterMenu.add(item);

        item = new JMenuItem("in");
        item.addActionListener(new JTree2ActionListener(this, userAct, UserActions.J_in));
        masterMenu.add(item);

        if (! isRoot) {
	    item = new JMenuItem("before");
	    item.addActionListener(new JTree2ActionListener(this, userAct, UserActions.J_before));
	    masterMenu.add(item); }
    }

    // class JTree2
    private void insertPopupMenuTitle(JPopupMenu popup, String title) {
        JMenuItem aMenuItem = new JMenuItem(title);
        popup.add(aMenuItem);
        aMenuItem.setEnabled(false);
	popup.addSeparator();
    }

    // Contextual menu of JTree associated with environment
    // class JTree2
    private JPopupMenu make_noSelectionPopupMenu(JTree tree, boolean isNotEmpty) {
	String complement;
        JPopupMenu popup = new JPopupMenu();
	
        insertPopupMenuTitle(popup, "Tree menu");
	if ( isNotEmpty ) {
	    complement = "(cut)";
	    add_JMenuItem(popup, "find", UserActions.J_search);
	    popup.addSeparator(); }
	else
	    complement = "";
	add_JMenuItem(popup, "paste"+complement, UserActions.J_paste, UserActions.J_cut);
	popup.addSeparator();
	add_JMenuItem(popup, "new module"+complement, UserActions.J_creermodule, UserActions.J_cut);
	add_JMenuItem(popup, "new process"+complement, UserActions.J_creerproc, UserActions.J_cut);
	add_JMenuItem(popup, "new node"+complement, UserActions.J_creernode, UserActions.J_cut);
	add_JMenuItem(popup, "new function"+complement, UserActions.J_creerfunc, UserActions.J_cut);
	add_JMenuItem(popup, "new action"+complement, UserActions.J_creeraction, UserActions.J_cut);
	popup.addSeparator();
	add_JMenuItem(popup, "load"+complement, UserActions.J_load, UserActions.J_root);
	add_JMenuItem(popup, "parse"+complement, UserActions.J_parse, UserActions.J_root);
	if ( isNotEmpty ) 
	    add_JMenuItem(popup, "save(root)", UserActions.J_save, UserActions.J_root);

        tree.add(popup);
        tree.addMouseListener(this);
        return (popup);
    }

    // class JTree2
    private void add_inBeforeCutMenu(JMenu superItem, String mainLabel, int mainAction, boolean isTop) {
        JMenu parentItem = new JMenu(mainLabel);
	add_JMenuSubItem(parentItem, "in", mainAction, UserActions.J_in);
	if (!isTop)
	    add_JMenuSubItem(parentItem, "before", mainAction, UserActions.J_before);
	add_JMenuSubItem(parentItem, "cut", mainAction, UserActions.J_cut);
        superItem.add(parentItem);
    }

    // class JTree2
    private void add_newModelItem(JPopupMenu popup, boolean isTop){
        JMenu newItem = new JMenu("new");
	add_inBeforeCutMenu(newItem, "process",  UserActions.J_creerproc, isTop);
	add_inBeforeCutMenu(newItem, "node",  UserActions.J_creernode, isTop);
	add_inBeforeCutMenu(newItem, "function",  UserActions.J_creerfunc, isTop);
	add_inBeforeCutMenu(newItem, "action",  UserActions.J_creeraction, isTop);
        popup.add(newItem);
    }
    
    // class JTree2
    private JPopupMenu make_modulePopupMenu(JTree tree) {
        JPopupMenu popup = new JPopupMenu();

        insertPopupMenuTitle(popup, "Module menu");
	add_JMenuItem(popup, "declarations", UserActions.J_locDecl);
	add_JMenuItem(popup, "name", UserActions.J_setName);
	add_JMenuItem(popup, "find", UserActions.J_search);
	popup.addSeparator();
	add_JMenuItem(popup, "delete", UserActions.J_delete);
	add_JMenuItem(popup, "cut", UserActions.J_cut);
	add_JMenuItem(popup, "copy", UserActions.J_copy);
	add_JMenuItem(popup, "paste in", UserActions.J_paste, UserActions.J_in);
	add_JMenuItem(popup, "new process in", UserActions.J_creerproc, UserActions.J_in);
	add_newModelItem(popup, true);
	popup.addSeparator();

	add_JMenuItem(popup, "load in", UserActions.J_load, UserActions.J_in);
	add_JMenuItem(popup, "parse in", UserActions.J_parse, UserActions.J_in);
	add_JMenuItem(popup, "save as", UserActions.J_saveAs, UserActions.J_root);
	add_JMenuItem(popup, "unparse as", UserActions.J_unparseAs); 
	
        tree.add(popup);
        tree.addMouseListener(this);
        return (popup);
    }

    // Contextual menu of JTree associated with a process model
    // class JTree2
    private JPopupMenu make_modelPopupMenu(JTree tree, boolean isRoot, boolean hasWindow) {
        JPopupMenu popup = new JPopupMenu();

        insertPopupMenuTitle(popup, "Model menu");
	if ( hasWindow ) 
	    add_JMenuItem(popup, "goto window", UserActions.J_gotoWindow);
	else if (!isRoot)
	    add_JMenuItem(popup, "copy to new window", UserActions.J_copyPaste2newWindow);
	popup.addSeparator();
        JMenu attributes = new JMenu("set"); // 10
	add_JMenuSubItem(attributes, "name", UserActions.J_setName);
	add_JMenuSubItem(attributes, "parameters", UserActions.J_param);
	add_JMenuSubItem(attributes, "inputs outputs", UserActions.J_inOut);
	add_JMenuSubItem(attributes, "specification", UserActions.J_spec);
	add_JMenuSubItem(attributes, "pragmas", UserActions.J_pragm);
	popup.add(attributes);

        JMenu type = new JMenu("category");
	add_JMenuSubItem(type, "function", UserActions.J_ch_typ_func);
	add_JMenuSubItem(type, "node", UserActions.J_ch_typ_node);
	add_JMenuSubItem(type, "process", UserActions.J_ch_typ_proc);
	add_JMenuSubItem(type, "action", UserActions.J_ch_typ_act);
        attributes.add(type);

	add_JMenuItem(popup, "declarations", UserActions.J_locDecl);

	add_JMenuItem(popup, "find", UserActions.J_search);

	popup.addSeparator();
	add_JMenuItem(popup, "cut", UserActions.J_cut);
	add_JMenuItem(popup, "copy", UserActions.J_copy);

        JMenu aMenu = new JMenu("paste");
        aMenu.addActionListener(new JTree2ActionListener(this, UserActions.J_paste));
	add_whereJMenuItem(aMenu, UserActions.J_paste, isRoot);
        popup.add(aMenu);

	popup.addSeparator();
	add_JMenuItem(popup, "new process in", UserActions.J_creerproc, UserActions.J_in);

	add_newModelItem(popup, isRoot);

	popup.addSeparator();
	if (!isRoot)
	    add_JMenuItem(popup, "load before", UserActions.J_load, UserActions.J_before);
	add_JMenuItem(popup, "load in", UserActions.J_load, UserActions.J_in);
	if (!isRoot)
	    add_JMenuItem(popup, "parse before", UserActions.J_parse, UserActions.J_before);
	add_JMenuItem(popup, "parse in", UserActions.J_parse, UserActions.J_in);
	add_JMenuItem(popup, "save as", UserActions.J_saveAs, UserActions.J_in);
	add_JMenuItem(popup, "unparse as", UserActions.J_unparseAs);

        tree.add(popup);
        tree.addMouseListener(this);
        return (popup);
    }


// assume: treePath is not null 2014/12/31 OK
    // class JTree2
    private void select_path(TreePath treePath) {
	// System.out.println("DEBUG entering JTree2::select_path");
        this_JTree.setSelectionPath(treePath);
        this_JTree.scrollPathToVisible(treePath);
        this_JTree.makeVisible(treePath);
	// System.out.println("DEBUG  exiting JTree2::select_path");
    }

// PLG added
// ====================================================================== SignalTreeNodes
    /** if index < 0 insert child in last position in targetParent
	if index >= 0 insert child in targetParent at position index (first position is 0)
     */
    private void insert_ntreeNodeIn(SignalTreeNode targetParent, SignalTreeNode child, int index) {
        this_C_interface.SignalTreeInsert(targetParent.get_ntreeValue(), child.get_ntreeValue(), index);
    }
    /** sets ntrees node associated with this_selectedSTNode as selected node in C world
     */
    // class JTree2
    private void select_ntreeNode() {
	this_C_interface.SignalTreeSelect(this_selectedSTNode.get_ntreeValue());
    }

    /** create and return a ntrees model, child of ntreeParent at position index, in C world
     */
    // class JTree2
    private long create_ntreeChildAtI(long ntreeParent, String modelName, int category, int index) {
	return (long)(this_C_interface.SignalProcCreate(ntreeParent, modelName, category, index));
    }

    // class JTree2
    private long create_ntreeModule(long ntreeParent, String moduleName) {
	return (long)(this_C_interface.SignalModuleCreate(ntreeParent, moduleName));
    }

// assume: aNode is not null 2014/12/31 OK
    // class JTree2
    private void select_pathOf(SignalTreeNode aNode) {
	// System.out.println("DEBUG entering JTree2::select_pathOf");
        TreePath treePath = new TreePath(aNode.getPath());
	// guarantee: treePath is not null
	select_path(treePath); 
	// System.out.println("DEBUG  exiting JTree2::select_pathOf");
    }

    /** sets aNode as the selected node.
	assume : aNode is not null */
    // class JTree2
    public void select_node(SignalTreeNode aNode) {
	// System.out.println("DEBUG entering JTree2::select_node");
	// guarantee: aNode is not null
        select_pathOf(aNode);
	this_selectedSTNode = aNode;
	select_ntreeNode();
	// System.out.println("DEBUG  exiting JTree2::select_node");
    }

    /** sets the root as the selected node.
	assume: this_JTreeRoot is not null
    */
    // class JTree2
    public void select_JTreeRoot() {
	select_node(this_JTreeRoot);

    }

    // assume: this_JTreeRoot is not null
    // class JTree2
    public void mutateActiveToSelectedSTNode() {
	if ( this_activeSTNode == null )
	    select_JTreeRoot();
	else {
	    // guarantee: this_activeSTNode is not null
	    select_node(this_activeSTNode);
	    this_activeSTNode = null; }
    }

    // assume this_JTreeRoot is not null
    // class JTree2
    public boolean isEmpty_nTrees(){
	return this_JTreeRoot.getChildCount() <= 0;
    }

    // assume this_JTree, this_JTreeRoot is not null
    // class JTree2
    public boolean isSelected_JTreeRoot(){
	TreePath selectedPath = this_JTree.getSelectionPath();
	if ( selectedPath == null ) {
	    select_JTreeRoot();
	    return true; }
	return ( ((SignalTreeNode) (selectedPath.getLastPathComponent())) == this_JTreeRoot );
    }

    /** @return false if the ntrees is empty or if this selected node is strictly inner than the ntrees root
	@return true: the selected node is the ntrees root
     */
    // class JTree2
    public boolean isSelected_ntreesRoot(){
	// guarantee this_JTreeRoot is not null
	if ( isEmpty_nTrees() )
	    return false;
	// guarantee: this_JTreeRoot is not null and has one or more children
	return ( this_selectedSTNode ==  (SignalTreeNode)this_JTreeRoot.getChildAt(0));
    }

    /** @return true if the selected node is a module ntrees root
     */
    // class JTree2
    public boolean isSelected_module() {
	if ( isSelected_ntreesRoot() )
	    return this_selectedSTNode.isModul();
	return false;
    }
    
    /** @return true if the selected node is a model
     */
    // class JTree2
    public boolean isSelected_model() {
	if ( isEmpty_nTrees() )
	    return false;
	// guarantee: this_JTreeRoot is not null and has one or more children
	return !( isSelected_module() );
    }
    
    /** @return true if the selected node is not the root (and is a model)
     */
    // class JTree2
    public boolean isSelected_innerModel() {
	return ! ( isEmpty_nTrees() || isSelected_JTreeRoot() || isSelected_ntreesRoot() );
    }
    
    // class JTree2
    private void Iset_name(SignalTreeNode selectedNode) {
        String actualName, implicitName = new String((String) selectedNode.getUserObject());
	if ( isSelected_module() )
	    actualName = JOptionPane.showInputDialog(this,"Module name?", implicitName); 
	else
	    actualName = JOptionPane.showInputDialog(this,"Model name?", implicitName); 
	if ( actualName == null )
	    return; 
	selectedNode.setUserObject(actualName);
	this_C_interface.SignalChangeProcName(selectedNode.get_ntreeValue(), actualName);
	this_JTree.expandPath(new TreePath(selectedNode.getPath()));
    }

    /** Recursive copy srcTreeNode into the_JTreeClipboardNodeModel
	A node must be created for each Signal sub-process.
	ASSUMES  the_JTreeClipboardNodeModel not null */
    // class JTree2
    private void innerCloneTreeNode(SignalTreeNode srcTreeNode) {
        long sigRef;
        int i;
        SignalTreeNode node;
        sigRef = srcTreeNode.getSignalChildRef(this_C_interface, 1);
        i = 1;
        while (sigRef != 0) { // for each Signal model child, creation of a node which is inserted
            node = new SignalTreeNode(this_C_interface, sigRef);  // 3 cannot be a module
            node.setAllowsChildren(true);
            the_JTreeClipboardNodeModel.insertNodeInto(node, srcTreeNode, srcTreeNode.getChildCount());
            i++;
            sigRef = srcTreeNode.getSignalChildRef(this_C_interface, i);
            innerCloneTreeNode(node); // ASSUMES  the_JTreeClipboardNodeModel not null
        }
    }

    /** sets the_JTreeClipboardNodeModel 
	ASSUMES the_JTreeClipboardNode not null
     */
    // class JTree2
    private void set_JTreeClipboardNodeModel() 
    {
        the_JTreeClipboardNodeModel = new DefaultTreeModel(the_JTreeClipboardNode);
        the_JTreeClipboard = new JTree(the_JTreeClipboardNodeModel);
        the_JTreeClipboard.setEditable(true);
        the_JTreeClipboardNodeModel.setAsksAllowsChildren(true);
    }

    /** The clipBoard a new copy of sourceNode
	ASSUMES sourceNode not null OK
     */
    // class JTree2
    private void clone_SNode2ClipBoard(SignalTreeNode sourceNode) 
    {
	long ntreeSource = sourceNode.get_ntreeValue();
        the_JTreeClipboardNode = new SignalTreeNode(this_C_interface, ntreeSource); // 3 can be a module
        the_JTreeClipboardNode.copySignalTreeNode(this_C_interface);
        the_JTreeClipboardNode.setAllowsChildren(true);
	set_JTreeClipboardNodeModel(); //sets the_JTreeClipboardNodeModel,  ASSUMES the_JTreeClipboardNode not null OK
        innerCloneTreeNode(the_JTreeClipboardNode); // ASSUMES  the_JTreeClipboardNodeModel not null
    }

    /** The clipBoard becomes the single reference to sourceNode
	ASSUMES sourceNode not null
     */
    // class JTree2
    private void move_SNode2ClipBoard(SignalTreeNode sourceNode)
    {
	long ntreeSource = sourceNode.get_ntreeValue();
	// System.out.println("DEBUG entering JTree2::move_SNode2ClipBoard");
	this_JTreeRootModel.removeNodeFromParent(sourceNode);
	the_JTreeClipboardNode = sourceNode;
	set_JTreeClipboardNodeModel(); //sets the_JTreeClipboardNodeModel,  ASSUMES the_JTreeClipboardNode not null
	this_C_interface.SignalTreeDelete(ntreeSource, true); // detach
	// System.out.println("DEBUG  exiting JTree2::move_SNode2ClipBoard");
}

    /** The clipBoard becomes the single reference to cutNode
	ASSUME cutNode not null
	assume targetParent contains at least cutNode
     */
    // class JTree2
    private void exchange_clipBoardWithSNode(SignalTreeNode targetParent, SignalTreeNode cutNode, int index)
    {
	// System.out.println("DEBUG entering JTree2::exchange_clipBoardWithSNode");
	insert_ntreeNodeIn(targetParent, the_JTreeClipboardNode, index); // Signal tree insertion
	this_JTreeRootModel.insertNodeInto(the_JTreeClipboardNode, targetParent, index);
	// assume: the_JTreeClipboardNode is not null
      	select_node(the_JTreeClipboardNode);
	this_JTree.expandPath(new TreePath(targetParent.getPath()));
	move_SNode2ClipBoard(cutNode); // ASSUMES cutNode not null
	// System.out.println("DEBUG  exiting JTree2::exchange_clipBoardWithSNode");
    }


    /** The clipBoard becomes the single reference to sourceNode
	ASSUMES the_JTreeClipboardNode not NULL
	if index < 0 copy the_JTreeClipboardNode in last position in targetParent
	if index >= 0 copy the_JTreeClipboardNode in targetParent at position index (first position is 0)
     */
    // class JTree2
    private void paste_clipBoardInSNode(SignalTreeNode targetParent, int index)
    {
	SignalTreeNode sourceNode = the_JTreeClipboardNode;
	insert_ntreeNodeIn(targetParent, sourceNode, index);
	// we need to copy the process of the temporary tree
	if ( index < 0 )
	    this_JTreeRootModel.insertNodeInto(sourceNode, targetParent, targetParent.getChildCount());
	else 
	    this_JTreeRootModel.insertNodeInto(sourceNode, targetParent, index);
	// guarantee: sourceNode is not null
	select_node(sourceNode);
	this_JTree.expandPath(new TreePath(targetParent.getPath()));
	clone_SNode2ClipBoard(sourceNode); // ASSUMES sourceNode not null OK
    }

    // assume: this_JTreeRoot is not null
    // class JTree2
    public void mousePressed(MouseEvent e) {
	// System.out.println("DEBUG entering JTree2::mousePressed");
	if ( this_MOUSE_position == UserActions.MOUSE_up ) {
	    int row = this_JTree.getRowForLocation(e.getX(), e.getY());
	    TreePath currentSelection = this_JTree.getPathForRow(row);
	    if (SwingUtilities.isLeftMouseButton(e)) {
		this_MOUSE_position = UserActions.MOUSE_left; }
	    else if (SwingUtilities.isMiddleMouseButton(e)) {
		this_MOUSE_position = UserActions.MOUSE_middle; }
	    else if (SwingUtilities.isRightMouseButton(e)) {
		JPopupMenu popup;
		this_MOUSE_position = UserActions.MOUSE_right;
		if (currentSelection == null) {
		    select_JTreeRoot(); 
		    if ( isEmpty_nTrees() )
			popup = this_emptyPopup;
		    else
			// guarantee: this_JTreeRoot is not null and has one or more children
			popup = this_emptySelectionPopup; }
		else {
		    // assume: currentSelection is not null
		    // assume: currentSelection.getLastPathComponent() is not null
		    select_node((SignalTreeNode) (currentSelection.getLastPathComponent()));
		    if ( this_selectedSTNode.isModul() ) 
			popup = this_commonModulePopup; 
		    else if ( isSelected_ntreesRoot() )
			popup = this_rootModelPopup;
		    else if ( this_selectedSTNode.has_dedicatedWindow() ) 
			popup = this_windowModelPopup;
		    else 
			popup = this_commonModelPopup; } 
		this_activeSTNode = this_selectedSTNode;
		if (this_JTree.getHeight() - e.getY() < 250)
		    popup.show(this_JTree, e.getX(), e.getY() - 235); // 235 = menu height
		else
		    popup.show(this_JTree, e.getX(), e.getY() - 10); } 
	    else { // none of LeftMouseButton, MiddleMouseButton, RightMouseButton
		this_MOUSE_position = UserActions.MOUSE_error; } }
	else {
	    if (SwingUtilities.isLeftMouseButton(e)) {
		if ( this_MOUSE_position != UserActions.MOUSE_left ) 
		    this_MOUSE_position = UserActions.MOUSE_error; }
	    else if (SwingUtilities.isMiddleMouseButton(e)) {
		if ( this_MOUSE_position != UserActions.MOUSE_middle ) 
		    this_MOUSE_position = UserActions.MOUSE_error; }
	    else if (SwingUtilities.isRightMouseButton(e)) {
		if ( this_MOUSE_position != UserActions.MOUSE_right ) 
		    this_MOUSE_position = UserActions.MOUSE_error; }
	    else
		this_MOUSE_position = UserActions.MOUSE_error; }
	// System.out.println("DEBUG  exiting JTree2::mousePressed");
    }
    
    // class JTree2
    public void mouseReleased(MouseEvent e) {
	// System.out.println("DEBUG entering JTree2::mouseReleased");
	if ( this_MOUSE_position == UserActions.MOUSE_left ) {
            int row = this_JTree.getRowForLocation(e.getX(), e.getY());
            TreePath currentSelection = this_JTree.getPathForRow(row);
            if (currentSelection == null) 
		select_JTreeRoot();
	    else
		// guarantee: currentSelection is not null
		// assume: currentSelection.getLastPathComponent() is not null
		select_node((SignalTreeNode) (currentSelection.getLastPathComponent()));
	    this_Pwindow.refresh();	
	    this_activeSTNode = null; }
	// else if ( this_MOUSE_position == UserActions.MOUSE_middle ){
	// }
	// else if ( this_MOUSE_position == UserActions.MOUSE_right ) {
	// }
	this_MOUSE_position = UserActions.MOUSE_up;
	// System.out.println("DEBUG  exiting JTree2::mouseReleased");
    }
    
    // assume: this_JTreeRoot is not null
    // the mouse is clicked only when another event is generated between press and release ?
    // class JTree2
    public void mouseClicked(MouseEvent e) {
	// System.out.println("DEBUG entering/exiting JTree2::mouseClicked");
    }
    
    // class JTree2
    public void mouseEntered(MouseEvent e) { 
	// System.out.println("DEBUG entering/exiting JTree2::mouseEntered");
    }
    // class JTree2
    public void mouseExited(MouseEvent e) { 
	// System.out.println("DEBUG entering/exiting JTree2::mouseExited");
    }

    /** Removes all nodes except the root node. */
    // class JTree2
    public void clear() {
        this_JTreeRoot.removeAllChildren();
        for (int j = 0; j < this_JTreeRoot.getChildCount(); j++)
            this_JTreeRootModel.removeNodeFromParent((SignalTreeNode) (this_JTreeRoot)
						     .getChildAt(j));
        String name = new String((String) this_JTreeRoot.getUserObject());
        // all the Signal trees have been deleted, it is necessary to build a new Signal root.
        this_JTreeRoot.setNewSignalRoot(this_C_interface, name);
        this_JTreeRootModel.reload();
	the_setNewNodeNumber = 1;
    }

    // class JTree2
    public boolean hasActiveSTNode() {
	return ( this_activeSTNode != null );
    }

    // class JTree2
    public SignalTreeNode get_selectedJNode(){
	TreePath selectedPath = this_JTree.getSelectionPath();
	if (  selectedPath == null )
	    return this_JTreeRoot;
	return (SignalTreeNode) (selectedPath.getLastPathComponent());
    }

    /** set the actual ntrees root as this selected node
	@return false if the ntrees is empty
     */
    // class JTree2
    public boolean setSelected_ntreesRoot(){
	SignalTreeNode ntreesRoot;
	if ( isEmpty_nTrees() )
	    return false;
	// guarantee: this_JTreeRoot is not null and has one or more children
	ntreesRoot = (SignalTreeNode)this_JTreeRoot.getChildAt(0);
	// guarantee: ntreesRoot is not null
	select_node(ntreesRoot);
	return true; 
    }
    
    /** save the selected node or model
	if complement = J_root save the root
	if complement = J_in, save a selected node
	if silent = true, browsing is proposed only if filename is missing
    */
    // PLG OK Nov 3
    // class JTree2
    private void load_JNode(int complement, boolean graphic) { // complement in (UserActions.) J_root, J_in, J_before
	if ( complement == UserActions.J_root ) 
	    this_Pwindow.load_root(graphic);
	else {
	    if ( isSelected_ntreesRoot() ) { 
		if ( complement == UserActions.J_before ) {
		    displayError("A single root is allowed");
		    this_Pwindow.cancel_action();
		    return; } }
	    this_Pwindow.load_node_in(graphic);
	}
	    
    }
    
     /** save the selected node or model
	if complement = J_root save the root
	if complement = J_in, save a selected node
	if silent = true, browsing is proposed only if filename is missing
    */
    // PLG OK Nov 3
    // class JTree2
    public void save_JNode(int complement, boolean silent) { // complement in UserActions.J_root, UserActions.J_in
	if ( isEmpty_nTrees() ) {
	    JOptionPane.showMessageDialog(this, "Empty window", "Save", JOptionPane.INFORMATION_MESSAGE);
	    this_Pwindow.cancel_action(); }
	else  // guarantee: this_JTreeRoot is not null and has one or more children
	    if ( complement == UserActions.J_root ) {
	    this_Pwindow.save_root(silent); }
	else if ( isSelected_ntreesRoot() )
	    this_Pwindow.save_root(silent);
	else
	    this_Pwindow.save_innerModel(); 
    }
    
   /** Destruction of the selected process. If the selected process is the root,
	the destruction	is not allowed. If ?withCopy is true, a copy of the process is done for
	the "paste".
    */
    // class JTree2
    public void cut_selectedJNode() {
	// System.out.println("DEBUG entering JTree2::cut_selectedJNode");
	if ( !isSelected_JTreeRoot() ) {
	    SignalTreeNode sourceNode = get_selectedJNode();
	    SignalTreeNode parent = (SignalTreeNode) (sourceNode.getParent());
	    move_SNode2ClipBoard(sourceNode);  // ASSUMES sourceNode not null OK
	    select_node(parent);
        } 
	// System.out.println("DEBUG  exiting JTree2::cut_selectedJNode");
    }

    /** Method copying a part of a tree to the clipBoard
	actual duplication of the STree is delayed */
    // class JTree2
    public void copy_selectedJNode() {
	if ( !isSelected_JTreeRoot() ) {
	    SignalTreeNode sourceNode = get_selectedJNode();
	    if ( sourceNode !=  this_JTreeRoot) {
		SignalTreeNode parent = (SignalTreeNode) (sourceNode.getParent());
		clone_SNode2ClipBoard(sourceNode); } } // ASSUMES sourceNode not null OK
    }

    /**
       if the_JTreeClipboardNode is null this JTree remains unchanged
       if the_JTreeClipboardNode is not null its content is copied to te selected node
       selected process
       return false 
    */
    // class JTree2
    private boolean paste_clipBoard(int complement) { // complement can be J_cut J_in, J_before
	SignalTreeNode currentNode;
	int index; 
	if ( the_JTreeClipboardNode == null ) {
	    JOptionPane.showMessageDialog(this, "Empty Clipboard", "Polychrony warning", JOptionPane.WARNING_MESSAGE);
	    return false; }
	if ( isSelected_JTreeRoot() ) {
	    if ( this_JTreeRoot.getChildCount() > 0 ) { // this_JTreeRoot has at most one child
		currentNode = (SignalTreeNode)(this_JTreeRoot.getChildAt(0)); 
		complement = UserActions.J_cut;
		exchange_clipBoardWithSNode(this_JTreeRoot, currentNode, 0); }
	    else {
		complement = UserActions.J_in; 
		// ASSUMES the_JTreeClipboardNode not NULL
		paste_clipBoardInSNode(this_JTreeRoot, -1); } }
	else {
	    currentNode = get_selectedJNode();
	    if ( complement == UserActions.J_in ) {
		if ( the_JTreeClipboardNode.isModul() ) {
		    displayError(Constants.J_ERR_insertModule);
		    return false; }
		// ASSUMES the_JTreeClipboardNode not NULL
		paste_clipBoardInSNode(currentNode, -1); }
	    else {
		SignalTreeNode targetParent;
		targetParent = (SignalTreeNode)(currentNode.getParent());
		if ( complement == UserActions.J_cut ) {
		    if ( the_JTreeClipboardNode.isModul() )
			if ( targetParent != this_JTreeRoot ) {
			    displayError(Constants.J_ERR_insertModule);
			    return false; }
		    index = targetParent.getIndex(currentNode); 
		    exchange_clipBoardWithSNode(targetParent, currentNode, index); } 
		else { // complement == UserActions.J_before
		    if ( targetParent == this_JTreeRoot ) {
			displayError("A single root is allowed");
			return false; }
		    index = targetParent.getIndex(currentNode); 
		    // ASSUMES the_JTreeClipboardNode not NULL
		    paste_clipBoardInSNode(targetParent, index); } } }
	return true;
    }
    
    /**
       Recursively pastes the sons of the temporary tree in the principal tree
       without copying the process. This method is useful for the creation of a
       new PolychronyWindow to create the new tree that will be pointing on the same
       processes than the tree from the calling PolychronyWindow.
       assume: tempNode, aNode are not null

    */
    // class JTree2
    private void pasteSpecial(SignalTreeNode tempNode, SignalTreeNode aNode) {
        for (int i = 0; i < tempNode.getChildCount(); i++) {
            // For each child, a node is created and inserted.
            SignalTreeNode childNode = new SignalTreeNode(this_C_interface, (((SignalTreeNode) tempNode.getChildAt(i)).get_ntreeValue())); // 3 not a module
	    // guarantee: childNode is not null
            childNode.setAllowsChildren(true);
            this_JTreeRootModel.insertNodeInto(childNode, aNode, aNode.getChildCount());
            this_JTree.expandPath(new TreePath(childNode.getPath()));
            select_pathOf(childNode);
            // apply to children.
            pasteSpecial((SignalTreeNode) tempNode.getChildAt(i), childNode);
        }
    }

    /** Gives the name of the selected node. */
    // class JTree2
    public String getSelectedName() {
	SignalTreeNode selectedNode = get_selectedJNode();
	long ntreeSource;
        if (selectedNode == this_JTreeRoot) 
            return "";
	ntreeSource = selectedNode.get_ntreeValue();
	return this_C_interface.getNameProc(ntreeSource);
    }

    /**
      recursive method to search a process whose name is processName returns
      false if the user has clicked on CANCEL or closed the search window, true
      if the search is finished
      assume: currentNode is not null
     */
    // class JTree2
    public boolean searchProc(SignalTreeNode currentNode, String processName) {
        boolean displayEndSearch = true;
        int stop = -1;
        Object[] options = { "Continue", "Cancel" };
	
        if ((processName == null) || processName.equals(""))
            return false;
	// guarantee: currentNode is not null
	if ( (this_C_interface.getNameProc(currentNode.get_ntreeValue())).equals(processName) ) { 
	    // a  process with the right name has been found
	    // set the process as the current process in the Signal world
	    select_pathOf(currentNode);
	    stop = JOptionPane.showOptionDialog(this,
						"\"" + processName + "\" found", "SEARCH...",
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE, null, options,
						options[0]);
	    
	    // stop = 1 means that options[1] (cancel) has been clicked
	    if ((stop == 1) || (stop == JOptionPane.CLOSED_OPTION))
		displayEndSearch = false; }
	if (displayEndSearch) {
	    int i = 0;
	    while (displayEndSearch && (i < currentNode.getChildCount())) {
		displayEndSearch = searchProc( (SignalTreeNode) currentNode.getChildAt(i),
					       processName);
		i += 1; } }
        return (displayEndSearch);
    }
    
    // assume: this_JTreeRoot is not null
    // called in LiaisonSignal <-- graphicalTrees::create_modeulInJTree
    // class JTree2
    public void initSelectedNode(boolean isTop) {
        if (isTop)
	    select_JTreeRoot();
	else
            this_selectedSTNode = get_selectedJNode();
    }
    
    // class JTree2
    // called in graphicalTrees::replaceInHierar, 
    public boolean deleteSelectedNode() {
        SignalTreeNode selectedNode = get_selectedJNode();
	// System.out.println("DEBUG entering JTree2::deleteSelectedNode");
	if ( selectedNode == this_JTreeRoot ) 
	    return false;
	this_selectedSTNode = (SignalTreeNode) (selectedNode.getParent());
	cut_selectedJNode();
	// this_Pwindow.refresh(); // PLG ??
	// System.out.println("DEBUG  exiting JTree2::deleteSelectedNode");
	return !isEmpty_nTrees();  // false guarantee: this_JTreeRoot is not null and has one or more children
    }
    
    // class JTree2
    public void upTempSelectedNode() {
        this_selectedSTNode = (SignalTreeNode) this_selectedSTNode.getParent();
    }

    // class JTree2
    private boolean insertObject(String nom, long SignalTree, int index) {
	// System.out.println("DEBUG entering JTree2::insertObject; nom is "+nom+", index is "+index);
	boolean toBeCut = false;
	SignalTreeNode currentNode = null;
	SignalTreeNode childNode = new SignalTreeNode(this_C_interface, SignalTree); // not a module checked
	if ( this_C_interface.isModul(SignalTree) ) {
	    if ( !isSelected_JTreeRoot() ) {
		displayError(Constants.J_ERR_insertModule);
		return false; } }
	if ( isSelected_JTreeRoot() ) {
	    if ( this_selectedSTNode.getChildCount() > 0 ) {
		toBeCut = true;
		currentNode = (SignalTreeNode)(this_JTreeRoot.getChildAt(0)); }
	    index = 0; }
        this_JTreeRootModel.insertNodeInto(childNode, this_selectedSTNode, index);
	select_node(childNode);
	this_JTree.expandPath(new TreePath(this_selectedSTNode.getPath()));
	if ( toBeCut ) {
	    move_SNode2ClipBoard(currentNode); } // ASSUMES curentNode not null
	// System.out.println("DEBUG exiting JTree2::insertObject");
	return true;
    }
    
    /** Used for nodes insertion (when loading a file for example) */
    // class JTree2
    public boolean insertJNodeLastIn(String nom, long SignalTree) {
	boolean res;
	// System.out.println("DEBUG entering JTree2::insertJNodeLastIn; nom is "+nom);
        res = insertObject(nom, SignalTree, this_selectedSTNode.getChildCount());
	// this_Pwindow.refresh(); // PLG ??
	// System.out.println("DEBUG  exiting JTree2::insertJNodeBefore; nom is "+nom);
        return res;
    }

    /** Used for nodes insertion (when loading a file for example) */
    // class JTree2
    public boolean insertJNodeBefore(String nom, long SignalTree) {
	// System.out.println("DEBUG entering JTree2::insertJNodeBefore; nom is "+nom);
	SignalTreeNode targetParent = (SignalTreeNode)(this_selectedSTNode.getParent());
	int index = targetParent.getIndex(this_selectedSTNode);
	boolean res;
	select_node(targetParent);
	res = insertObject(nom, SignalTree, index);
	// System.out.println("DEBUG  exiting JTree2::insertJNodeBefore; nom is "+nom);
        return res;
    }

    /** Creates a new son process at the level of the (father) selected process */
    // class JTree2
    private void new_JModule() {
        SignalTreeNode targetParent = null, currentNode = null, createdNode = null;
        String implicitName, actualName ;
	boolean cutCurrent;
	
	// System.out.println("DEBUG entering JTree2::new_JModule");
        targetParent = this_JTreeRoot;
	if ( targetParent.getChildCount() > 0 ) {
	    cutCurrent = true;
	    currentNode = (SignalTreeNode)(this_JTreeRoot.getChildAt(0)); }
	else
	    cutCurrent = false;
	implicitName = new String("Module") + the_setNewNodeNumber;
	actualName = JOptionPane.showInputDialog(this, "Module name?", implicitName); 
	if ( actualName == null )
	    actualName = implicitName; 
	long createdNtrees = create_ntreeModule(targetParent.get_ntreeValue(), actualName);
        createdNode = new SignalTreeNode(this_C_interface, actualName, createdNtrees, true); // a module
        this_JTreeRootModel.insertNodeInto(createdNode, targetParent, 0);
        select_node(createdNode);
        the_setNewNodeNumber++;
	if ( cutCurrent ) 
	    move_SNode2ClipBoard(currentNode);   // ASSUMES curentNode not null
	// System.out.println("DEBUG  exiting JTree2::new_JModule");
    }
    
    /** Creates a new son process at the level of the (father) selected process */
    // class JTree2
    private void new_JModel(int userAct, int complement) { // complement can be J_cut J_in, J_before
        SignalTreeNode targetParent = null, currentNode;
        String implicitName, actualName ;
	int index; 
	// System.out.println("DEBUG entering JTree2::new_JModel");
	if (userAct == UserActions.J_creerproc)
	    implicitName = new String("Process");
	else if (userAct == UserActions.J_creernode)
	    implicitName = new String("Node");
	else if (userAct == UserActions.J_creerfunc)
	    implicitName = new String("Function");
	else if (userAct == UserActions.J_creeraction)
	    implicitName = new String("Action"); 
	else
	    implicitName = new String("ERROR_OPTION"); 
	implicitName = implicitName  + the_setNewNodeNumber;
	actualName = JOptionPane.showInputDialog(this, "Model name?", implicitName); 
	if ( actualName == null )
	    actualName = implicitName; 
	currentNode = get_selectedJNode();

	if ( isSelected_JTreeRoot() ) {
	    targetParent = this_JTreeRoot;
	    if ( targetParent.getChildCount() > 0 ) {
		complement = UserActions.J_cut;
		currentNode = (SignalTreeNode)targetParent.getChildAt(0); }
	    else
		complement = UserActions.J_root;
	    index = 0; }
	else if ( complement == UserActions.J_cut ) {
	    targetParent = (SignalTreeNode)(currentNode.getParent());
	    index = targetParent.getIndex(currentNode); }
	else if ( complement == UserActions.J_in ) {
	    targetParent = currentNode;
	    index = currentNode.getChildCount(); }
	else { // UserActions.J_before
	    targetParent = (SignalTreeNode)(currentNode.getParent());
	    if ( targetParent == this_JTreeRoot ) {
		displayError("A single root is allowed");
		return; }
	    index = targetParent.getIndex(currentNode); }
	
        SignalTreeNode createdNode;
	long createdNtrees = create_ntreeChildAtI(targetParent.get_ntreeValue(), actualName, userAct, index);
	createdNode = new SignalTreeNode(this_C_interface, actualName, createdNtrees, false);
        this_JTreeRootModel.insertNodeInto(createdNode, targetParent, index);
        select_node(createdNode);
        the_setNewNodeNumber++;
	if (  complement == UserActions.J_cut )	
	    move_SNode2ClipBoard(currentNode);   // ASSUMES curentNode not null
	// System.out.println("DEBUG  exiting JTree2::new_JModel");
    }

    /** Creates a new son process at the level of the (father) selected process */
    // class JTree2
    public void gotoWindow() {
	PolychronyWindow destination = this_selectedSTNode.get_dedicatedWindow();
	if ( destination != null )
	    destination.selectPwindow();
    }

    /**
       Searchs of a process in the tree considering ntreeSource recursively
       /iterate the graphical objects of the tree. */
    // class JTree2
    private boolean searchNodePosition(SignalTreeNode noeud, long boxSource) {
        if (noeud == null) {
            nodeFound = this_JTreeRoot;
            return true;
        }
        if (noeud.get_ntreeValue() == boxSource) { // PLG NOT CONSISTANT: box vs ntrees ???
            nodeFound = noeud;
            return true;
        }
        for (int k = 0; k < noeud.getChildCount(); k++)
            if (searchNodePosition((SignalTreeNode) noeud.getChildAt(k),
                                   boxSource))
                return true;
        return false;
    }

    /** Method that sets the JTree view when opening a file or refreshing */
    // assume: this_JTreeRoot is not null
    // class JTree2
    public void refresh() {
	// System.out.println("DEBUG entering JTree2::display_selectedTreePath");
        TreePath currentSelection = this_JTree.getSelectionPath();
        if (currentSelection == null) {
            // aligns the scrollBar to the left when opening a file
            JViewport viewTreewin = this_scrollPane.getViewport();
            viewTreewin.setViewPosition(new Point(0, 0));
            this_scrollPane.setViewport(viewTreewin);
            select_JTreeRoot(); } 
	else // guarantee: currentSelection is not null
	    select_path(currentSelection); 
	// System.out.println("DEBUG  exiting JTree2::display_selectedTreePath");
    }

    // class JTree2
    public void setCurrentObject(int SignalTreeBox) {
        searchNodePosition((SignalTreeNode) this_JTreeRoot, SignalTreeBox);
        select_node(nodeFound);
	// this_Pwindow.refresh(); // PLG ??
    }

    // class JTree2
    public void actionPerformedTreeListener(int userAction, int complement) {
        String str = "";
        TreePath selectionPath = this_JTree.getSelectionPath();
        SignalTreeNode selectedNode = (SignalTreeNode) (selectionPath.getLastPathComponent());
	// System.out.println("DEBUG entering JTree2::JTree2 actionPerformedTreeListener");
	this_Pwindow.set_selectedAction(userAction, complement);
        if (userAction == UserActions.J_copyPaste2newWindow) {
	    mutateActiveToSelectedSTNode();
	    new PolychronyWindow(this_Pwindow);
        } else if (userAction == UserActions.J_setName) {
	    Iset_name(selectedNode);
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_gotoWindow) {
	    gotoWindow();
        } else if (userAction == UserActions.J_creermodule) {
            new_JModule();
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_creerproc) {
            new_JModel(UserActions.J_creerproc, complement);
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_creernode) {
            new_JModel(UserActions.J_creernode, complement);
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_creeraction) {
            new_JModel(UserActions.J_creeraction, complement);
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_creerfunc) {
            new_JModel(UserActions.J_creerfunc, complement);
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_load) {
	    load_JNode(complement, true); 
	} else if (userAction == UserActions.J_parse) {
	    load_JNode(complement, false);
        } else if (userAction == UserActions.J_save) {
            save_JNode(complement, true);
        } else if (userAction == UserActions.J_saveAs) {
            save_JNode(complement, false);
	} else if (userAction == UserActions.J_cut) {
	    cut_selectedJNode();
	    this_Pwindow.refresh(true);
        } else if (userAction == UserActions.J_copy) {
            copy_selectedJNode();
        } else if (userAction == UserActions.J_paste) {
	    if ( paste_clipBoard(complement) ) {
		this_Pwindow.refresh(true);
		this_Pwindow.refresh(); } // PLG ??
        } else if (userAction == UserActions.J_search) {
            str = (String) JOptionPane.showInputDialog(this,
                    "Process name ?",
                    "SEARCH PROCESS...", JOptionPane.QUESTION_MESSAGE, null,
                    null, searchString);
            if (str != null) { // str is null when the user clicks on "cancel" or
                // closes the window
                if (!str.equals(searchString))
                    searchString = str;
                if (searchProc(selectedNode, str))
                    JOptionPane.showMessageDialog(this, "End of search");
            }
        } else {
            this_C_interface.sig_iterate(); // PLG CHECK
        }
    }

    // class JTree2
    public SignalTreeNode getTreeRoot() {
        return this_JTreeRoot;
    }

    // class JTree2
    public boolean isCurrentPolychronyWindow() {
        return gwCourante;
    }

    // class JTree2
    public void setCurrentPolychronyWindow(boolean b) {
        gwCourante = b;
    }

    // class JTree2
    public SignalTreeNode getFoundNode() {
        return nodeFound;
    }

    // class JTree2
    public PolychronyWindow getPolychronyWindow() {
        return this_Pwindow;
    }

    // class JTree2
    public LiaisonSignal get_C_interface() {
        return this_C_interface;
    }

    // class JTree2
    public JTree getTree() {
        return this_JTree;
    }

    /** JTree2 initialization: define non null this_scrollPane, this_JTreeRoot, this_JTreeRootModel, this_JTree, JTree2 and inner menus
     */
    // class JTree2
    private void initialize(LiaisonSignal l, PolychronyWindow gw, String hyperRootName) {
        this_C_interface = l;
        this_Pwindow = gw;

        this_JTreeRoot = new SignalTreeNode(this_C_interface, hyperRootName);
        this_JTreeRoot.setAllowsChildren(true);
        this_JTreeRootModel = new DefaultTreeModel(this_JTreeRoot);
        this_JTreeRootModel.addTreeModelListener(new JTree2ModelListener(this));
        this_JTreeRootModel.setAsksAllowsChildren(true);

	this_activeSTNode = null;
        // creation of the tree data-model
        //  creation of the tree based on the tree data-model defined before
        this_JTree = new JTree(this_JTreeRootModel);
        this_JTree.setEditable(true);
        this_JTree.putClientProperty("JTree.lineStyle", "Angled"); // link style between the nodes
        this_JTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        this_JTree.setShowsRootHandles(true);
        this_JTree.setCellRenderer(new JTree2CellRenderer(this_C_interface, this_JTreeRoot, this_Pwindow));

        this_emptyPopup = make_noSelectionPopupMenu(this_JTree, false);
        this_emptySelectionPopup = make_noSelectionPopupMenu(this_JTree, true);
        this_commonModulePopup = make_modulePopupMenu(this_JTree);
        this_rootModelPopup = make_modelPopupMenu(this_JTree, true, false);
        this_windowModelPopup = make_modelPopupMenu(this_JTree, false, true);
        this_commonModelPopup = make_modelPopupMenu(this_JTree, false, false);
        // Colours, scroll bars ans size
        setBackground(get_color3());
        setLayout(new BorderLayout());
        this_scrollPane = new JScrollPane(this_JTree);
        this_scrollPane.setBackground(get_color3());
        this_scrollPane.setPreferredSize(new Dimension(300, 580));
        add(this_scrollPane);
	select_JTreeRoot();
	this_MOUSE_position = UserActions.MOUSE_up;
    }

    /** JTree2 contained in a root Rwindow
     */
    public JTree2(LiaisonSignal l, PolychronyWindow gw) {
	// System.out.println("DEBUG entering JTree2::JTree2 root tree creation");
        initialize(l, gw, "Working area");
	this_JTree.setDragEnabled(true);
//	this_JTree.setDropMode(DropMode.ON_OR_INSERT);
	this_JTree.setRootVisible(false);
	// System.out.println("DEBUG  exiting JTree2::JTree2 creation");
    }

    /** JTree2 contained in a child  Cwindow
     */
    public JTree2(JTree2 sourceTree, LiaisonSignal l, PolychronyWindow gw) {
	// System.out.println("DEBUG entering JTree2::JTree2 child tree creation");
        initialize(l, gw, "Child area");
	if ( sourceTree != null )
	    if ( !(sourceTree.isSelected_JTreeRoot()) ) {
		sourceTree.copy_selectedJNode();
		paste_clipBoard(UserActions.J_cut);  // ASSUMES the_JTreeClipboardNode not null
	    }
	this_JTree.setRootVisible(false);
	// System.out.println("DEBUG  exiting JTree2::JTree2 creation");
    }
    
}

