 /*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputAdapter;


/**
 * It creates a icon bar for a Signal library (set of processes). Assume : For a
 * library L in a directory D, the icons must be in the sub-directory D/L_ICO
 */
public class LibBar extends JToolBar {
    private static final long serialVersionUID = 1L;
    private long signalRef; // Tree of the library (produced by the Signal parser)
    private String fileName, fileNameWithoutSuffix, defaultIconDir;
    private JPopupMenu popup;

    private Color getMyColor1() {
        return pKGUIColors.Color1;
    }

    private void makePopup(LiaisonSignal C_interface) {
        JMenuItem menuItem;
        // Create the popup menu.
        popup = new JPopupMenu();
        menuItem = new JMenuItem("remove");
        menuItem.addActionListener(new MyListernerPopup(this, 1, C_interface));
        popup.add(menuItem);
        menuItem = new JMenuItem("reload");
        menuItem.addActionListener(new MyListernerPopup(this, 2, C_interface));
        popup.add(menuItem);
    }

    private void removeAll(GraphCanvas vGraphCanvas) {
        int n = getComponentCount();
        for (int i = 0; i < n; i++)
            vGraphCanvas.removeLibBut((LibButton) getComponent(i));
        removeAll();
    }

    public void install(long ntreesArg, GraphCanvas vGraphCanvas, LiaisonSignal C_interface) {
        boolean stop = false;

        removeAll(vGraphCanvas);
        signalRef = ntreesArg;
        int i = 1;
        setBackground(getMyColor1());
        while (!stop) {
            String name = C_interface.getNameGif(ntreesArg, i); // getting the name of the i-th
            // process of the library.
            if (name.equals(""))
                stop = true;
            else {
                LibButton but = new LibButton(name, i, C_interface, this,
                                              fileNameWithoutSuffix, defaultIconDir);
                add((JButton) but);
                vGraphCanvas.addLibBut(but);
                i++;
            }
        }
    }

    public long getSignalRefTree() {
        return signalRef;
    }

    public String getFilename() {
        return fileName;
    }

    public JPopupMenu getpopup() {
        return popup;
    }

    /** 
     */
    public LibBar(long ntreesArg, GraphCanvas vGraphCanvas, LiaisonSignal C_interface, String fname, String fnamews,
                  String imagesDirectory, String LibName) {
        super(LibName);
        fileName = fname;
        fileNameWithoutSuffix = fnamews;
        defaultIconDir = imagesDirectory;
	install(ntreesArg, vGraphCanvas, C_interface);
        makePopup(C_interface);
        ActLibBar ActLibBarListener = new ActLibBar(this, C_interface);
        addMouseListener(ActLibBarListener);
        addMouseMotionListener(ActLibBarListener);
    }

}

class ActLibBar extends MouseInputAdapter {
    LiaisonSignal this_C_interface;
    LibBar barre;

    public ActLibBar(LibBar bar, LiaisonSignal ls) {
        this_C_interface = ls;
        barre = bar;
    }

    public void mousePressed(MouseEvent e) {
        if (SwingUtilities.isRightMouseButton(e))
            if (e.isPopupTrigger())
                barre.getpopup().show(e.getComponent(), e.getX(), e.getY());
    }
}

class MyListernerPopup implements ActionListener {
    LibBar vbar;
    int codeOp;
    LiaisonSignal this_C_interface;

    MyListernerPopup(LibBar b, int code, LiaisonSignal ls) {
        vbar = b;
        codeOp = code;
        this_C_interface = ls;
    }

    public void actionPerformed(ActionEvent e) {
        this_C_interface.ListernerPopupLibBar(vbar, codeOp);
    }
}
