 /*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;


import javax.swing.RepaintManager;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

public class PrintUtilities implements Printable {

    private int format;
    private Component componentToBePrinted;

    public PrintUtilities(GraphCanvas componentToBePrinted, int form) {
        this.componentToBePrinted = componentToBePrinted;
        format = form;
    }

    public void print() {
        PrinterJob printJob = PrinterJob.getPrinterJob();
        PageFormat pf = printJob.defaultPage();
        pf.setOrientation(format);
        printJob.setPrintable(this, pf);
        if (printJob.printDialog())
            try {
                printJob.print();
            } catch (PrinterException pe) {
                System.out.println("Error printing: " + pe);
            }
    }

    public int print(Graphics g, PageFormat pageFormat, int pageIndex) {
        if (pageIndex > 0) {
            return (NO_SUCH_PAGE);
        } else {
            /*
             * ((GraphCanvas)componentToBePrinted).gc2.setFont(new Font("Helvetica",Font.PLAIN,24));
             * ((GraphCanvas)componentToBePrinted).gc2.drawString(haut,400,50);
             * ((GraphCanvas)componentToBePrinted).gc2.setFont(new Font("Helvetica",Font.PLAIN,12));
             * ((GraphCanvas)componentToBePrinted).gc2.drawString(bas,600,300);
             */
            Graphics2D g2d = (Graphics2D) g;
            /*
             * g2d.gc2.setFont(new Font("Helvetica",Font.PLAIN,24));
             * g2d.gc2.drawString(haut,400,50); g2d.gc2.setFont(new Font("Helvetica",Font.PLAIN,12));
             * g2d.gc2.drawString(bas,600,300);
             */

            if (format == PageFormat.LANDSCAPE) {
                g2d.scale(0.7, 0.7);
                g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
            } else if (format == PageFormat.PORTRAIT) {
                g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
                g2d.scale(0.5, 0.5);
            }
            disableDoubleBuffering(componentToBePrinted);
            componentToBePrinted.paint(g2d);
            enableDoubleBuffering(componentToBePrinted);
            return (PAGE_EXISTS);
        }
    }

    /**
     * The speed and quality of printing suffers dramatically if any of the
     * containers have double buffering turned on. So this turns if off
     * globally.
     *
     * @see enableDoubleBuffering
     */
    public static void disableDoubleBuffering(Component c) {
        RepaintManager currentManager = RepaintManager.currentManager(c);
        currentManager.setDoubleBufferingEnabled(false);
    }

    /** Re-enables double buffering globally. */
    public static void enableDoubleBuffering(Component c) {
        RepaintManager currentManager = RepaintManager.currentManager(c);
        currentManager.setDoubleBufferingEnabled(true);
    }

}
