/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */


package common;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;


import common.Constants;
import common.GraphCanvas;
import common.InfoWindow;
import common.JTree2;
import common.LiaisonSignal;
import common.ParamWindow;
import common.TextWindow;
import common.UserActions;
import common.LibBar;
import common.CompilerWin;
import common.ChooseFont;
import common.PrintWindow;
import common.PolychronyWindow;
import common.pKGUIColors;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;

// import common.PolychronyWindow;


public class TextWindow extends JFrame {
    private static final long serialVersionUID = 1L;
    private int this_textCategory; // among C_CanvasText, C_HierarText, C_ParamText, C_
    private int carWidth; // width of the caracter "N" for the current font
    private PolychronyWindow this_parentPwindow;
    private Color vcolor = pKGUIColors.Color1;
    private JTextArea this_textArea = new JTextArea();
    private Font currentFont;
    private static String textCopy = "";
    private String searchString = ""; // contains the current string to search

    public TextWindow(PolychronyWindow gwin, int textType, int x0, int y0,
                      int width, int height, String title) {
	this_parentPwindow = gwin;
        this_textCategory = textType;
        setLocation(x0, y0);
        setSize(width, height);
        setTitle(title);
        currentFont = this_textArea.getFont();
        FontMetrics fmetrics = getFontMetrics(currentFont);
        carWidth = fmetrics.charWidth('N');// used to calculate the number of
        // columns of the window
        JPanel panel = new JPanel();
        getContentPane().add(panel);
        panel.setLayout(new BorderLayout());
        panel.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
        // Color of the window's background
        panel.setBackground(vcolor);
        //  Panel that will contain the buttons
        JPanel panelBouton = new JPanel();
        panelBouton.setLayout(new FlowLayout(FlowLayout.LEFT));
        panelBouton.setBackground(vcolor);
        // Creation of an Edit menu with Copy, Paste and Cut
        JMenuBar menuBar = new JMenuBar();
        JMenu menu;
        JMenuItem menuItem;
        menu = new JMenu("Edit");
        menu.getAccessibleContext().setAccessibleDescription("Edition Tools");
        // a group of JMenuItems
        menuItem = new JMenuItem("Copy");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y,
                                ActionEvent.CTRL_MASK));
        menuItem.getAccessibleContext().setAccessibleDescription(
            "Copy selected text");
        ActionListener copyAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // Gets the selected text
                textCopy = this_textArea.getSelectedText();
            }
        };
        menuItem.addActionListener(copyAct);
        menu.add(menuItem);
        menuItem = new JMenuItem("Paste");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                                ActionEvent.CTRL_MASK));
        menuItem.getAccessibleContext().setAccessibleDescription(
            "Paste selected text");
        ActionListener pasteAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // replacing the selection with textCopy
                this_textArea.replaceRange(textCopy, this_textArea.getSelectionStart(), this_textArea.getSelectionEnd());
            }
        };
        menuItem.addActionListener(pasteAct);
        menu.add(menuItem);
        menuItem = new JMenuItem("Cut");
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        menuItem.getAccessibleContext().setAccessibleDescription( "Cut selected text");
        ActionListener cutAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // Gets the selected text
                textCopy = this_textArea.getSelectedText();
                // Replaces the selection with nothing
                this_textArea.replaceRange("", this_textArea.getSelectionStart(), this_textArea.getSelectionEnd());
            }
        };
        menuItem.addActionListener(cutAct);
        menu.add(menuItem);
        menuBar.add(menu);
        panelBouton.add(menuBar);
        // Creates a buttons bar with the commands Confirm Search Replace
        JButton search = new JButton("Find");
        panelBouton.add(search);
        ActionListener searchAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                searchText();
            }
        };
        search.addActionListener(searchAct);
        JButton replace = new JButton("Replace");
        panelBouton.add(replace);
        ActionListener replaceAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                replaceText();
            }
        };
        replace.addActionListener(replaceAct);
        JLabel bourrage = new JLabel(" ");
        bourrage.setBackground(vcolor);
        panelBouton.add(bourrage);
        JButton confirm = new JButton("Ok");
        panelBouton.add(confirm);
        ActionListener confirmAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		// System.out.println("DEBUG      in  TextWindow::confirmAct 1");
                this_parentPwindow.sig_iterate(UserActions.J_confirm);
		// System.out.println("DEBUG      in  TextWindow::confirmAct 3");
            }
        };
        confirm.addActionListener(confirmAct);
        JButton cancel = new JButton("Cancel");
        panelBouton.add(cancel);
        ActionListener cancelAct = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        };
        cancel.addActionListener(cancelAct);
        /* WindowListener closeWin =*/ new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                setVisible(false);
            }
        };
        panel.add(panelBouton, BorderLayout.NORTH);
        // background color on the text zone
        this_textArea.setBackground(Color.white);
        // In order not to cut the words at the end of a line in the text zone
        this_textArea.setWrapStyleWord(true);
        // to return to a new line automatically
        this_textArea.setLineWrap(false);
        // a scroll is added for the text zone
        JScrollPane scroll = new JScrollPane(this_textArea);
        panel.add(scroll, BorderLayout.CENTER);
    }

    public int get_textCategory() {
	return this_textCategory;
    }
    public void append(String s) {
	// System.out.println("DEBUG entering/exiting TextWindow::append category "+this_textCategory);
        this_textArea.append(s);
    }

    public void resetText() {
	// System.out.println("DEBUG entering/exiting TextWindow::resetText category "+this_textCategory);
        this_textArea.setText("");
    }

    public String getText() {
	// System.out.println("DEBUG entering/exiting TextWindow::getText category "+this_textCategory);
	return (this_textArea.getText());
    }

    public void setCaretPosition(int n) {
	// System.out.println("DEBUG entering/exiting TextWindow::setCaretPosition ");
        this_textArea.setCaretPosition(n);
    }

    // returns the number of columns of the text zone
    public int getColumns() {
	// System.out.println("DEBUG entering/exiting TextWindow::getColumns ");
        int col = (int) (getSize().getWidth() / carWidth);
        return (col);
    }

    public void searchText() {
        String stringJTextArea = getText();
        int indice = 0;
        boolean fin = false;
        int stop;
        Object[] options = { "Continue", "Cancel" };
        boolean found = false;
        String s = "";
        String message = "What do you search ?";
        s = JOptionPane.showInputDialog(this, message, "SEARCH...",
                                        JOptionPane.QUESTION_MESSAGE);
        if (s.equals("")) {
            fin = true;
            found = true;
        }
        while (indice < stringJTextArea.length() && !fin) {
            if (stringJTextArea.regionMatches(indice, s, 0, s.length())) {
                this_textArea.setCaretPosition(indice);
                this_textArea.setSelectionStart(indice);
                this_textArea.setSelectionEnd(indice + s.length());
                stop = JOptionPane
                       .showOptionDialog(this, "\"" + s + "\" found",
                                         "SEARCH...", JOptionPane.YES_NO_OPTION,
                                         JOptionPane.QUESTION_MESSAGE, null, options,
                                         options[0]);
                this_textArea.setCaretPosition(indice);
                if (stop == 1) {
                    fin = true;
                    found = true;
                }
            }
            if (indice == stringJTextArea.length() - 1) {
                fin = true;
            }
            indice++;
        }
        if (!found && fin) {
            JOptionPane.showMessageDialog(this, "No more \"" + s + "\" found");
        }
    }

    public String getString1() {
        String mess = "What do you want to replace ?";
        searchString = JOptionPane.showInputDialog(this, mess, "REPLACE...",
                       JOptionPane.QUESTION_MESSAGE);
        return (searchString);
    }

    public String getString2() {
        String mess = "Replace with ?";
        return (JOptionPane.showInputDialog(this, mess, "REPLACE...",
                                            JOptionPane.QUESTION_MESSAGE));
    }

    public int getUserChoice() {
        Object[] options = { "Replace", "Skip", "Cancel" };
        int stop;

        stop = JOptionPane.showOptionDialog(this, "\"" + searchString
                                            + "\" found", "REPLACE...", JOptionPane.YES_NO_CANCEL_OPTION,
                                            JOptionPane.QUESTION_MESSAGE, null, options, options[0]);
        if (stop == JOptionPane.OK_OPTION)
            return (1);
        if ((stop == JOptionPane.CANCEL_OPTION)
                || (stop == JOptionPane.CLOSED_OPTION))
            return (3);
        else
            return (2);
    }

    public void replaceText() {
        int indice = 0;
        int stop;
        boolean fin = false, found = false;
        String s1 = new String();
        String s2 = new String();
        String stringJTextArea = getText();
        s1 = getString1();
        if (s1.equals("")) {
            fin = true;
            found = true; }
        else {
            s2 = getString2();
            if (s2.equals("")) {
                fin = true;
                found = true; }
	    else {
                while (indice < stringJTextArea.length() && !fin) {
                    if (stringJTextArea.regionMatches(indice, s1, 0,
                                                      s1.length())) {
                        this_textArea.setCaretPosition(indice);
                        this_textArea.setSelectionStart(indice);
                        this_textArea.setSelectionEnd(indice + s1.length());
                        stop = getUserChoice();
                        this_textArea.setCaretPosition(indice);
                        if (stop == 3) {
                            fin = true;
                            found = true;
                        }
                        if (stop == 1) {
                            this_textArea.replaceRange(s2, indice, indice + s1.length());
                            indice += s2.length() - s1.length();
                            stringJTextArea = this_textArea.getText();
                        }
                    }
                    if (indice == stringJTextArea.length() - 1)
                        fin = true;
                    indice++;
                }
            }
            if (!found && fin)
                JOptionPane.showMessageDialog(this, "No more \"" + s1 + "\" found");
        }
    }
}
