/*
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr

   This file is part of Polychrony Signal GUI (http://www.irisa.fr/espresso/polychrony).

   Polychrony Signal GUI is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal GUI is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal GUI; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
 */
/*
  This module manages 2 tables which allows to accede to the graphical trees
  from the graphical objects numbers. These tables accelerate the access 
  of the objects designated by the user.
  
  The table "tab_acces" contains the numbers of boxes, ports, links and paths.
  The table "t_acces_seg" contains the numbers of the segments.
  These tables contain also the object halos which are the zone in wich the user
  can click to designate the object (the halos are rectangles).
  For ports, paths and links, the halos are empty.

  The object numbers must unique; these numbers are managed by three hash tables
  which are used when there is a tree copy or when a new file is loaded.
 */

#define  taille_tab 7 /* size of the access tables */
#define taille_t_hash 64 /* size of the hash tables */

typedef struct hash
{
    int ovovovov, ovovnvnv;
    struct hash *suivant;
}t_hash_num; 



class graphicTablesManager{

#include "graphicTablesManager_m.h"
#include "graphicTablesManager_g.h"

  private :
 
  acces_gra *tab_acces[taille_tab]; /* access table for list of boxes, ports, paths and links */ 
  acces_seg *t_acces_seg[taille_tab];/* access table for list of segments */ 
  
  t_hash_num *this_boxHashTable[taille_t_hash]; /* hash table for boxes */
  t_hash_num *this_pathHashTable[taille_t_hash]; /* hash table for paths */
  t_hash_num *this_segmentHashTable[taille_t_hash]; /* hash table for segments */
					      
  graphObj_type the_freeGraphObjIdentity; /* last allocated graph object number */  
  // int design; /* last allocated object number */  
  typcoord halo[2];
};
